package jdbcacsess;

import java.io.File;
import java.io.IOException;
import java.lang.Thread.UncaughtExceptionHandler;

import javax.swing.SwingUtilities;

import jdbcacsess.gui.JFrameMain;
import jdbcacsess.gui.JFrameSetting;
import jdbcacsess.gui.common.GUIManager;
import jdbcacsess.gui.common.JDialogMessage;

import org.w3c.dom.Element;

public class JdbcAcsess {

	final public static String major = "1";

	final public static String minor = "8";

	final public static String date = "2010/05/29";

	public static void main(String[] args) {
		try {

			File path = new File(SettingFile.getUserPath());
			if (!path.isDirectory()) {
				if(path.mkdir()){
					throw new IOException("Couldnt create diretrory.:"+path.getAbsolutePath());
				}
				JDialogMessage.infoDialog("設定ファイル保存ディレクトリ " + path.toString() + " を作成しました", "初回起動メッセージ");
			}
			LogOutput.setLogger(path);

			new SettingFile();
			LogOutput.setLevel(JFrameSetting.getLoggerLevel());
			LogOutput.info("ログレベルを" + JFrameSetting.getLoggerLevel() + "に設定しました");

			checkSettingFile();
			
		} catch (Exception err) {
			JDialogMessage.errorDialog(err,"起動失敗");
			return;
		}

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				Thread.setDefaultUncaughtExceptionHandler(new UncaughtExceptionHandler() {
					
					public void uncaughtException(Thread t, Throwable e) {
						JDialogMessage.errorDialog(e);
					}
				});
				
				try {
					GUIManager.setUIManager();
					new JFrameMain();
				} catch (Exception e) {
					JDialogMessage.errorDialog(e);
				}
			}
		});

		envDisplay();
	}


	private static void checkSettingFile() {
		// 設定ファイルのバージョンチェック
		SettingFile sf = SettingFile.getInstance();
		Element element = sf.getRootElement("version");
		String major = "1";
		String minor = "0";
		if (element.getAttributes().getLength() != 0) {
			major = element.getAttribute("major");
			minor = element.getAttribute("minor");
		}
		if (major.equals("1") && minor.equals("0")) {
			LogOutput.info("設定ファイルの<component>を<component2>へ変換します。");
			new ConvertToCommponent2();
		}

		if (Double.valueOf(major + "." + minor) < Double.valueOf(JdbcAcsess.major + "." + JdbcAcsess.minor)) {
			LogOutput.info("新バージョンです");
			element.setAttribute("major", JdbcAcsess.major);
			element.setAttribute("minor", JdbcAcsess.minor);
		}
		LogOutput.info("バージョンチェック終了");
	}

	private static void envDisplay() {
		// Logger.global.info("file.encoding:"
		// + System.getProperty("file.encoding"));
		// StringBuilder sb = new StringBuilder("defaultCharset:"
		// + Charset.defaultCharset());
		//
		// Logger.global.info(sb.toString());
		//
		// GregorianCalendar gc = new GregorianCalendar();
		// Logger.global.info("カレンダー:" + gc.toString());
		// Logger.global.info("タイムゾーン:" + gc.getTimeZone().toString());
	}

}
