package jdbcacsess;

public class PrefixStringBuffer {

	String first;

	String other;

	StringBuffer buf;

	boolean flg;

	String lineSeparator = System.getProperty("line.separator");

	public PrefixStringBuffer(String first, String other) {
		this.first = first;
		this.other = other;
		clear();
	}

	/**
	 * 項目を追加します。１回目の呼出であればfirst、２回目以降の呼出であればotherを付加してから項目は追加されます。
	 * 
	 * @param item
	 */
	public PrefixStringBuffer appendItem(String item) {
		if (flg) {
			buf.append(first);
			flg = !flg;
		} else {
			buf.append(other);
		}
		buf.append(item);
		return this;
	}

	/**
	 * 文字列を単純に追加します。
	 * 
	 * @param str
	 */
	public PrefixStringBuffer append(String str) {
		buf.append(str);
		return this;
	}

	/**
	 * 文字列を挿入します。
	 * 
	 * @param offset
	 * @param str
	 */
	public PrefixStringBuffer insert(int offset, String str) {
		buf.insert(offset, str);
		return this;
	}

	/**
	 * 編集状態をクリアします。
	 */
	public void clear() {
		flg = true;
		buf = new StringBuffer();
	}

	/**
	 * 編集中の文字列を取得します。<br>
	 * toString()は使用しないでください、デバック用に加工された形式なので、通常の使用には向きません。
	 * 
	 * @return
	 */
	public String getString() {
		return buf.toString();
	}

	/**
	 * 編集中のstringbufferを取得します。
	 * 
	 * @return
	 */
	public StringBuffer getStringBuffer() {
		return buf;
	}

	/**
	 * デバック用の情報表示
	 */
	@Override
	public String toString() {
		return "PrefixStringBuffer=[" + "first=" + first + "other=" + other
				+ "buf=" + buf.toString() + "]";
	}

	/**
	 * 実行環境に応じた改行を追加します。
	 */
	public PrefixStringBuffer appendLineSeparator() {
		buf.append(lineSeparator);
		return this;
	}

	private boolean isLastLF() {
		if (buf.length() == 0) {
			return false;
		}
		if (buf.substring(buf.length() - 1, buf.length()).equals("\n")) {
			return true;
		}
		return false;
	}

	/**
	 * 編集中文字列の長さを取得します。
	 * 
	 * @return
	 */
	public int length() {
		return buf.length();
	}

	/**
	 * 文字列終端がLFである事を保証して、編集中の文字列を取得します
	 * 
	 * @return
	 */
	public String getStringLastLF() {
		return getString() + (isLastLF() ? "" : "\n");
	}
}
