/**
 * 
 */
package jdbcacsess.createdata;

import java.util.ArrayList;

import javax.swing.table.AbstractTableModel;

/**
 * 固定値リストデータを生成する方法を管理する
 * 
 */
public class FixListTableModel extends AbstractTableModel {
	private static final long serialVersionUID = 6917533489069613264L;

	ArrayList<String> fixList;

	/**
	 * コンストラクタ
	 */
	public FixListTableModel() {
		super();
		fixList = new ArrayList<String>();
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	public int getColumnCount() {
		return 2;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	public int getRowCount() {
		return fixList.size() + 1;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getColumnName(int)
	 */
	@Override
	public String getColumnName(int columnIndex) {
		return (columnIndex == 0) ? "順番" : "値";
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#isCellEditable(int, int)
	 */
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return (columnIndex == 0) ? false : true;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		if (fixList.size() > rowIndex) {
			if (columnIndex == 0) {
				return rowIndex + 1;
			} else {
				return fixList.get(rowIndex);
			}
		}
		return (columnIndex == 0) ? "*" : null;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#setValueAt(java.lang.Object, int, int)
	 */
	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		if (columnIndex != 1) {
			return;
		}

		if (fixList.size() > rowIndex) {
			fixList.set(rowIndex, (String) aValue);
			return;
		}
		fixList.add((String) aValue);
	}

	/**
	 * 行を削除します
	 * 
	 * @param rowIndex
	 *            行インデックス
	 */
	public void delete(int rowIndex) {
		fixList.remove(rowIndex);
		fireTableRowsDeleted(rowIndex, rowIndex);
	}

	/**
	 * 位置を指定して追加します
	 * 
	 * @param rowIndex
	 *            行インデックス
	 * @param s
	 *            追加したい文字列
	 */
	public void insert(int rowIndex, String s) {
		fixList.add(rowIndex, s);
		fireTableRowsInserted(rowIndex, rowIndex);
	}

	/**
	 * 空行を指定位置に追加します
	 * 
	 * @param rowIndex
	 */
	public void insert(int rowIndex) {
		insert(rowIndex, "");
	}

	/**
	 * モデルのデータをリストで返却します
	 * 
	 * @return fixList を戻します。
	 */
	public ArrayList<String> getFixList() {
		return fixList;
	}

	/**
	 * モデルのデータを設定します
	 * 
	 * @param fixList
	 *            設定する fixList。
	 */
	public void setFixList(ArrayList<String> fixList) {
		this.fixList = fixList;
	}

}