/**
 * 
 */
package jdbcacsess.createdata;

import java.io.Serializable;

import jdbcacsess.createdata.chararctor.CharType;
import jdbcacsess.createdata.chararctor.FixType;
import jdbcacsess.createdata.chararctor.JComboboxCharType;
import jdbcacsess.createdata.chararctor.NumberHalfBitType;
import jdbcacsess.createdata.chararctor.NumberHalfHexType;
import jdbcacsess.createdata.chararctor.NumberHalfType;
import jdbcacsess.createdata.chararctor.UpperAlphaHalfType;
import jdbcacsess.sql.ColumnInfoTable;

public class SequenceParm implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1149012559461319437L;

	private int keta;

	private CharType charType;

	private String initialValue;

	private double stepValue;

	private double counter = 0;

	@Override
	public String toString() {
		return "[SequenceParm:" + charType + "," + initialValue + "," + keta
				+ "]";
	}

	/**
	 * カウンターを初期化します
	 */
	public void setCounterInitialValue() {
		counter = charType.getIndexOfString(getInitialValue());
	}

	/**
	 * 次の文字列を取得する。
	 * 
	 * @return 次の文字列
	 */
	public String getNext() {
		// カウンターの値を基に文字列を取得
		String str = charType.getStringAtIndex((long) counter, getKeta());
		if ("".equals(str)) {
			// 固定値の時は空文字列なので、初期値を常に返却する
			return getInitialValue();
		}

		// 次回の為に加算する
		counter += stepValue;

		// 最大値を超えたら最初に戻す
		if (charType.getMaxIndex(getKeta()) < counter) {
			counter = 0;
		}
		return str;
	}

	/**
	 * コンストラクタ
	 */
	public SequenceParm() {
		charType = JComboboxCharType.getCharType(UpperAlphaHalfType.class);
		initialValue = "A";
		keta = 1;
		stepValue = 1.0D;
	}

	/**
	 * @param columnInfoTable
	 */
	public SequenceParm(ColumnInfoTable columnInfoTable) {
		switch (columnInfoTable.getSqlType().getCategoly()) {
		case NUM:
			charType = JComboboxCharType.getCharType(NumberHalfType.class);
			keta = 1;
			break;
		case BIN:
			charType = JComboboxCharType.getCharType(NumberHalfHexType.class);
			keta = 2;
			break;
		case BIT:
			charType = JComboboxCharType.getCharType(NumberHalfBitType.class);
			keta = 1;
			break;
		case CHAR:
			charType = JComboboxCharType.getCharType(UpperAlphaHalfType.class);
			keta = 1;
			break;
		default:
			charType = JComboboxCharType.getCharType(FixType.class);
			keta = 1;
			break;
		}
		initialValue = "";
		if (0 < columnInfoTable.getColumnSize()
				&& columnInfoTable.getColumnSize() < 50) {
			keta = columnInfoTable.getColumnSize();
		}
		initialValue = charType.getStringAtIndex(columnInfoTable
				.getColumnIndex() - 1, keta);
		if (initialValue.length() > keta) {
			initialValue = initialValue.substring(initialValue.length() - keta);
		}
		stepValue = 1.0D;
	}

	/**
	 * @return charType を戻します。
	 */
	public CharType getCharType() {
		return charType;
	}

	/**
	 * @param charType
	 *            設定する charType。
	 */
	public void setCharType(CharType sequenceParmStrings) {
		this.charType = sequenceParmStrings;
	}

	/**
	 * @return initialValue を戻します。
	 */
	public String getInitialValue() {
		return initialValue;
	}

	/**
	 * @param initialValue
	 *            設定する initialValue。
	 */
	public void setInitialValue(String initialValue) {
		this.initialValue = initialValue;
	}

	/**
	 * @return keta を戻します。
	 */
	public int getKeta() {
		return keta;
	}

	/**
	 * @param keta
	 *            設定する keta。
	 */
	public void setKeta(int keta) {
		this.keta = keta;
	}

	/**
	 * @return stepValue を戻します。
	 */
	public double getStepValue() {
		return stepValue;
	}

	/**
	 * @param stepValue
	 *            設定する stepValue。
	 */
	public void setStepValue(double stepValue) {
		this.stepValue = stepValue;
	}

	/**
	 * @param charType
	 */
	public void changeCharType(CharType charType) {
		long indexOld = this.charType.getIndexOfString(this.initialValue);
		this.charType = charType;
		changeInitialValue(this.charType.getStringAtIndex(indexOld, this.keta));
		// System.out.println(" change()" + toString());
	}

	/**
	 * @param keta
	 */
	public void changeKeta(Integer keta) {
		if (keta < 0) {
			this.keta = 1;
		} else {
			this.keta = keta;
		}
		this.initialValue = this.charType
				.fullKeta(this.keta, this.initialValue);
		// System.out.println(" change()" + toString());
	}

	/**
	 * @param initialValue
	 */
	public void changeInitialValue(String initialValue) {
		this.initialValue = initialValue;
		if (this.keta < initialValue.length()) {
			this.keta = initialValue.length();
		}
		this.initialValue = this.charType
				.fullKeta(this.keta, this.initialValue);
		// System.out.println(" change()" + toString());
	}

}