/**
 * 
 */
package jdbcacsess.createdata;

import java.sql.SQLException;
import java.util.ArrayList;

import javax.swing.JComboBox;
import javax.swing.table.AbstractTableModel;

import jdbcacsess.createdata.chararctor.CharType;
import jdbcacsess.sql.SchemaTableColumnName;
import jdbcacsess.sql.SqlExec;

public class SequenceParmTableModel extends AbstractTableModel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -4617591687037164376L;

	private ArrayList<SequenceParm> sequenceParmList;

	private final String[] columnHeader = { "No.", "    文字種    ", "桁数", "初期値",
			"増分値" };

	private final Class<?>[] columnClass = { Integer.class, JComboBox.class,
			Integer.class, String.class, Double.class };

	/*
	 * (非 Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		StringBuffer str = new StringBuffer("#");
		for (SequenceParm iterable_element : sequenceParmList) {
			str.append( "[" ).append( iterable_element.toString() ).append( "]" );
		}
		str.append( "#");
		return str.toString();
	}

	/**
	 * コンストラクタ
	 */
	public SequenceParmTableModel() {
		super();
		sequenceParmList = new ArrayList<SequenceParm>();
	}

	/**
	 * スキーマテーブル名を指定したコンストラクタ
	 * 
	 * @param stcn
	 *            スキーマテーブル名
	 */
	public SequenceParmTableModel(SchemaTableColumnName stcn) {
		super();
		sequenceParmList = new ArrayList<SequenceParm>();
		try {
			sequenceParmList.add(new SequenceParm(SqlExec
					.getColumnInfoTable(stcn)));
		} catch (SQLException e) {
			sequenceParmList = new ArrayList<SequenceParm>();
			e.printStackTrace();
		}
	}

	/**
	 * @return sequenceParmList を戻します。
	 */
	public ArrayList<SequenceParm> getSequenceParmList() {
		return sequenceParmList;
	}

	/**
	 * @param sequenceParmList
	 *            設定する sequenceParmList。
	 */
	public void setSequenceParmList(ArrayList<SequenceParm> sequenceParmList) {
		this.sequenceParmList = sequenceParmList;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	public int getColumnCount() {
		return columnHeader.length;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getColumnName(int)
	 */
	@Override
	public String getColumnName(int columnIndex) {
		return columnHeader[columnIndex];
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getColumnClass(int)
	 */
	@Override
	@SuppressWarnings("unchecked")
	public Class getColumnClass(int columnIndex) {
		return columnClass[columnIndex];
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	public int getRowCount() {
		return sequenceParmList.size() + 1;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#isCellEditable(int, int)
	 */
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return (columnIndex == 0) ? false : true;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		if (columnIndex == 0) {
			if (sequenceParmList.size() > rowIndex) {
				return rowIndex + 1;
			} else {
				return "*";
			}
		}

		SequenceParm sequeenceParm = null;
		if (sequenceParmList.size() > rowIndex) {
			sequeenceParm = sequenceParmList.get(rowIndex);
		} else {
			return null;
		}
		switch (columnIndex) {
		case 0:
			return rowIndex + 1;
		case 1:
			return sequeenceParm.getCharType();
		case 2:
			return sequeenceParm.getKeta();
		case 3:
			return sequeenceParm.getInitialValue();
		case 4:
			return sequeenceParm.getStepValue();
		default:
			break;
		}
		return null;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#setValueAt(java.lang.Object, int, int)
	 */
	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		// System.out.println(hashCode()+"setValueAt
		// 開始["+rowIndex+","+columnIndex+"]");
		if (aValue == null) {
			return;
		}
		if (sequenceParmList.size() <= rowIndex) {
			sequenceParmList.add(new SequenceParm());
		}
		SequenceParm sequeenceParm = sequenceParmList.get(rowIndex);
		switch (columnIndex) {
		case 1:
			sequeenceParm.changeCharType((CharType) aValue);
			break;
		case 2:
			sequeenceParm.changeKeta((Integer) aValue);
			break;
		case 3:
			sequeenceParm.changeInitialValue((String) aValue);
			break;
		case 4:
			sequeenceParm.setStepValue((Double) aValue);
			break;
		default:
			break;
		}
		// System.out.println("setValueAt
		// 変更後["+rowIndex+","+columnIndex+"]"+sequeenceParm);
		fireTableRowsUpdated(rowIndex, rowIndex);
	}

	/**
	 * デフォルトを指定行に追加
	 * 
	 * @param rowIndex
	 */
	public void insert(int rowIndex) {
		sequenceParmList.add(rowIndex, new SequenceParm());
		fireTableRowsInserted(rowIndex, rowIndex);
	}

	/**
	 * デフォルトを最終行に追加
	 */
	public void lastinsert() {
		insert(sequenceParmList.size());
	}

	/**
	 * 行を削除します
	 * 
	 * @param rowIndex
	 */
	public void delete(int rowIndex) {
		sequenceParmList.remove(rowIndex);
		fireTableRowsDeleted(rowIndex, rowIndex);
	}

	/**
	 * 順序の構成要素生成方法を初期化します
	 */
	public void resetItemData() {
		for (SequenceParm sp : sequenceParmList) {
			sp.setCounterInitialValue();
		}
	}

	/**
	 * 順序の構成要素を次のデータ連結し返却します
	 * 
	 * @return 次のデータ
	 */
	public String getNextItemData() {
		StringBuffer str = new StringBuffer( "" );
		if (sequenceParmList.isEmpty()) {
			return null;
		}
		for (SequenceParm sp : sequenceParmList) {
			str.append( sp.getNext() );
		}
		return str.toString();
	}

}