package jdbcacsess.csv;

import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;

import jdbcacsess.JdbcacsessException;
import jdbcacsess.gui.JFrameSetting;
import jdbcacsess.gui.common.ConstItemHeader;
import jdbcacsess.gui.common.JDialogMessage;
import jdbcacsess.sql.ColumnInfoResult;
import jdbcacsess.sql.QueryExecuteListener;
import jdbcacsess.sql.QueryExecuteStatus;
import jdbcacsess.sql.SchemaTableName;
import jdbcacsess.sql.SqlAnalyze;

public class CsvExport implements QueryExecuteListener {
	private CsvInfo csvInfo;

	private BufferedWriter ot;

	public CsvExport(CsvInfo info) {
		csvInfo = info;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.util.OutputResultble#init()
	 */
	public void init() throws IOException, FileNotFoundException {
		FileOutputStream fos = new FileOutputStream(csvInfo.getFileName());
		ot = new BufferedWriter(new OutputStreamWriter(fos, csvInfo
				.getEncoding().getValue()));
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.util.OutputResultble#setResultHeader(java.util.Vector)
	 */
	public void setResultHeader(ArrayList<ColumnInfoResult> rowData,
			SqlAnalyze sqlAnalyze,boolean flg) throws IOException {
		if (csvInfo.getItemHeader() == ConstItemHeader.TRUE) {
			setResultDetail(rowData);
		}
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.util.OutputResultble#setResultDetail(java.util.Vector)
	 */
	@SuppressWarnings("unchecked")
	public void setResultDetail(ArrayList rowData) throws IOException {
		CSVLine csvline = new CSVLine();
		csvline.setToken(csvInfo.getColumnDelimiter().getValue());

		String nullValue = JFrameSetting.getColumnOutNullValue();
		for (int i = 0; i < rowData.size(); i++) {
			Object o = rowData.get(i);
			if (o != null) {
				csvline.addItem(o.toString());
			} else {
				csvline.addItem(nullValue);
			}
		}
		String str = csvline.getLine() + csvInfo.getLineSeparator().getValue();
		ot.write(str, 0, str.length());
		return;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.util.OutputResultble#notifyEnd()
	 */
	public void completeQuery(QueryExecuteStatus breakStatus) throws JdbcacsessException {
		try {
			if (ot != null) {
				ot.close();
			}
		} catch (IOException e) {
			// TODO 自動生成された catch ブロック
			e.printStackTrace();
		}
		if (!breakStatus.isErr()) {
			SchemaTableName stn = csvInfo.getPropTableName();
			if (!stn.getCompleteTableName().equals("")) {
				csvInfo.currentDataSave();
				JDialogMessage.infoDialog(breakStatus.getCnt()
						+ "件出力しました\n今回の出力条件をCSV入力履歴に載せました。", "CSV出力終了しました "
						+ csvInfo.getPropTableName());
			} else {
				JDialogMessage.infoDialog(
						breakStatus.getCnt() + "件出力しました\n複数テーブルの出力結果なので、\n"
								+ "今回の出力条件はCSV入力履歴に載せまん", "CSV出力終了しました");
			}
		}
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.sql.QueryExecuteListener#status(jdbcacsess.sql.QueryExecuteStatus)
	 */
	public void status(QueryExecuteStatus breakStatus) {
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.sql.QueryExecuteListener#errorException(java.lang.Exception)
	 */
	public void errorException(Exception ex) {
	}

}
