package jdbcacsess.csv;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;

import jdbcacsess.InfoListSetting;
import jdbcacsess.gui.common.ConstColumnDelimiter;
import jdbcacsess.gui.common.ConstCommitCnt;
import jdbcacsess.gui.common.ConstDupliKeyAction;
import jdbcacsess.gui.common.ConstEncoding;
import jdbcacsess.gui.common.ConstItemHeader;
import jdbcacsess.gui.common.ConstLineSeparator;
import jdbcacsess.sql.SchemaTableName;

import org.w3c.dom.Element;

/**
 * <p>
 * タイトル: CSVパラメータアクセス処理
 * </p>
 * <p>
 * 説明: CSVアクセス方法パラメータの管理を行う。
 * </p>
 * <p>
 * １． プロパティファイルへのCSVアクセスパラメータの保存及び読込
 * </p>
 * <p>
 * ２． 各パラメータのコンボボックスデータの作成（固定値として本クラスで保持）
 * </p>
 * <p>
 * ３． CSVアクセス方法パラメータに従い、CSVファイルへの出力とCSVファイルからの入力を行う
 * </p>
 * <p>
 * 著作権: Copyright (c) 2002
 * </p>
 * <p>
 * 会社名:
 * </p>
 * 
 * @author 未入力
 * @version 1.0
 */
/**
 * @author sima
 * 
 */
public class CsvInfo extends InfoListSetting<CsvInfo> {
	private String fileName = "";

	private SchemaTableName tableName = new SchemaTableName("");


	private ConstColumnDelimiter columnDelimiter = ConstColumnDelimiter.COMMA;
	private ConstLineSeparator lineSeparator = ConstLineSeparator.DEFAULT;
	private ConstItemHeader itemHeader = ConstItemHeader.TRUE;
	private ConstDupliKeyAction duplicateKeyAction = ConstDupliKeyAction.NONE;
	private ConstCommitCnt commitCnt = ConstCommitCnt.AUTO;
	private ConstEncoding encoding = ConstEncoding.searchName("");


	private String lastModified;

	private String lastImportExecuted = "";

	private SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	@Override
	public String toString() {
		return "[CsvInfo]" + "fileName=" + fileName + "tableName=" + tableName
				+ "columnDelimiter=" + columnDelimiter + "lineSeparator="
				+ lineSeparator + "encoding=" + encoding + "hasItemHeader="
				+ itemHeader + "duplicateKeyAction=" + duplicateKeyAction
				+ "commitCnt=" + commitCnt;
	}

	@Override
	public String getKeyValue() {
		return getFileName();
	}

	@Override
	public void setKeyValue(String key) {
		setFileName(key);
	}

	@Override
	public String getBaseTagName() {
		return "csvinfo";
	}

	@Override
	public CsvInfo getInstance() {
		return new CsvInfo();
	}

	/**
	 * XML要素からインスタンスへ設定する。但し、ファイル名は対象外なので呼出元で設定して下さい
	 * 
	 * @param cvsInfoNode
	 *            入力となるXML要素
	 */
	@Override
	public void setFromElement(Element cvsInfoNode) {
		setPropTableName(new SchemaTableName(cvsInfoNode
				.getAttribute("TableName")));
		setColumnDelimiter(cvsInfoNode.getAttribute("ColumnDelimiter"));
		setLineSeparator(cvsInfoNode.getAttribute("LineSeparator"));
		setEncoding(cvsInfoNode.getAttribute("Encoding"));
		setItemHeader(cvsInfoNode.getAttribute("ItemHeader"));

		setDuplicateKeyAction(cvsInfoNode.getAttribute("DuplicateKeyAction"));
		
		setCommitCnt(cvsInfoNode.getAttribute("CommitCnt"));

		File file = new File(getFileName());
		if (file.exists()) {
			lastModified = sdf.format(new Date(file.lastModified()));
		} else {
			lastModified = "not found";
		}

		lastImportExecuted = cvsInfoNode.getAttribute("LastImportExecuted");
	}

	/**
	 * インスタンスからXML要素へ追加設定する。但し、ファイル名は対象外なので呼出元で設定して下さい。
	 * 
	 * @param cvsInfoNode
	 *            出力先となるXML要素
	 */
	@Override
	public void addToElement(Element cvsInfoNode) {
		cvsInfoNode.setAttribute("TableName", getPropTableName().getCompleteTableName());
		cvsInfoNode.setAttribute("ColumnDelimiter", getColumnDelimiter().toString());
		cvsInfoNode.setAttribute("LineSeparator", getLineSeparator().toString());
		
		cvsInfoNode.setAttribute("Encoding", getEncoding().toString());

		cvsInfoNode.setAttribute("ItemHeader", getItemHeader().toString());
		cvsInfoNode.setAttribute("DuplicateKeyAction",getDuplicateKeyAction().toString());
		cvsInfoNode.setAttribute("CommitCnt", getCommitCnt().toString());
		
		cvsInfoNode.setAttribute("LastImportExecuted", lastImportExecuted);
	}

	public Vector<String> getHeader() {
		Vector<String> list = new Vector<String>();
		list.add("ファイル名");
		list.add("ファイル更新日時");
		list.add("テーブル名");
		list.add(ConstColumnDelimiter.getEnumName());
		list.add(ConstLineSeparator.getEnumName());
		list.add(ConstEncoding.getEnumName());
		list.add(ConstItemHeader.getEnumName());
		list.add(ConstDupliKeyAction.getEnumName());
		list.add(ConstCommitCnt.getEnumName());
		list.add("Import実行日時");
		return list;
	}

	public String getData(int index) {
		String rtn = "";
		switch (index) {
		case 0:
			rtn = getFileName();
			break;
		case 1:
			rtn = lastModified;
			break;
		case 2:
			rtn = getPropTableName().getCompleteTableName();
			break;
		case 3:
			rtn = getColumnDelimiter().toString();
			break;
		case 4:
			rtn = getLineSeparator().toString();
			break;
		case 5:
			rtn = getEncoding().toString();
			break;
		case 6:
			rtn = getItemHeader().toString();
			break;
		case 7:
			rtn = getDuplicateKeyAction().toString();
			break;
		case 8:
			rtn = getCommitCnt().toString();
			break;
		case 9:
			if (lastImportExecuted.equals("")) {
				rtn = " ";
			} else {
				rtn = lastImportExecuted;
			}
			break;
		default:
			break;
		}
		return rtn;
	}

	/**
	 * 対象ファイルのsetter
	 */
	public void setFileName(String fileName) {
		this.fileName = fileName;
	}

	/**
	 * 対象ファイルのgetter
	 */
	public String getFileName() {
		return fileName;
	}

	/**
	 * テーブル名のsetter
	 */
	public void setPropTableName(SchemaTableName stn) {
		this.tableName = stn;
	}

	/**
	 * テーブル名のgetter
	 */
	public SchemaTableName getPropTableName() {
		return tableName;
	}

	/**
	 * 項目区切り文字
	 */
	public void setColumnDelimiter(String propColumndelimiter) {
		this.columnDelimiter = ConstColumnDelimiter.searchName(propColumndelimiter);
	}
	public void setColumnDelimiter(ConstColumnDelimiter columnDelimiter) {
		this.columnDelimiter = columnDelimiter;
	}
	public ConstColumnDelimiter getColumnDelimiter() {
		return columnDelimiter;
	}

	/**
	 * 改行文字
	 */
	public void setLineSeparator(String propLineSeparator) {
		lineSeparator = ConstLineSeparator.searchName(propLineSeparator);
	}
	public void setLineSeparator(ConstLineSeparator lineSeparator) {
		this.lineSeparator = lineSeparator;
	}
	public ConstLineSeparator getLineSeparator() {
		return lineSeparator;
	}

	/**
	 * 重複時対応
	 */
	public void setDuplicateKeyAction(String propDuplicateKeyAction) {
		duplicateKeyAction = ConstDupliKeyAction.searchName(propDuplicateKeyAction);
	}
	public void setDuplicateKeyAction(ConstDupliKeyAction duplicateKeyAction) {
		this.duplicateKeyAction = duplicateKeyAction;
	}
	public ConstDupliKeyAction getDuplicateKeyAction() {
		return duplicateKeyAction;
	}

//	public DefaultComboBoxModel getModelEncoding() {
//		DefaultComboBoxModel modelEncoding = new DefaultComboBoxModel();
//		modelEncoding.addElement("実行環境デフォルト");
//		modelEncoding.addElement("UTF-8");
//		modelEncoding.addElement("EUC_JP");
//		modelEncoding.addElement("Shift_JIS");
//		modelEncoding.addElement("Windows-31J");
//		return modelEncoding;
//	}
//
	/**
	 * エンコード
	 */
	public void setEncoding(String encoding) {
		this.encoding = ConstEncoding.searchName(encoding);
	}
	public void setEncoding(ConstEncoding encoding) {
		this.encoding = encoding;
	}
	public ConstEncoding getEncoding() {
		return encoding;
	}

	/**
	 * 見出し有無
	 */
	public void setItemHeader(String propItemheader) {
		this.itemHeader = ConstItemHeader.searchName(propItemheader);
	}
	public void setItemHeader(ConstItemHeader propItemHeader) {
		this.itemHeader = propItemHeader;
	}
	public ConstItemHeader getItemHeader() {
		return itemHeader;
	}

	/**
	 * commitタイミング
	 */
	public void setCommitCnt(String propCommitCnt) {
		this.commitCnt = ConstCommitCnt.searchName(propCommitCnt);
	}
	public void setCommitCnt(ConstCommitCnt propCommitCnt) {
		this.commitCnt = propCommitCnt;
	}
	public ConstCommitCnt getCommitCnt() {
		return commitCnt;
	}

	/**
	 * 最終Import実行日時を現在時刻に設定する
	 */
	public void setCurrentImportExecuted() {
		lastImportExecuted = sdf.format(new Date(System.currentTimeMillis()));
	}

	/**
	 * 最終Import実行日時を初期化する
	 */
	public void clearImportExecuted() {
		lastImportExecuted = "";
	}

}