/**
 * 
 */
package jdbcacsess.csv;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

import jdbcacsess.JdbcacsessException;
import jdbcacsess.gui.common.ConstItemHeader;
import jdbcacsess.gui.common.JDialogMessage;
import jdbcacsess.gui.common.JFrameBase;

/**
 * @author sima
 * 
 */
public class CsvInputDataView extends CsvInput {

	private JFrame jFrame = null; // @jve:decl-index=0:visual-constraint="50,27"

	private JTable jTable = null;

	Vector<String> columnNames;

	Vector<Vector<String>> rowDatas;

	int maxColumnCnt;

	/**
	 * @param info
	 */
	public CsvInputDataView(CsvInfo info) {
		super(info);

		if (!load()) {
			return;
		}

		// 画面表示
		jFrame = new JFrameCsvInputDataView(csvInfo.getFileName());
		jFrame.setVisible(true);

	}

	private boolean load() {
		// ファイルオープン
		try {
			openInputFile();
		} catch (JdbcacsessException e) {
			return false;
		}

		// ファイル読込
		inputCnt = 0;
		try {
			getColumnNames();
			getRowData();

		} catch (JdbcacsessException e) {
			return false;
		} finally {
			closeInputFile();
		}
		// ヘッダが無かった時は自動生成
		if (columnNames.isEmpty()) {
			if (!rowDatas.isEmpty()) {
				for (int i = 0; i < maxColumnCnt; i++) {
					columnNames.add(Integer.valueOf(i).toString());
				}
			}
		}

		boolean err = false;
		// ヘッダの不足部分にダミーを追加
		int cnt = maxColumnCnt - columnNames.size();
		for (int i = 0; i < cnt; i++) {
			columnNames.add("■?■");
			err = true;
		}

		// 見出し確定後に、見出しから最大カラム数を算出
		if (maxColumnCnt < columnNames.size()) {
			maxColumnCnt = columnNames.size();
		}

		// 明細の不足部分にダミーを追加
		for (Vector<String> rows : rowDatas) {
			cnt = maxColumnCnt - rows.size();
			for (int i = 0; i < cnt; i++) {
				rows.add("■?■");
				err = true;
			}
		}
		// カラム数不一致のダイアログ
		if (err) {
			JDialogMessage.errorDialog(new Exception(
					"先頭100件以内にカラム数が一致してないデータがあります。\n" + "右端の『■?■』を確認して下さい。\n"
							+ "しつこいですが、ファイル全体はチェックしてません。"));
		}

		return true;
	}

	/**
	 * @throws IOException
	 * @throws JdbcacsessException
	 */
	private void getRowData() throws JdbcacsessException {
		rowDatas = new Vector<Vector<String>>();
		try {
			while (inputCnt <= 100) {
				Vector<String> rowData = readFile();
				if (rowData == null) {
					break;
				}
				// 最大カラム数を算出
				if (maxColumnCnt < rowData.size()) {
					maxColumnCnt = rowData.size();
				}
				rowDatas.add(rowData);
			}
		} catch (IOException e) {
			JDialogMessage.errorDialog(e);
			throw new JdbcacsessException(e);
		}
	}

	/**
	 * @throws JdbcacsessException
	 */
	private void getColumnNames() throws JdbcacsessException {
		if (csvInfo.getItemHeader() == ConstItemHeader.TRUE) {
			// CSVの見出しをListに作成する。
			try {
				columnNames = this.readFile();
			} catch (IOException e) {
				JDialogMessage.errorDialog(e);
				throw new JdbcacsessException(e);
			}
			if (columnNames == null) {
				Exception e = new Exception("見出し有りなのに、空ファイルです");
				JDialogMessage.errorDialog(e);
				throw new JdbcacsessException(e);
			}
		} else {
			columnNames = new Vector<String>();
		}
	}

	/**
	 * 
	 */
	private void actionPerfromedReLoad() {
		if (!load()) {
			return;
		}
		jTable.setModel(new DefaultTableModel(rowDatas, columnNames));
	}

	public class JFrameCsvInputDataView extends JFrameBase {
		@Override
		public String getAttrKey(){ 
			return null;
		}
		@Override
		public String getAttrValue(){ 
			return null;
		}

		/**
		 * 
		 */
		private static final long serialVersionUID = 2655462988366965325L;

		private JPanel jContentPane = null;

		private JPanel jPanelMain = null;

		private JScrollPane jScrollPane = null;

		private JPanel jPanel1 = null;

		private JButton jButtonReLoad = null;

		private JFrameCsvInputDataView(String title) {
			initialize();
			setTitle(title);
		}

		public void initialize() {
			setSize(new Dimension(400, 200));
			setContentPane(getJContentPane());
		}

		/**
		 * This method initializes jContentPane
		 * 
		 * @return javax.swing.JPanel
		 */
		private JPanel getJContentPane() {
			if (jContentPane == null) {
				jContentPane = new JPanel();
				jContentPane.setLayout(new BorderLayout());
				jContentPane.add(getJPanelMain(), java.awt.BorderLayout.CENTER);
				jContentPane.add(getJPanel1(), java.awt.BorderLayout.SOUTH);
			}
			return jContentPane;
		}

		/**
		 * This method initializes jPanel
		 * 
		 * @return javax.swing.JPanel
		 */
		private JPanel getJPanelMain() {
			if (jPanelMain == null) {
				jPanelMain = new JPanel();
				jPanelMain.setLayout(new BorderLayout());
				jPanelMain.add(getJScrollPane(), java.awt.BorderLayout.CENTER);
			}
			return jPanelMain;
		}

		/**
		 * This method initializes jScrollPane
		 * 
		 * @return javax.swing.JScrollPane
		 */
		private JScrollPane getJScrollPane() {
			if (jScrollPane == null) {
				jScrollPane = new JScrollPane();
				jScrollPane.setViewportView(getJTable());
			}
			return jScrollPane;
		}

		/**
		 * This method initializes jTable
		 * 
		 * @return javax.swing.JTable
		 */
		private JTable getJTable() {
			if (jTable == null) {
				jTable = new JTable(rowDatas, columnNames);
				jTable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF);
				jTable.setDefaultEditor(Object.class, null);
			}
			return jTable;
		}

		/**
		 * This method initializes jPanel1
		 * 
		 * @return javax.swing.JPanel
		 */
		private JPanel getJPanel1() {
			if (jPanel1 == null) {
				jPanel1 = new JPanel();
				jPanel1.add(getJButtonReLoad(), null);
			}
			return jPanel1;
		}

		/**
		 * This method initializes jButton
		 * 
		 * @return javax.swing.JButton
		 */
		private JButton getJButtonReLoad() {
			if (jButtonReLoad == null) {
				jButtonReLoad = new JButton();
				jButtonReLoad.setText("再読込");
				jButtonReLoad
						.addActionListener(new java.awt.event.ActionListener() {
							public void actionPerformed(
									java.awt.event.ActionEvent e) {
								actionPerfromedReLoad();
							}
						});
			}
			return jButtonReLoad;
		}
		@Override
		public void frameClosing() throws JdbcacsessException {
			// TODO 自動生成されたメソッド・スタブ
			
		}

	}

}
