package jdbcacsess.csv;

import java.io.IOException;
import java.io.InterruptedIOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Vector;

import jdbcacsess.JdbcacsessException;
import jdbcacsess.LogOutput;
import jdbcacsess.gui.common.ConstCommitCnt;
import jdbcacsess.gui.common.ConstDupliKeyAction;
import jdbcacsess.gui.common.ConstItemHeader;
import jdbcacsess.gui.common.JDialogMessage;
import jdbcacsess.sql.PrepareExecuteStatement;
import jdbcacsess.sql.SqlExec;

/**
 * @author sima CSVファイルからデータを読み込みDB更新する。 切り貼りで作ったクラスなので、非常にバランスが悪くなった。
 *         けど、作り直す自信が無い。
 */
public class CsvInputImportToDB extends CsvInput implements Runnable {
	private Connection con;

	private int resultCntUpdate;

	private int resultCntInsert;

	private int resultCntDelete;

	public CsvInputImportToDB(CsvInfo info) {
		super(info);
	}

	public void run() {
		inputCnt = 0;
		resultCntUpdate = 0;
		resultCntInsert = 0;
		resultCntDelete = 0;

		in = null;
		con = null;
		try {
			// 入力ファイルのオープン
			openInputFile();

			// 更新用にDB接続用意
			connectDataBase();

			// CSV見出しのチェックを行う
			CsvImportSqlExecute cise = headerCheck();

			if (csvInfo.getDuplicateKeyAction() == ConstDupliKeyAction.ALLDELETE) {
				// テーブルの全データを削除する
				allDelete();
			}

			// ファイルからデータを読み込みDB更新を行う
			fileImport(cise);

			JDialogMessage.infoDialog(msg(), "Import成功");
		} catch (JdbcacsessException cie) {
			if (con != null) {
				try {
					if (!con.getAutoCommit()){
						try {
							con.rollback();
							LogOutput.info("RollBackしました");
						} catch (SQLException e) {
							JDialogMessage.errorDialog(e, "RollBackに失敗" + msg());
						}
					}
				} catch (SQLException e) {
					JDialogMessage.errorDialog(e, "AutoCommit取得");
				}
			}
		} finally {
			closeInputFile();
			SqlExec.disconnect(con);
			csvInfo.setCurrentImportExecuted();
			try {
				csvInfo.currentDataSave();
			} catch (JdbcacsessException e) {
				JDialogMessage.errorDialog(e);
			}
		}
	}

	private String msg() {
		return "\n" + csvInfo.getPropTableName().getCompleteTableName()
				+ " のImport終了" + "\n  INFILE: " + inputCnt + " レコード"
				+ "\n  UPDATE: " + resultCntUpdate + " 行" + "\n  INSERT: "
				+ resultCntInsert + " 行" + "\n  DELETE: " + resultCntDelete
				+ " 行" + "\n";
	}

	private void connectDataBase() throws JdbcacsessException {
		try {
			con = SqlExec.getNewConnection();
			if (csvInfo.getCommitCnt() == ConstCommitCnt.AUTO) {
				con.setAutoCommit(true);
			} else {
				con.setAutoCommit(false);
			}
		} catch (SQLException e) {
			JDialogMessage.errorDialog(e, "DB接続でエラー" + msg());
			throw new JdbcacsessException(e);
		}
	}

	private void allDelete() throws JdbcacsessException {
		PrepareExecuteStatement pes = new PrepareExecuteStatement();
		pes.addStm("DELETE FROM "
				+ csvInfo.getPropTableName().getCompleteTableName());
		try {
			pes.prepare(con);
			resultCntDelete = pes.executeUpdate();
		} catch (SQLException e) {
			JDialogMessage.errorDialog(e, "全件削除でエラー" + pes.getStm() + msg());
			throw new JdbcacsessException(e);
		}
	}

	private CsvImportSqlExecute headerCheck() throws JdbcacsessException {
		Vector<String> header = new Vector<String>();
		if (csvInfo.getItemHeader() == ConstItemHeader.TRUE) {
			// CSVの見出しをListに作成する。
			try {
				header = this.readFile();
			} catch (IOException e) {
				JDialogMessage.errorDialog(e);
				throw new JdbcacsessException(e);
			}
			if (header == null) {
				Exception e = new Exception("見出し有りなのに、空ファイルです" + msg());
				JDialogMessage.errorDialog(e);
				throw new JdbcacsessException(e);
			}
		}

		CsvImportSqlExecute cise=null;
		try {
			cise = new CsvImportSqlExecute(con, csvInfo.getPropTableName(),
					header);
		} catch (Exception e) {
			JDialogMessage.errorDialog(e);
			throw new JdbcacsessException(e);
		}

		// 見出しカラム名のチェック
		if (csvInfo.getItemHeader() == ConstItemHeader.TRUE) {
			// 見出し有りのときは、テーブル定義に存在しないカラムは許さない
			String str = cise.hasAllColumn();
			if (!str.equals("")) {
				Exception e = new Exception("CSVの見出しカラムがテーブル定義に無い\n" + str+ msg());
				JDialogMessage.errorDialog(e);
				throw new JdbcacsessException(e);
			}
			// 重複時置換えるには、主キーが全て揃っていないと更新できない
			// 主キーの無いテーブルはUPDATE指定出来ないという事になる
			if (csvInfo.getDuplicateKeyAction() == ConstDupliKeyAction.UPDATE
					|| csvInfo.getDuplicateKeyAction() == ConstDupliKeyAction.NONE) {
				str = cise.hasPrimeKey();
				if (!str.equals("")) {
					Exception e = new Exception("主キー定義カラムがCSVの見出しに無い\n" + str+ msg());
					JDialogMessage.errorDialog(e);
					throw new JdbcacsessException(e);
				}
			}
		}
		return cise;
	}

	private void fileImport(CsvImportSqlExecute cise)
			throws JdbcacsessException {
		// 入力Fのレコード分ループ処理
		int commitCnt = 1;
		try {
			for (Vector<String> rowData = readFile(); rowData != null; rowData = readFile()) {
				switch (csvInfo.getDuplicateKeyAction()) {
				case UPDATE:
					// 置換え指定なので、UPDATE発行し、データが無かった時はカラム指定INSERT
					int updateCnt = cise.execUpdate(rowData);
					if (updateCnt == 0) {
						resultCntInsert += cise
								.execInsertColumnAndValue(rowData);
					} else {
						resultCntUpdate += updateCnt;
					}
					break;
				case NONE:
					// データが無い時だけINSERTする。データが存在した時は何もしない
					if (cise.querySelectCount(rowData) == 0) {
						resultCntInsert += cise
								.execInsertColumnAndValue(rowData);
					}
					break;
				default:
					if (csvInfo.getItemHeader() == ConstItemHeader.TRUE) {
						// 置換え指定以外なので、カラム有指定で単純INSERT
						resultCntInsert += cise
								.execInsertColumnAndValue(rowData);
					} else {
						// 置換え指定以外なので、カラム無指定で単純INSERT
						resultCntInsert += cise
								.execInsertOnlyValuePhrase(rowData);
					}
					break;
				}
				// １００件毎COMITが指定された時の処理
				if (csvInfo.getCommitCnt() != ConstCommitCnt.AUTO && csvInfo.getCommitCnt() != ConstCommitCnt.LAST) {
					commitCnt++;
					if (commitCnt > csvInfo.getCommitCnt().getValue()) {
						con.commit();
						LogOutput.info("Commitしました(loop)");
						commitCnt = 1;
					}
				}
			}
			// AutoCommit以外のCommit処理
			if (csvInfo.getCommitCnt() != ConstCommitCnt.AUTO) {
				LogOutput.info("Commitしました");
				con.commit();
			}
		} catch (SQLException e) {
			JDialogMessage.errorDialog(e, "DB更新中にエラー" + msg());
			throw new JdbcacsessException(e);
		} catch (InterruptedIOException e){
			JDialogMessage.infoDialog(msg(), "Import中止");
			throw new JdbcacsessException(e);
		} catch (Exception e) {
			JDialogMessage.errorDialog(e);
			throw new JdbcacsessException(e);
		}
	}

}
