/**
 * 
 */
package jdbcacsess.csv;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import jdbcacsess.JdbcacsessException;

/**
 * @author sima
 * 
 */
public class ImportHistoryTableModel extends AbstractTableModel {

	private static final long serialVersionUID = -2574911637821580582L;

	private List<CsvInfo> csvInfoList;

	private Vector<String> header;

	private int columnCount;

	private List<Integer> convList;

	private int sortColumnIndex;

	private boolean orderByAsc;

	public ImportHistoryTableModel() throws JdbcacsessException {
		super();
		header = new CsvInfo().getHeader();
		columnCount = header.size();
		sortColumnIndex = 0;
		orderByAsc = false;
		convList = new ArrayList<Integer>();
		setSortColumnIndex(sortColumnIndex);
	}

	@Override
	public String getColumnName(int idx) {
		String allow = "";
		// ソート方向記号を出したいのだが、見出し文字列だけを書き換えるイベントって無いのか？
		// fireTableStructureChanged()で全体書き換えだとセル幅が変わってしまうので駄目。
		// if (idx==sortColumnIndex){
		// allow = (orderByAsc)?"↓":"↑";
		// }
		return header.get(idx) + allow;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	public int getRowCount() {
		return csvInfoList.size();
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	public int getColumnCount() {
		return columnCount;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int rowIndex, int columnIndex) {
		return csvInfoList.get(convRowIndex(rowIndex)).getData(columnIndex);
	}

	public String getKey(int rowIndex) {
		return csvInfoList.get(convRowIndex(rowIndex)).getFileName();
	}

	public void remove(String key) throws JdbcacsessException {

		for (CsvInfo ci : csvInfoList) {
			if (ci.getFileName().equals(key)) {
				csvInfoList.remove(ci);
				ci.removeParameter();
				break;
			}
		}
	}

	public CsvInfo getCsvInfo(int rowIndex) {
		return csvInfoList.get(convRowIndex(rowIndex));

	}

	private int convRowIndex(int index) {
		return convList.get(index);
	}

	public void reLoad() throws JdbcacsessException {
		orderByAsc = !orderByAsc;
		setSortColumnIndex(sortColumnIndex);
	}

	/**
	 * @param columnIndex
	 * @throws JdbcacsessException 
	 */
	public void setSortColumnIndex(int columnIndex) throws JdbcacsessException {
		if (sortColumnIndex == columnIndex) {
			orderByAsc = !orderByAsc;
		}
		sortColumnIndex = columnIndex;

		csvInfoList = new CsvInfo().getInfoList();
		TreeMap<String, Integer> map = new TreeMap<String, Integer>();
		for (int rowIndex = 0; rowIndex < csvInfoList.size(); rowIndex++) {
			map.put(csvInfoList.get(rowIndex).getData(columnIndex) + rowIndex,
					rowIndex);
		}

		convList.clear();
		for (Map.Entry<String, Integer> i : map.entrySet()) {
			if (orderByAsc) {
				convList.add(i.getValue());
			} else {
				convList.add(0, i.getValue());
			}
		}
		fireTableDataChanged();

	}
}
