package jdbcacsess.csv;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.io.File;
import java.sql.SQLException;

import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingConstants;
import javax.swing.ToolTipManager;

import jdbcacsess.gui.JFrameSetting;
import jdbcacsess.gui.common.ComponentProperty;
import jdbcacsess.gui.common.ConstColumnDelimiter;
import jdbcacsess.gui.common.ConstCommitCnt;
import jdbcacsess.gui.common.ConstDupliKeyAction;
import jdbcacsess.gui.common.ConstEncoding;
import jdbcacsess.gui.common.ConstItemHeader;
import jdbcacsess.gui.common.ConstLineSeparator;
import jdbcacsess.gui.common.JDialogMessage;
import jdbcacsess.gui.common.JPopupMenuEdit;
import jdbcacsess.sql.SchemaTableName;
import jdbcacsess.sql.SqlExec;

/**
 * @author sima
 */
public class JDialogCsv extends JDialog {
	private static final long serialVersionUID = 1216128860809624609L;

	private CsvInfo csvInfo;  //  @jve:decl-index=0:

	private CsvMode csvMode;

	private JButton jButtonCancel = null;

	private JButton jButtonFileChoice = null;

	private JButton jButtonOK = null;

	// private JButton jButtonDataView = null;

	private JComboBox jComboBoxCommitTiming = null;

	private JComboBox jComboBoxCrLf = null;

	private JComboBox jComboBoxDelimiter = null;

	private JComboBox jComboBoxDuplicateKey = null;

	private JComboBox jComboBoxOutputHeader = null;

	private JComboBox jComboBoxStringCode = null;

	private JComboBox jComboBoxTableName = null;

	private JPanel jContentPane = null;

	private JLabel jLabelColumnDelimiter;

	private JLabel jLabel2;

	private JLabel jLabel3;

	private JLabel jLabelLineSeparator;

	private JLabel jLabel5;

	private JLabel jLabel6;

	private JLabel jLabel7;

	private JLabel jLabel9;

	private JLabel jLabelModeIO = new JLabel();

	private JPanel jPanelButton = null;

	private JPanel jPanelData = null;

	private JPanel jPanelProf = null;

	private DefaultComboBoxModel modelTableName;

	private JScrollPane jScrollPane = null;

	private JTextArea jTextAreaFileName = null;

	private JButton jButtonChangeFileNameFromTable = null;

	private JPanel jPanel = null;

	public JDialogCsv(CsvMode mode) {
		csvMode = mode;

		csvInfo = new CsvInfo();
		csvInfo.currentDataLoad();

		initialize();
	}

	public JDialogCsv(CsvMode mode, CsvInfo csvInfo) {
		csvMode = mode;

		this.csvInfo = csvInfo;

		initialize();
	}

	/**
	 * @return csvInfo を戻します。
	 */
	public CsvInfo getCsvInfo() {
		return csvInfo;
	}

	private void initialize() {
		new ComponentProperty().restoreWindowPosition(this, 500, 400);
		setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
		setContentPane(getJContentPane());

		setAlwaysOnTop(true);
		addWindowListener(new java.awt.event.WindowAdapter() {
			@Override
			public void windowClosing(java.awt.event.WindowEvent e) {
				dialogClosing();
				cancel();
			}

			@Override
			public void windowOpened(java.awt.event.WindowEvent e) {
				jButtonOK.requestFocusInWindow();
			}
		});

		if (csvMode == CsvMode.IMPORT) {
			jLabelModeIO.setText("ＣＳＶファイル->ＤＢ （インポート）");
			this.setTitle("ＣＳＶ入力");
			// jPanelButton.add(getJButtonDataView(), null);
		}
		if (csvMode == CsvMode.EXPORT) {
			jLabelModeIO.setText("ＤＢ->ＣＳＶファイル （エクスポート）");
			this.setTitle("ＣＳＶ出力");
			jComboBoxTableName.setEnabled(false);
			jComboBoxDuplicateKey.setEnabled(false);
			jComboBoxCommitTiming.setEnabled(false);
		}
		if (csvMode == CsvMode.EDIT) {
			jLabelModeIO.setText("インポート条件編集");
			this.setTitle("編集");
			jButtonOK.setText("保存");
			// jPanelButton.add(getJButtonDataView(), null);
		}
		if (csvMode == CsvMode.CREATEDATA) {
			jLabelModeIO.setText("データ自動生成");
			this.setTitle("ＣＳＶ出力");
			jComboBoxTableName.setEnabled(false);
			jComboBoxDuplicateKey.setEnabled(false);
			jComboBoxCommitTiming.setEnabled(false);
		}

		modelTableName = new DefaultComboBoxModel();
		jComboBoxTableName.setModel(getTableLists());
		String completeTableName = csvInfo.getPropTableName().getCompleteTableName();
		jComboBoxTableName.setSelectedItem(completeTableName.equals("") ? null
				: completeTableName);
		
		jComboBoxDelimiter.setModel(new DefaultComboBoxModel(ConstColumnDelimiter.values()));
		jComboBoxDelimiter.setSelectedItem(csvInfo.getColumnDelimiter());

		jComboBoxCrLf.setModel(new DefaultComboBoxModel(ConstLineSeparator.values()));
		jComboBoxCrLf.setSelectedItem(csvInfo.getLineSeparator());

		jComboBoxStringCode.setModel(new DefaultComboBoxModel(ConstEncoding.values()));
		jComboBoxStringCode.setSelectedItem(csvInfo.getEncoding());
		
		jComboBoxOutputHeader.setModel(new DefaultComboBoxModel(ConstItemHeader.values()));
		jComboBoxOutputHeader.setSelectedItem(csvInfo.getItemHeader());

		jComboBoxDuplicateKey.setModel(new DefaultComboBoxModel(ConstDupliKeyAction.values()));
		jComboBoxDuplicateKey.setSelectedItem(csvInfo.getDuplicateKeyAction());

		jComboBoxCommitTiming.setModel(new DefaultComboBoxModel(ConstCommitCnt.values()));
		jComboBoxCommitTiming.setSelectedItem(csvInfo.getCommitCnt());

		jTextAreaFileName.setText(csvInfo.getFileName());
		jTextAreaFileName.setFont(JFrameSetting.getGuiFont());
		new JPopupMenuEdit(jTextAreaFileName);

		getRootPane().setDefaultButton(jButtonOK);
		setModal(true);
		setVisible(true);
	}

	private void dialogClosing() {
		new ComponentProperty().storeWindowPosition(this);
	}

	private void actionPerformedFileChoice(ActionEvent e) {
		File file = new File(jTextAreaFileName.getText());
		JFileChooser jFileChooser = new JFileChooser(file.getAbsolutePath());
		jFileChooser.setMultiSelectionEnabled(false);

		int ans = jFileChooser.showOpenDialog(this);
		if (ans == JFileChooser.APPROVE_OPTION) {
			jTextAreaFileName.setText(jFileChooser.getSelectedFile().getPath());
		}
	}

	private void actionPerformedOK(ActionEvent e) {
		csvInfo.setFileName(jTextAreaFileName.getText());
		csvInfo.setPropTableName(new SchemaTableName(
				(String) jComboBoxTableName.getSelectedItem()));
		csvInfo.setColumnDelimiter((ConstColumnDelimiter) jComboBoxDelimiter.getSelectedItem());
		csvInfo.setLineSeparator((ConstLineSeparator)jComboBoxCrLf.getSelectedItem());

		csvInfo.setEncoding((ConstEncoding) jComboBoxStringCode.getSelectedItem());
		csvInfo.setItemHeader((ConstItemHeader) jComboBoxOutputHeader.getSelectedItem());
		
		csvInfo.setDuplicateKeyAction((ConstDupliKeyAction)jComboBoxDuplicateKey.getSelectedItem());
		csvInfo.setCommitCnt((ConstCommitCnt)jComboBoxCommitTiming.getSelectedItem());

		csvInfo.clearImportExecuted();

		if (csvMode == CsvMode.IMPORT || csvMode == CsvMode.EDIT) {
			if ((csvInfo.getDuplicateKeyAction() == ConstDupliKeyAction.UPDATE || csvInfo
					.getDuplicateKeyAction() == ConstDupliKeyAction.NONE)
					&& csvInfo.getItemHeader() == ConstItemHeader.FALSE) {
	
				if (JOptionPane.showConfirmDialog(null,
						"WHERE句を作成する為にはCSVデータに見出しが必要です。"
								+ System.getProperty("line.separator")
								+ "『常にINSERT』で処理しますか？", "重複データの『UPDATE/何もしない』無効",
						JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == JOptionPane.OK_OPTION) {
					jComboBoxDuplicateKey.setSelectedItem(ConstDupliKeyAction.ALLWAYINSERT);
					csvInfo.setDuplicateKeyAction((ConstDupliKeyAction) jComboBoxDuplicateKey.getSelectedItem());
				} else {
					return;
				}
			}
		}
		dialogClosing();
		dispose();
	}

	private void actionPerformedCancel(ActionEvent e) {
		cancel();
	}

	// ダイアログを閉じる
	void cancel() {
		csvInfo = null;
		dispose();
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJPanelProf(), java.awt.BorderLayout.NORTH);
			jContentPane.add(getJPanelData(), java.awt.BorderLayout.CENTER);
			jContentPane.add(getJPanelButton(), java.awt.BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	private JPanel getJPanelProf() {
		if (jPanelProf == null) {
			jPanelProf = new JPanel();
			jPanelProf.add(jLabelModeIO);
			jLabelModeIO.setFont(new java.awt.Font("Dialog", 0, 14));
			jLabelModeIO.setHorizontalAlignment(SwingConstants.CENTER);
		}
		return jPanelProf;
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getJTextArea());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jTextArea
	 * 
	 * @return javax.swing.JTextArea
	 */
	private JTextArea getJTextArea() {
		if (jTextAreaFileName == null) {
			jTextAreaFileName = new JTextArea();
			jTextAreaFileName.setLineWrap(true);
			jTextAreaFileName.setFont(new java.awt.Font("Dialog",
					java.awt.Font.PLAIN, 12));
			jTextAreaFileName.setTabSize(4);
		}
		return jTextAreaFileName;
	}

	private JButton getJButtonFileChoice() {
		if (jButtonFileChoice == null) {
			jButtonFileChoice = new JButton();
			jButtonFileChoice.setText("参照...");
			jButtonFileChoice
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(ActionEvent e) {
							actionPerformedFileChoice(e);
						}
					});
		}
		return jButtonFileChoice;
	}

	private JComboBox getJComboBoxCommitTiming() {
		if (jComboBoxCommitTiming == null) {
			jComboBoxCommitTiming = new JComboBox();
			jComboBoxCommitTiming.setFont(new java.awt.Font("Dialog",
					java.awt.Font.PLAIN, 12));
		}
		return jComboBoxCommitTiming;
	}

	private JComboBox getJComboBoxCrLf() {
		if (jComboBoxCrLf == null) {
			jComboBoxCrLf = new JComboBox();
			jComboBoxCrLf.setFont(new java.awt.Font("Dialog",
					java.awt.Font.PLAIN, 12));
		}
		return jComboBoxCrLf;
	}

	private JComboBox getJComboBoxDelimiter() {
		if (jComboBoxDelimiter == null) {
			jComboBoxDelimiter = new JComboBox();
			jComboBoxDelimiter.setFont(new java.awt.Font("Dialog",
					java.awt.Font.PLAIN, 12));
		}
		return jComboBoxDelimiter;
	}

	private JComboBox getJComboBoxDuplicateKey() {
		if (jComboBoxDuplicateKey == null) {
			jComboBoxDuplicateKey = new JComboBox();
			jComboBoxDuplicateKey.setFont(new java.awt.Font("Dialog",
					java.awt.Font.PLAIN, 12));
		}
		return jComboBoxDuplicateKey;
	}

	private JComboBox getJComboBoxOutputHeader() {
		if (jComboBoxOutputHeader == null) {
			jComboBoxOutputHeader = new JComboBox();
			jComboBoxOutputHeader.setFont(new java.awt.Font("Dialog",
					java.awt.Font.PLAIN, 12));
		}
		return jComboBoxOutputHeader;
	}

	private JComboBox getJComboBoxStringCode() {
		if (jComboBoxStringCode == null) {
			jComboBoxStringCode = new JComboBox();
			jComboBoxStringCode.setFont(new java.awt.Font("Dialog",
					java.awt.Font.PLAIN, 12));
		}
		return jComboBoxStringCode;
	}

	private JComboBox getJComboBoxTableName() {
		if (jComboBoxTableName == null) {
			jComboBoxTableName = new JComboBox();
			jComboBoxTableName.setFont(new java.awt.Font("Dialog",
					java.awt.Font.PLAIN, 12));
		}
		return jComboBoxTableName;
	}

	private JPanel getJPanelData() {
		if (jPanelData == null) {
			GridBagConstraints gridBagConstraints16 = new GridBagConstraints();
			gridBagConstraints16.gridx = 4;
			gridBagConstraints16.gridy = 0;
			GridBagConstraints gridBagConstraints22 = new GridBagConstraints();
			gridBagConstraints22.fill = java.awt.GridBagConstraints.BOTH;
			gridBagConstraints22.gridy = 0;
			gridBagConstraints22.weightx = 1.0;
			gridBagConstraints22.weighty = 1.0;
			gridBagConstraints22.gridwidth = 2;
			gridBagConstraints22.insets = new java.awt.Insets(5, 2, 5, 0);
			gridBagConstraints22.anchor = java.awt.GridBagConstraints.SOUTH;
			gridBagConstraints22.gridheight = 1;
			gridBagConstraints22.gridx = 2;
			GridBagConstraints gridBagConstraints15 = new GridBagConstraints(0,
					0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
					GridBagConstraints.BOTH, new Insets(5, 0, 5, 0), 0, 0);
			gridBagConstraints15.gridx = 0;
			gridBagConstraints15.fill = java.awt.GridBagConstraints.NONE;
			gridBagConstraints15.anchor = java.awt.GridBagConstraints.EAST;
			gridBagConstraints15.gridheight = 1;
			gridBagConstraints15.insets = new java.awt.Insets(5, 0, 5, 3);
			gridBagConstraints15.gridy = 0;
			GridBagConstraints gridBagConstraints14 = new GridBagConstraints(1,
					9, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
					GridBagConstraints.BOTH, new Insets(5, 0, 5, 0), 0, 0);
			gridBagConstraints14.gridy = 8;
			gridBagConstraints14.gridwidth = 2;
			gridBagConstraints14.anchor = java.awt.GridBagConstraints.CENTER;
			gridBagConstraints14.insets = new java.awt.Insets(5, 2, 5, 0);
			gridBagConstraints14.gridx = 2;
			GridBagConstraints gridBagConstraints13 = new GridBagConstraints(1,
					8, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
					GridBagConstraints.BOTH, new Insets(5, 0, 5, 0), 0, 0);
			gridBagConstraints13.gridy = 7;
			gridBagConstraints13.gridwidth = 2;
			gridBagConstraints13.anchor = java.awt.GridBagConstraints.CENTER;
			gridBagConstraints13.insets = new java.awt.Insets(5, 2, 5, 0);
			gridBagConstraints13.gridx = 2;
			GridBagConstraints gridBagConstraints12 = new GridBagConstraints(1,
					7, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
					GridBagConstraints.BOTH, new Insets(5, 0, 5, 0), 0, 0);
			gridBagConstraints12.gridy = 6;
			gridBagConstraints12.gridwidth = 2;
			gridBagConstraints12.anchor = java.awt.GridBagConstraints.CENTER;
			gridBagConstraints12.insets = new java.awt.Insets(5, 2, 5, 0);
			gridBagConstraints12.gridx = 2;
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints(1,
					6, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
					GridBagConstraints.BOTH, new Insets(5, 0, 5, 0), 0, 0);
			gridBagConstraints11.gridy = 5;
			gridBagConstraints11.gridwidth = 2;
			gridBagConstraints11.anchor = java.awt.GridBagConstraints.CENTER;
			gridBagConstraints11.insets = new java.awt.Insets(5, 2, 5, 0);
			gridBagConstraints11.gridx = 2;
			GridBagConstraints gridBagConstraints10 = new GridBagConstraints(1,
					5, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
					GridBagConstraints.BOTH, new Insets(5, 0, 5, 0), 0, 0);
			gridBagConstraints10.gridy = 4;
			gridBagConstraints10.gridwidth = 2;
			gridBagConstraints10.anchor = java.awt.GridBagConstraints.CENTER;
			gridBagConstraints10.insets = new java.awt.Insets(5, 2, 5, 0);
			gridBagConstraints10.gridx = 2;
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints(1,
					3, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
					GridBagConstraints.BOTH, new Insets(5, 0, 5, 0), 0, 0);
			gridBagConstraints9.gridy = 3;
			gridBagConstraints9.gridwidth = 2;
			gridBagConstraints9.anchor = java.awt.GridBagConstraints.CENTER;
			gridBagConstraints9.insets = new java.awt.Insets(5, 2, 5, 0);
			gridBagConstraints9.gridx = 2;
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints(1,
					2, 1, 1, 0.0, 0.0, GridBagConstraints.NORTH,
					GridBagConstraints.BOTH, new Insets(5, 0, 5, 0), 0, 0);
			gridBagConstraints8.gridy = 2;
			gridBagConstraints8.gridwidth = 2;
			gridBagConstraints8.anchor = java.awt.GridBagConstraints.CENTER;
			gridBagConstraints8.insets = new java.awt.Insets(5, 2, 5, 0);
			gridBagConstraints8.gridx = 2;
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints(0,
					9, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
					GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0);
			gridBagConstraints7.gridy = 8;
			gridBagConstraints7.anchor = java.awt.GridBagConstraints.EAST;
			gridBagConstraints7.insets = new java.awt.Insets(0, 0, 0, 3);
			gridBagConstraints7.gridx = 0;
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints(0,
					8, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
					GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0);
			gridBagConstraints6.gridy = 7;
			gridBagConstraints6.anchor = java.awt.GridBagConstraints.EAST;
			gridBagConstraints6.insets = new java.awt.Insets(0, 0, 0, 3);
			gridBagConstraints6.gridx = 0;
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints(0,
					7, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
					GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0);
			gridBagConstraints5.gridy = 6;
			gridBagConstraints5.anchor = java.awt.GridBagConstraints.EAST;
			gridBagConstraints5.insets = new java.awt.Insets(0, 0, 0, 3);
			gridBagConstraints5.gridx = 0;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints(0,
					6, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
					GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0);
			gridBagConstraints4.gridy = 5;
			gridBagConstraints4.anchor = java.awt.GridBagConstraints.EAST;
			gridBagConstraints4.insets = new java.awt.Insets(0, 0, 0, 3);
			gridBagConstraints4.gridx = 0;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints(0,
					5, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
					GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0);
			gridBagConstraints3.gridy = 4;
			gridBagConstraints3.anchor = java.awt.GridBagConstraints.EAST;
			gridBagConstraints3.insets = new java.awt.Insets(0, 0, 0, 3);
			gridBagConstraints3.gridx = 0;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints(0,
					3, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
					GridBagConstraints.NONE, new Insets(5, 0, 5, 0), 0, 0);
			gridBagConstraints2.gridy = 3;
			gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
			gridBagConstraints2.insets = new java.awt.Insets(5, 0, 5, 3);
			gridBagConstraints2.gridx = 0;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints(0,
					2, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
					GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0);
			gridBagConstraints1.gridy = 2;
			gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
			gridBagConstraints1.insets = new java.awt.Insets(0, 0, 0, 3);
			gridBagConstraints1.gridx = 0;
			jPanelData = new JPanel();
			jPanelData.setLayout(new GridBagLayout());

			jLabelColumnDelimiter = new JLabel();
			jLabelColumnDelimiter.setText(ConstColumnDelimiter.getEnumName());
			jLabelColumnDelimiter
					.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN,
							12));
			jLabel3 = new JLabel();
			jLabel3.setHorizontalAlignment(SwingConstants.CENTER);
			jLabel3.setHorizontalTextPosition(SwingConstants.CENTER);
			jLabel3
					.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN,
							12));
			jLabel3.setText("ファイル名");
			jLabelLineSeparator = new JLabel();
			jLabelLineSeparator.setText(ConstLineSeparator.getEnumName());
			jLabelLineSeparator
					.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN,
							12));
			jLabel5 = new JLabel();
			jLabel5.setToolTipText("");
			jLabel5
					.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN,
							12));
			jLabel5.setText("CSVファイルの日本語コード");
			jLabel6 = new JLabel();
			jLabel6.setText("項目見出し");
			jLabel6
					.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN,
							12));
			jLabel7 = new JLabel();
			jLabel7.setText("重複データの扱い");
			jLabel7
					.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN,
							12));
			jLabel9 = new JLabel();
			jLabel9.setText("対象テーブル");
			jLabel9
					.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN,
							12));
			jLabel2 = new JLabel();
			jLabel2.setText("COMMIT方法");

			jLabel2
					.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN,
							12));
			jPanelData.add(getJComboBoxDelimiter(), gridBagConstraints9);
			jPanelData.add(jLabelColumnDelimiter, gridBagConstraints2);
			jPanelData.add(jLabel3, gridBagConstraints15);
			jPanelData.add(jLabelLineSeparator, gridBagConstraints3);
			jPanelData.add(getJComboBoxCrLf(), gridBagConstraints10);
			jPanelData.add(jLabel5, gridBagConstraints4);
			jPanelData.add(getJComboBoxStringCode(), gridBagConstraints11);
			jPanelData.add(jLabel6, gridBagConstraints5);
			jPanelData.add(getJComboBoxOutputHeader(), gridBagConstraints12);
			jPanelData.add(jLabel7, gridBagConstraints6);
			jPanelData.add(getJComboBoxDuplicateKey(), gridBagConstraints13);
			jPanelData.add(getJComboBoxTableName(), gridBagConstraints8);
			jPanelData.add(jLabel9, gridBagConstraints1);
			jPanelData.add(getJComboBoxCommitTiming(), gridBagConstraints14);
			jPanelData.add(jLabel2, gridBagConstraints7);
			jPanelData.add(getJScrollPane(), gridBagConstraints22);
			jPanelData.add(getJPanel(), gridBagConstraints16);
		}
		return jPanelData;
	}

	private DefaultComboBoxModel getTableLists() {
		try {
			if (!SqlExec.isConnect()) {
				JDialogMessage.notConnectErrorDialog();
				return modelTableName;
			}
			for (String schema : SqlExec.getSchemaNames()) {
				for (SchemaTableName table : SqlExec
						.getCompleteTableNames(schema)) {
					modelTableName.addElement(table.getCompleteTableName());
				}
			}
		} catch (SQLException e) {
			JDialogMessage.errorDialog(e, "データベース情報取得");
			return modelTableName;
		}
		String tbl = csvInfo.getPropTableName().getCompleteTableName();
		if (!tbl.equals("")) {
			if (modelTableName.getIndexOf(tbl) < 0) {
				modelTableName.addElement(tbl);
			}
		}
		return modelTableName;
	}

	private JPanel getJPanelButton() {
		if (jPanelButton == null) {
			jPanelButton = new JPanel();
			jPanelButton.add(getJButtonOK(), null);
			jPanelButton.add(getJButtonCancel(), null);
		}
		return jPanelButton;
	}

	private JButton getJButtonOK() {
		if (jButtonOK == null) {
			jButtonOK = new JButton();
			jButtonOK.setText("実行");
			jButtonOK.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					actionPerformedOK(e);
				}
			});
		}
		return jButtonOK;
	}

	private JButton getJButtonCancel() {
		jButtonCancel = new JButton();
		jButtonCancel.setText("取消");
		jButtonCancel.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				actionPerformedCancel(e);
			}
		});
		return jButtonCancel;
	}

	/**
	 * This method initializes jButton
	 * 
	 * @return javax.swing.JButton
	 */
	class JButtonChangeFileNameFromTable extends JButton {
		private static final long serialVersionUID = 7702261660963423442L;

		public JButtonChangeFileNameFromTable() {
			super();
			ToolTipManager.sharedInstance().registerComponent(this);
		}

		@Override
		public String getToolTipText() {
			String newStr = getFileNameFromTable(jTextAreaFileName.getText());
			return "<html>ファイル名を 『テーブル名.csv』 に変更します。<br>"
					+ "設定されるファイル名は<br><STRONG><CODE>"
					+ newStr
					+ "</CODE></STRONG><BR>になります。"
					+ (jTextAreaFileName.getText().equals(newStr) ? "あっ！変更前後は同じ内容です。"
							: "") + "</html>";
		}
	}

	private JButton getJButtonChangeFileNameFromTable() {
		if (jButtonChangeFileNameFromTable == null) {
			jButtonChangeFileNameFromTable = new JButtonChangeFileNameFromTable();
			jButtonChangeFileNameFromTable.setText("テーブル名から");
			jButtonChangeFileNameFromTable
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							jTextAreaFileName
									.setText(getFileNameFromTable(jTextAreaFileName
											.getText()));
						}
					});
		}
		return jButtonChangeFileNameFromTable;
	}

	private String getFileNameFromTable(String str) {
		int idx = str.lastIndexOf(System.getProperty("file.separator"));
		if (idx == -1) {
			idx = str.length();
		}
		return str.subSequence(0, idx) + System.getProperty("file.separator")
				+ modelTableName.getSelectedItem() + ".csv";
	}

	/**
	 * This method initializes jPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel();
			jPanel.setLayout(new BoxLayout(getJPanel(), BoxLayout.Y_AXIS));
			jPanel.add(getJButtonFileChoice(), null);
			jPanel.add(getJButtonChangeFileNameFromTable(), null);
		}
		return jPanel;
	}

} // @jve:decl-index=0:visual-constraint="10,10"
