package jdbcacsess.dbconnect;

import java.io.IOException;

import jdbcacsess.InfoListSetting;
import jdbcacsess.dbconnect.DBMSinfo.DBMSinfoTemplate;

import org.w3c.dom.Element;
import org.xml.sax.SAXException;

/**
 * <p>
 * タイトル: 接続情報クラス
 * </p>
 * <p>
 * 説明: データベース接続に使用するユーザ設定情報をオブジェクトとして管理する。
 * また、プロパティにファイルにパラメータとして格納する形式との相互変換も提供する。
 * </p>
 * <p>
 * 著作権: Copyright (c) 2002
 * </p>
 * <p>
 * 会社名:
 * </p>
 * 
 * @author 未入力
 * @version 1.0
 */

public class ConnectInfo extends InfoListSetting<ConnectInfo> {
	/**
	 * ユーザ設定情報名称
	 */
	private String profName = "";

	/**
	 * データベース製品名
	 */
	private DBMSinfoTemplate dbmsInfoTemplate = new DBMSinfoTemplate();

	/**
	 * ホスト名
	 */
	private String hostName = "";

	/**
	 * ポート番号
	 */
	private String portNumber = "";

	/**
	 * データベース名
	 */
	private String dataBaseName = "";

	/**
	 * ユーザＩＤ
	 */
	private String userID = "";

	/**
	 * パスワード
	 */
	private String password = "";

	/**
	 * パスワード保存有無
	 */
	private String savePassword = "";

	/**
	 * 付加文字列
	 */
	private String optionString = "";

	/**
	 * ドライバクラス名
	 */
	private String classPathTextInput = "";

	/**
	 * ドライバパス名
	 */
	private String driverPath = "";

	/**
	 * デバック用に、各接続情報フィールドを文字列表現にして返却する。
	 */
	@Override
	public String toString() {
		String str;
		str = "DBMS=" + getDBMS() + ";hostName=" + getHostName()
				+ ";portNumber=" + getPortNumber() + ";dataBaseName="
				+ getDataBaseName() + ";optionString=" + getOptionString()
				+ ";userID=" + getUserID() + ";password=" + getPassword()
				+ ";DriverPath=" + getDriverPath();
		return str;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.InfoListSetting#getKeyValue()
	 */
	@Override
	protected String getKeyValue() {
		return getProfName();
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.InfoListSetting#setKeyValue(java.lang.String)
	 */
	@Override
	protected void setKeyValue(String key) {
		setProfName(key);
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.InfoListSetting#getBaseTagName()
	 */
	@Override
	protected String getBaseTagName() {
		return "connectinfo";
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.InfoListSetting#getInstance()
	 */
	@Override
	protected ConnectInfo getInstance() {
		return new ConnectInfo();
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.InfoListSetting#setFromElement(org.w3c.dom.Element)
	 */
	@Override
	public void setFromElement(Element node) {
		setDBMS(node.getAttribute("DBMS"));
		setHostName(node.getAttribute("HostName"));
		setPortNumber(node.getAttribute("PortNumber"));
		setDataBaseName(node.getAttribute("DataBaseName"));
		setOptionString(node.getAttribute("OptionString"));
		setUserID(node.getAttribute("UserID"));
		setPassword(node.getAttribute("Password"));
		setClassPathTextInput(node.getAttribute("ClassPathTextInput"));
		setDriverPath(node.getAttribute("DriverPath"));
		setSavePassword(node.getAttribute("SavePassword"));
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.InfoListSetting#addToElement(org.w3c.dom.Element)
	 */
	@Override
	public void addToElement(Element node) {
		node.setAttribute("DBMS", getDBMS());
		node.setAttribute("HostName", getHostName());
		node.setAttribute("PortNumber", getPortNumber());
		node.setAttribute("DataBaseName", getDataBaseName());
		node.setAttribute("OptionString", getOptionString());
		node.setAttribute("UserID", getUserID());
		node.setAttribute("Password", getPassword());
		node.setAttribute("ClassPathTextInput", getClassPathTextInput());
		node.setAttribute("DriverPath", getDriverPath());
		node.setAttribute("SavePassword", getSavePassword());
	}

	/**
	 * @param profName
	 *            設定する profName。
	 */
	public void setProfName(String profName) {
		this.profName = profName;
	}

	/**
	 * @return profName を戻します。
	 */
	public String getProfName() {
		return profName;
	}

	/**
	 * @param dBMS
	 *            設定する dBMS。
	 * @throws IOException
	 * @throws SAXException
	 */
	public void setDBMS(String dBMS) {
		try {
			dbmsInfoTemplate = new DBMSinfo().getDBMSname(dBMS);
			
		} catch (SAXException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * @return dBMS を戻します。
	 */
	public String getDBMS() {
		return dbmsInfoTemplate.name;
	}

	/**
	 * @param hostName
	 *            設定する hostName。
	 */
	public void setHostName(String hostName) {
		this.hostName = hostName;
	}

	/**
	 * @return hostName を戻します。
	 */
	public String getHostName() {
		return hostName;
	}

	/**
	 * @param portNumber
	 *            設定する portNumber。
	 */
	public void setPortNumber(String portNumber) {
		this.portNumber = portNumber;
	}

	/**
	 * @return portNumber を戻します。
	 */
	public String getPortNumber() {
		return portNumber;
	}

	/**
	 * @param dataBaseName
	 *            設定する dataBaseName。
	 */
	public void setDataBaseName(String dataBaseName) {
		this.dataBaseName = dataBaseName;
	}

	/**
	 * @return dataBaseName を戻します。
	 */
	public String getDataBaseName() {
		return dataBaseName;
	}

	/**
	 * @param userID
	 *            設定する userID。
	 */
	public void setUserID(String userID) {
		this.userID = userID;
	}

	/**
	 * @return userID を戻します。
	 */
	public String getUserID() {
		return userID;
	}

	/**
	 * @param password
	 *            設定する password。
	 */
	public void setPassword(String password) {
		this.password = password;
	}

	/**
	 * @return password を戻します。
	 */
	public String getPassword() {
		return password;
	}

	/**
	 * @return optionString を戻します。
	 */
	public String getOptionString() {
		return optionString;
	}

	/**
	 * @param optionString
	 *            設定する optionString。
	 */
	public void setOptionString(String optionString) {
		this.optionString = optionString;
	}

	/**
	 * @return
	 */
	public String getUrl() {
		String url = dbmsInfoTemplate.url;

		url = replaceString(url, "%HostName%", getHostName());
		url = replaceString(url, "%DataBaseName%", getDataBaseName());
		url = replaceString(url, "%PortNumber%",
				("".equals(getPortNumber()) ? "" : ":") + getPortNumber());
		url = replaceString(url, "%PortNumberRAW%", getPortNumber());
		url = url + getOptionString();
		return url;
	}

	/**
	 * @param org
	 * @param b
	 * @param a
	 * @return
	 */
	private String replaceString(String org, String b, String a) {
		StringBuffer newString = new StringBuffer(org);
		int start = org.indexOf(b);
		if (start == -1) {
			return org;
		}
		newString.replace(start, start + b.length(), a);
		return newString.toString();
	}

	/**
	 * @return
	 */
	public String getClassPath() {
		return dbmsInfoTemplate.classPath;
	}

	/**
	 * @return className を戻します。
	 */
	public String getClassPathTextInput() {
		return classPathTextInput;
	}

	/**
	 * @param className
	 *            設定する className。
	 */
	public void setClassPathTextInput(String className) {
		this.classPathTextInput = className;
	}

	/**
	 * @return diverPath を戻します。
	 */
	public String getDriverPath() {
		return driverPath;
	}

	/**
	 * @param diverPath
	 *            設定する diverPath。
	 */
	public void setDriverPath(String diverPath) {
		this.driverPath = diverPath;
	}

	/**
	 * @return savePassword を戻します。
	 */
	public String getSavePassword() {
		return savePassword;
	}

	/**
	 * @param savePassword
	 *            設定する savePassword。
	 */
	public void setSavePassword(String savePassword) {
		this.savePassword = savePassword;
	}

}
