package jdbcacsess.dbconnect;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * @author sima DBMS毎の接続方法を設定ファイルから読み込み、全体の一覧やそれぞれの値を提供する。
 *         また、ConnectionInfoで持つパラメータから接続URLを編集する。
 * 
 * なお、DBMSinfoTemplate.urlで返される文字列には、URLのテンプレートのテンプレートだけなので、
 * ホスト名やデータベース名等の可変パラメータは埋め込まれていません。
 * また、可変文字列部分(%...%)の書き換え機能は提供しません
 */
public class DBMSinfo {
	static class DBMSinfoTemplate{
		String name="";
		String classPath="";
		String url="";
	}

	/**
	 * @return DBMS一覧
	 * @throws SAXException
	 * @throws IOException
	 */
	public ArrayList<String> getDBMSList() throws SAXException,
			IOException {
		XMLReader parser = XMLReaderFactory.createXMLReader();

		DBMSlisSAXHandler handler = new DBMSlisSAXHandler();
		parser.setContentHandler(handler);

		InputStream fs = DBMSinfo.class.getResourceAsStream("DBMS.xml");
		InputSource in = new InputSource(new InputStreamReader(fs));
		parser.parse(in);

		return handler.getDBMSnames();
	}

	/**
	 * @param sname
	 *            設定する dBMSname。
	 * @throws SAXException
	 * @throws IOException
	 */
	public DBMSinfoTemplate getDBMSname(String sname) throws SAXException, IOException {
		
		if (sname == null) {
			return new DBMSinfoTemplate();
		}
		if (sname.equals("")) {
			return new DBMSinfoTemplate();
		}

		DBMSinfoTemplate rtn = new DBMSinfoTemplate();
		rtn.name=sname;

		XMLReader parser = XMLReaderFactory.createXMLReader();

		DBMSkeySAXHandler handler = new DBMSkeySAXHandler(sname);

		parser.setContentHandler(handler);

		InputStream fs = this.getClass().getResourceAsStream("DBMS.xml");
		InputSource in = new InputSource(new InputStreamReader(fs));
		parser.parse(in);

		rtn.url = "jdbc"
				+ ("".equals(handler.getSubprotocol()) ? "" : ":"
						+ handler.getSubprotocol())
				+ ("".equals(handler.getSubname()) ? "" : ":"
						+ handler.getSubname());
		rtn.classPath = handler.getDriver();
		return rtn;
	}

	/**
	 * @author sima DBMS一覧を読み取るSAXハンドラ
	 */
	static class DBMSlisSAXHandler extends DefaultHandler {
		ArrayList<String> DBMSnames;

		ArrayList<String> getDBMSnames() {
			return DBMSnames;
		}

		@Override
		public void startDocument() {
			DBMSnames = new ArrayList<String>();
		}

		// 要素の開始通知
		@Override
		public void startElement(String namespaceURI, String localName,
				String qName, Attributes attributes) {

			if (qName.equals("DBMS")) {
				for (int i = 0; i < attributes.getLength(); i++) {
					String attrQName = attributes.getQName(i);// 属性の前置修飾子付きの修飾名を取得
					String attrValue = attributes.getValue(i);// 属性の値を取得
					if (attrQName.equals("name")) {
						DBMSnames.add(attrValue);
					}
				}
			}
		}
	}

	/**
	 * @author sima DBMSに対応する各パラメータを読み取るSAXハンドラ
	 */
	static class DBMSkeySAXHandler extends DefaultHandler {
		private String DBMSname;

		private StringBuffer sb;

		private boolean searchHit;

		private String subprotocol;

		private String subname;

		private String driver;

		public DBMSkeySAXHandler(String str) {
			this.DBMSname = str;
		}

		// 要素の開始通知
		@Override
		public void startElement(String namespaceURI, String localName,
				String qName, Attributes attributes) {

			sb = new StringBuffer();

			if (!qName.equals("DBMS")) {
				return;
			}
			for (int i = 0; i < attributes.getLength(); i++) {
				String attrQName = attributes.getQName(i);// 属性の前置修飾子付きの修飾名を取得
				String attrValue = attributes.getValue(i);// 属性の値を取得
				if (attrQName.equals("name")) {
					if (attrValue.equals(DBMSname)) {
						searchHit = true;
					} else {
						searchHit = false;
					}
				}
			}
		}

		/**
		 * テキストデータ読み込み時
		 */
		@Override
		public void characters(char[] ch, int offset, int length) {
			sb.append(ch, offset, length);
		}

		/**
		 * 要素の終了タグ読み込み時
		 */
		@Override
		public void endElement(String uri, String localName, String qName) {
			if (!searchHit) {
				return;
			}
			if (qName.equals("subprotocol")) {
				subprotocol = sb.toString();
			}
			if (qName.equals("subname")) {
				subname = sb.toString();
			}
			if (qName.equals("driver")) {
				driver = sb.toString();
			}

		}

		String getDriver() {
			return driver;
		}

		String getSubname() {
			return subname;
		}

		String getSubprotocol() {
			return subprotocol;
		}
	}

}
