/**
 * 
 */
package jdbcacsess.gui;

import java.awt.Component;

import javax.swing.DefaultCellEditor;
import javax.swing.JTable;
import javax.swing.SwingConstants;

import jdbcacsess.JTextFieldEdit;
import jdbcacsess.gui.common.JDialogMessage;
import jdbcacsess.sql.ColumnInfoManageble;
import jdbcacsess.sql.ColumnInfoResult;

/**
 * textfieldを使用してセルデータの編集を行う。<br>
 * 編集後のデータはColumnInfoResultで指定されているコンストラクタを使用してオブジェクト化される。
 * コンストラクタとして使用されるのは、単一のStringを引数とするものをだけである。<br>
 * textfield のdata => xxxxx(String data) という流れ。<br>
 * <br>
 * 作成されたオブジェクトはそのままテーブルモデルで保持され、更新時にもsetObjectへそのまま引きわたされる。
 * 
 * @author sima
 * 
 */
public class CellEditorSqlTable extends DefaultCellEditor {

	private static final long serialVersionUID = -4689064363177695401L;

	public CellEditorSqlTable() {
		super(new JTextFieldEdit(true));
	}

	transient java.lang.reflect.Constructor<?> constructor;

	Object value;

	/* (non-Javadoc)
	 * @see javax.swing.DefaultCellEditor#stopCellEditing()
	 */
	@Override
	public boolean stopCellEditing() {
		
		String s = (String) super.getCellEditorValue();

		if (constructor.getDeclaringClass() == String.class) {
			// コンストラクタがSringなので、新たにインスタンス生成する必要は無い
			value = s;
		}else{
			try {
				// 単一のStringを引数とするコンストラクタを使用してインスタンス生成
				value = constructor.newInstance(new Object[] { s });
			} catch (Exception e) {
				JDialogMessage.errorDialog(e);
				return false;
			}
		}
		return super.stopCellEditing();
	}

	/* (non-Javadoc)
	 * @see javax.swing.DefaultCellEditor#getTableCellEditorComponent(javax.swing.JTable, java.lang.Object, boolean, int, int)
	 */
	@Override
	public Component getTableCellEditorComponent(JTable table, Object editStartValue,
			boolean isSelected, int row, int column) {
		
		this.value = null;
		
		// 編集終了後に使用するコンストラクタを準備する
		ColumnInfoResult cim = ((ColumnInfoManageble)table.getModel()).getColumnInfo(row, column);
		constructor = cim.getConstructor();
		
		
		// これから更新するデータのカラムクラス名から、右詰め・左詰めを設定する
		// データのオブジェクト判定では、NULLに対応できなくなる
		JTextFieldEdit jtfe = (JTextFieldEdit)getComponent();
		if (java.lang.Number.class.isAssignableFrom(cim.getColumnClass())) {
			jtfe.setHorizontalAlignment(SwingConstants.RIGHT);
		} else {
			jtfe.setHorizontalAlignment(SwingConstants.LEFT);
		}

		return super.getTableCellEditorComponent(table, editStartValue, isSelected, row, column);
	}

	//自前の編集後オブジェクト返すようにする
	@Override
	public Object getCellEditorValue() {
		return value;
	}

}
