/**
 *
 */
package jdbcacsess.gui;

import jdbcacsess.JdbcacsessException;
import jdbcacsess.sql.column.Binary;
import jdbcacsess.sql.column.DenyEditInCell;
import jdbcacsess.sql.column.GetColumnArray;
import jdbcacsess.sql.column.GetColumnBinary;

/**
 * @author sima 検索結果テーブルのセル値と更新状態を保持する
 */
class CellInfo {
	private boolean cellEditable = true;

	private CellMode mode = CellMode.CELL_NON;

	private Object value = null;

	private boolean defaultInsert = false;

	public CellInfo copy() {
		CellInfo cellInfo = new CellInfo();
		cellInfo.value = this.value;
		if (value instanceof GetColumnBinary) {
			if (!((GetColumnBinary) value).isFullData()) {
				cellInfo.value = new Binary();
			}
		}
		cellInfo.cellEditable = true;
		cellInfo.mode = CellMode.CELL_INSERT;
		cellInfo.defaultInsert = this.defaultInsert;
		return cellInfo;
	}

	public CellMode getMode() {
		return mode;
	}

	public Object getValue() {
		return value;
	}

	public boolean isCellEditable() {
		return cellEditable;
	}

	public void setMode(CellMode mode) {
		switch (mode) {
		case CELL_DELETE:
			cellEditable = false;
			break;
		case CELL_DELETE_COMPLETE:
			cellEditable = false;
			break;
		case CELL_DENYEDIT_IN_CELL:
			cellEditable = false;
			break;
		default:
			cellEditable = true;
			break;
		}
		this.mode = mode;
	}

	public void setValue(Object value) {
		this.value = value;

		if (value instanceof DenyEditInCell
				|| value instanceof JdbcacsessException) {
			setMode(CellMode.CELL_DENYEDIT_IN_CELL);
			return;
		}
		if (value instanceof String) {
			String str = (String) value;
			for (int i = 0; i < str.length(); i++) {
				char c = str.charAt(i);
				if (c == '\n' || c == '\r') {
					setMode(CellMode.CELL_DENYEDIT_IN_CELL);
					return;
				}
			}
		}
		if (value instanceof GetColumnBinary) {
			if (!((GetColumnBinary) value).isFullData()) {
				setMode(CellMode.CELL_DENYEDIT_IN_CELL);
				return;
			}
		}
		if (value != null) {
			// Arrayの更新は不可
			if (value instanceof GetColumnArray) {
				setMode(CellMode.CELL_DENYEDIT_IN_CELL);
				return;
			} else {
				// toString()が実装されているクラスを探し、java.lang.objectであれば編集させない。
				if (toStringMethodNoHaving(value)) {
					setMode(CellMode.CELL_DENYEDIT_IN_CELL);
					return;
				}
			}
		}
	}
	
	private boolean toStringMethodNoHaving(Object value){
		try {
			if (value.getClass().getMethod("toString", (Class[])null).getDeclaringClass() == Object.class) {
				return true;
			}
		} catch (SecurityException e) {
			// TODO 自動生成された catch ブロック
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO 自動生成された catch ブロック
			e.printStackTrace();
		}
		return false;
	}

	/**
	 * @return defaultInsert を戻します。
	 */
	public boolean isDefaultInsert() {
		return defaultInsert;
	}

	/**
	 * @param defaultInsert
	 *            設定する defaultInsert。
	 */
	public void setDefaultInsert(boolean defaultInsert) {
		this.defaultInsert = defaultInsert;
	}

}