package jdbcacsess.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;

import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import jdbcacsess.JdbcacsessException;

public class CellRendererSqlTable extends DefaultTableCellRenderer {

	private static final long serialVersionUID = 7761617446993569948L;

	public static final Font tableFont = JFrameSetting.getGuiFont();
	
	private int autoResizeWidth = JFrameSetting.getAutoResizeWidth();
	
	@Override
	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {

		super.getTableCellRendererComponent(table, value, isSelected, hasFocus,
				row, column);

		if (table.getModel() instanceof JTableSqlModel) {
		} else {
			return this;
		}

		JTableSqlModel sqlTableModel = (JTableSqlModel) table.getModel();
		CellMode mode = sqlTableModel.getMode(row, column);
		if (!isSelected) {
			switch (mode) {
			case CELL_UPDATE:
				setForeground(Color.WHITE);
				setBackground(Color.RED);
				break;
			case CELL_UPDATE_COMPLETE:
				setForeground(Color.RED);
				setBackground(Color.WHITE);
				break;
			case CELL_DELETE:
				setForeground(Color.LIGHT_GRAY);
				setBackground(Color.BLACK);
				break;
			case CELL_DELETE_COMPLETE:
				setForeground(Color.BLACK);
				setBackground(Color.LIGHT_GRAY);
				break;
			case CELL_INSERT:
				setForeground(Color.WHITE);
				setBackground(Color.BLUE);
				break;
			case CELL_INSERT_COMPLETE:
				setForeground(Color.BLUE);
				setBackground(Color.WHITE);
				break;
			case CELL_DENYEDIT_IN_CELL:
				setForeground(Color.MAGENTA);
				setBackground(Color.WHITE);
				break;

			default: // CELL_NON
				setForeground(Color.BLACK);
				setBackground(Color.WHITE);
				break;
			}
		}

		String msg = sqlTableModel.getColumnInfo(column).getColumnComment()
				+ "  ["
				+ sqlTableModel.getColumnInfo(column).getColumnClass()
						.getName() + "]";
		if (mode == CellMode.CELL_DENYEDIT_IN_CELL) {
			if (value instanceof JdbcacsessException) {
				setToolTipText("<html>" + msg
						+ "<br>ResultSetからgetした時例外が発生しました。<br>" + value
						+ "<br>値を変更する時は、一旦『NULLに更新』を行ってください。" + "</html>");
			} else {
				setToolTipText("<html>" + msg
						+ "<br>セル更新出来ません。以下のいずれかの場合です。<ol>"
						+ "<li>バイナリ読込量を超えている：バイナリ読込量を上げて再検索して下さい</li>"
						+ "<li>改行を含んでいるテキスト：テキスト編集画面で更新して下さい</li>"
						+ "<li>『カラム更新(ファイルから)』直後：再検索して下さい</li>"
						+ "<li>toString()を実装してない：ResultSetのgetterを変更するか、独自クラスを用意する</li>"
						+ "</ol></html>");
			}
		} else {
			setToolTipText("<html>" + msg + "</html>");
		}

		// フォントの指定
		setFont(tableFont);

		if (sqlTableModel.getColumnInfo(column).isPrimaryKey()) {
			setFont(getFont().deriveFont(Font.BOLD + Font.ITALIC));
		} else {
			setFont(getFont().deriveFont(Font.PLAIN));
		}

		if (value == null) {
			setValue("#NULL#");
			setFont(getFont().deriveFont(getFont().getStyle(), 10));
			setHorizontalAlignment(CENTER);
		} else if (value instanceof JdbcacsessException) {
			setValue("#ERROR#");
			setFont(getFont().deriveFont(getFont().getStyle(), 10));
			setHorizontalAlignment(CENTER);
		} else if (java.lang.Number.class.isAssignableFrom(value.getClass())) {
			setHorizontalAlignment(RIGHT);
		} else {
			setHorizontalAlignment(LEFT);
		}

		if (sqlTableModel.isDefaultInsert(row, column)) {
			setValue("#DEFAULT#");
			setFont(getFont().deriveFont(getFont().getStyle(), 10));
			setHorizontalAlignment(CENTER);
		}

		// カラム幅の指定
		if ( autoResizeWidth!=0 ) {

			String s = ( autoResizeWidth < getText().length() ? getText().substring(0, autoResizeWidth-1) : getText());
			TableColumn tc = table.getColumnModel().getColumn(column);
			int width = getFontMetrics(getFont()).stringWidth(s) + 4;

			if (tc.getPreferredWidth() < width ) {
				tc.setPreferredWidth(width);
			}
		}

		return this;
	}

}
