package jdbcacsess.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.sql.ParameterMetaData;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import jdbcacsess.JdbcacsessException;
import jdbcacsess.gui.common.ColumnWidth;
import jdbcacsess.gui.common.ComponentProperty;
import jdbcacsess.gui.common.JDialogMessage;
import jdbcacsess.sql.ColumnInfoManageble;
import jdbcacsess.sql.ColumnInfoParameter;
import jdbcacsess.sql.ColumnInfoResult;
import jdbcacsess.sql.PrepareExecute;
import jdbcacsess.sql.SqlMappingInfo;

public class JDialogPrepareParameter extends JDialog {
	static class TableRow extends ColumnInfoParameter{
		Object value;
		TableRow(int columnIndex, 
				String columnName, 
				int columnType,
				String columnTypeName, 
				int nullable, 
				String columnClassName,
				int precision, 
				int scale,
				int parameterMode){
			super(columnIndex, columnName, columnType, columnTypeName, nullable, columnClassName, precision, scale,parameterMode);
		}
		public Object getValue() {
			return value;
		}
		public void setValue(Object value) {
			this.value = value;
		}
	}

	/**
	 * 
	 */
	private static final long serialVersionUID = -5033911003404888825L;

	private JScrollPane jScrollPane = null;

	private JTable jTable = null;

	private JPanel jPanel = null;

	private PreparedTableModel tableModel = null;

	private JPanel jPanel1 = null;

	private JButton jButtonSqlExec = null;

	private JButton jButtonCancel = null;
	
	PrepareExecute prepareExecute;
	
	private boolean executeAns;

	public JDialogPrepareParameter(PrepareExecute prepareExecute) throws JdbcacsessException {
		this.prepareExecute = prepareExecute;

		tableModel = new PreparedTableModel();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
		setAlwaysOnTop(true);
		setModal(true);
		
		this.setSize(new Dimension(300, 200));
		this.setTitle("SQL文のパラメータ入力");

		this.setContentPane(getJPanel());
		jTable.setModel(tableModel);
		jTable.setCellSelectionEnabled(true);
		ColumnWidth.setHeaderWidth(jTable, 2);
		ColumnWidth.setColumnWidth(jTable);
		
		jTable.getColumnModel().getColumn(PreparedTableModel.valueColumnIndex).setCellEditor(new CellEditorSqlTable());
		
		setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
		new ComponentProperty().restoreWindowPosition(this, 300, 200);
		addWindowListener(new java.awt.event.WindowAdapter() {
			@Override
			public void windowClosing(java.awt.event.WindowEvent e) {
				dialogClosing();
			}
		});

		getRootPane().setDefaultButton(jButtonSqlExec);
		executeAns=false;

		
		setVisible(true);
	}

	public void dialogClosing() {
		new ComponentProperty().storeWindowPosition(this);
	}
	private void actionPerformedExecute(ActionEvent e){
		for (TableRow element : tableModel) {
			if (element.isModeInput()){
				prepareExecute.addParm(element.getColumnIndex(), element.getValue(), element);
			}
		}
		executeAns=true;
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getJTable());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jTable
	 * 
	 * @return javax.swing.JTable
	 */
	private JTable getJTable() {
		if (jTable == null) {
			jTable = new JTable();
		}
		return jTable;
	}

	/**
	 * This method initializes jPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel();
			jPanel.setLayout(new BorderLayout());
			jPanel.add(getJScrollPane(), BorderLayout.CENTER);
			jPanel.add(getJPanel1(), BorderLayout.SOUTH);
		}
		return jPanel;
	}

	class PreparedTableModel extends AbstractTableModel implements Iterable<TableRow>,Iterator<TableRow>,ColumnInfoManageble{

		private static final long serialVersionUID = 3811848012149308202L;


		ArrayList<TableRow> rows = new ArrayList<TableRow>();

		PreparedTableModel( ) throws JdbcacsessException {
			SqlMappingInfo smi = new SqlMappingInfo();
			
			try {
				ParameterMetaData pmd = prepareExecute.getPrepared().getParameterMetaData();
				for (int param = 1; param < pmd.getParameterCount() + 1; param++) {
					TableRow row = new TableRow(
							param,
							"Parameter" + Integer.valueOf(param).toString(),
							pmd.getParameterType(param),
							pmd.getParameterTypeName(param),
							pmd.isNullable(param),
							pmd.getParameterClassName(param),
							pmd.getPrecision(param),
							pmd.getScale(param),
							pmd.getParameterMode(param)
							);
					smi.search(row, false);
					rows.add(row);
					
				}
			} catch (Exception e) {
				throw new JdbcacsessException(e);
			}
		}

		public int getColumnCount() {
			return header.length;
		}
		
		@Override
		public String getColumnName(int column){
			return header[column];
		}

		public int getRowCount() {
			return rows.size();
		}

		final  String[] header = { 
				"カラム名", 
				"モード", 
				"型",
				"設定する値" };
		final static int valueColumnIndex = 3;
		
		public Object getValueAt(int rowIndex, int columnIndex) {
			TableRow tb = rows.get(rowIndex);
			switch (columnIndex){
			  case 0:  return tb.getColumnName();
			  case 1:  return tb.getParameterMode();
			  case 2:  return tb.getColumnTypeName();
			  case valueColumnIndex:  return tb.getValue();
			}
			return null;
		}
		
	    @Override
		public boolean isCellEditable(int rowIndex, int columnIndex) {
	    	if (columnIndex==valueColumnIndex && rows.get(rowIndex).isModeInput()){
	    		return true;
	    	}
	    	return false;
        }

		@Override
		public void setValueAt(Object aValue, int row, int column)  {
			super.setValueAt(aValue, row, column);
			if (column==valueColumnIndex){
				rows.get(row).setValue(aValue);
			}
		}
		
		int iterator_idx;
		public Iterator<TableRow> iterator() {
			iterator_idx = 0;
			return this;
		}

		public boolean hasNext() {
			if (rows.size()-1 < iterator_idx ){
				return false;
			}
			return true;
		}

		public TableRow next() {
			TableRow rtn = rows.get(iterator_idx);
			iterator_idx++;
			return rtn;
		}

		public void remove() {
		}

		public ColumnInfoResult getColumnInfo(int row, int column) {
			return rows.get(row);
		}
		
	}

	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			jPanel1 = new JPanel();
			jPanel1.setLayout(new FlowLayout());
			jPanel1.add(getJButtonSqlExec(), null);
			jPanel1.add(getJButtonCancel(), null);
		}
		return jPanel1;
	}

	/**
	 * This method initializes jButtonSqlExec	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonSqlExec() {
		if (jButtonSqlExec == null) {
			jButtonSqlExec = new JButton();
			jButtonSqlExec.setText("SQL実行");
			jButtonSqlExec.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					try {
						actionPerformedExecute(e);
					} catch (Exception e1) {
						JDialogMessage.errorDialog(e1);
						return;
					}

					dialogClosing();
					dispose();
				}
			});
		}
		return jButtonSqlExec;
	}

	/**
	 * This method initializes jButtonCancel	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonCancel() {
		if (jButtonCancel == null) {
			jButtonCancel = new JButton();
			jButtonCancel.setText("取消");
			jButtonCancel.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					dialogClosing();
					dispose();
				}
			});
		}
		return jButtonCancel;
	}

	public boolean isExecuteAns() {
		return executeAns;
	}

}
