package jdbcacsess.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.CubicCurve2D;
import java.awt.geom.Line2D;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Map.Entry;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToggleButton;
import javax.swing.border.Border;
import javax.swing.border.MatteBorder;

import jdbcacsess.PrefixStringBuffer;
import jdbcacsess.SettingFile;
import jdbcacsess.gui.JPanelConditionTable.JoinStatus;
import jdbcacsess.gui.JPanelConditionTable.SelectColumns;
import jdbcacsess.gui.JPanelConditionTable.JPanelConditionColumn.JToggleButtonJoinPointer;
import jdbcacsess.gui.common.JFrameBase;
import jdbcacsess.sql.SchemaTableName;
import jdbcacsess.sql.DBObjectTableColumns.DBObjectTableColumn;

import org.w3c.dom.Element;

public class JFrameCreateSqlFromTree extends JFrameBase {
	@Override
	public String getAttrKey(){ 
		return null;
	}
	@Override
	public String getAttrValue(){ 
		return null;
	}


	class JoinPointerMouseListener implements MouseListener {

		private Border oldBorder;

		private Condition currentCondition;

		private Border activeBorder = new MatteBorder(2, 2, 2, 2, Color.red);

		public void mouseClicked(MouseEvent e) {
			if (!jToggleButtonOuterJoin.isSelected()) {
				return;
			}
			jToggleButtonOuterJoin.setSelected(false);
			JToggleButtonJoinPointer c = (JToggleButtonJoinPointer) e
					.getComponent();
			c.setEnabled(true);

			if (currentCondition != null) {
				if (c.getJoinStatus() == JoinStatus.OUTER_ALL) {
					c.setJoinStatus(JoinStatus.OUTER_NONE);
					currentCondition.getEither(c).setJoinStatus(
							JoinStatus.OUTER_NONE);
				} else {
					setOuterToJoinPointers(c, currentCondition.getEither(c));
				}
				c.setBorder(oldBorder);
			}

			currentCondition = null;
			oldBorder = null;
		}

		public void mouseEntered(MouseEvent e) {
			if (!jToggleButtonOuterJoin.isSelected()) {
				return;
			}
			JToggleButtonJoinPointer c = (JToggleButtonJoinPointer) e
					.getComponent();
			c.setEnabled(false);

			currentCondition = searchConditions(c);
			if (currentCondition != null) {
				oldBorder = c.getBorder();
				c.setBorder(activeBorder);
			}
		}

		public void mouseExited(MouseEvent e) {
			if (!jToggleButtonOuterJoin.isSelected()) {
				return;
			}
			JToggleButtonJoinPointer c = (JToggleButtonJoinPointer) e
					.getComponent();
			c.setEnabled(true);

			if (currentCondition != null) {
				c.setBorder(oldBorder);
				currentCondition = null;
			}
		}

		public void mousePressed(MouseEvent e) {
		}

		public void mouseReleased(MouseEvent e) {
		}

		private Condition searchConditions(JToggleButtonJoinPointer c) {
			for (Condition item : conditions) {
				if (item.getEither(c) != null) {
					return item;
				}
			}
			return null;
		}

		/**
		 * 指定したコンポーネントと同じテーブルの組み合わせに対して、全て外部結合マークを設定
		 * 
		 * @param baseComponent
		 * @param eitherComponent
		 */
		private void setOuterToJoinPointers(
				JToggleButtonJoinPointer baseComponent,
				JToggleButtonJoinPointer eitherComponent) {
			for (Condition pair : new SameTableIterator(baseComponent,
					eitherComponent, false)) {
				pair.equalsJPanelColumns(baseComponent).setJoinStatus(
						JoinStatus.OUTER_ALL);

				JToggleButtonJoinPointer c = pair.getEither(pair
						.equalsJPanelColumns(baseComponent));
				if (c.getJoinStatus() != JoinStatus.OUTER_ALL) {
					c.setJoinStatus(JoinStatus.OUTER_EQUAL);
				}
			}
		}
	}

	/**
	 * カラムを結ぶ線を表示するパネル
	 * 
	 * @author sss
	 * 
	 */
	class ConditionLine extends JPanel {
		private static final long serialVersionUID = -3428287174938981130L;

		private Line2D.Double line = new Line2D.Double();

		@Override
		protected void paintComponent(Graphics g) {
			super.paintComponent(g);
			Graphics2D g2 = (Graphics2D) g;
			if (firsstClickJoinPointer != null && mousePoint != null) {
				line.setLine(firsstClickJoinPointer.getConnecterPoint(),
						mousePoint);
				g2.draw(line);
			}
			for (int i = 0; i < conditions.size(); i++) {
				g2.draw(conditions.get(i).getLine());
			}
		}

	}

	/**
	 * @author sss 同一のテーブルを持つコンポーネントを走査するイテレータ
	 */
	class SameTableIterator implements Iterable<Condition> {
		private Iterator<Condition> iterator;

		private ArrayList<Condition> list = new ArrayList<Condition>();

		public SameTableIterator(JToggleButtonJoinPointer component,
				boolean outerOnly) {
			for (Condition item : conditions) {
				if (item.equalsJPanelColumns(component) != null) {
					if ((outerOnly && item.getStartComponent().getJoinStatus() != JoinStatus.OUTER_NONE)
							|| !outerOnly) {
						list.add(item);
					}
				}
			}
			iterator = list.iterator();
		}

		public SameTableIterator(JToggleButtonJoinPointer component1,
				JToggleButtonJoinPointer component2, boolean outerOnly) {
			for (Condition item : conditions) {
				if (item.equalsTable(component1, component2)) {
					if ((outerOnly && item.getStartComponent().getJoinStatus() != JoinStatus.OUTER_NONE)
							|| !outerOnly) {
						list.add(item);
					}
				}
			}
			iterator = list.iterator();
		}

		public Iterator<Condition> iterator() {
			return iterator;
		}
	}

	/**
	 * @author sima 条件ボタンのアクション
	 */
	class JoinPointerActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			JToggleButtonJoinPointer component = (JToggleButtonJoinPointer) e
					.getSource();
			if (!component.isSelected()) {
				deSelect(component);
			} else {
				select(component);
			}
			repaint();
		}

		/**
		 * 選択解除された時
		 * 
		 * @param component
		 */
		private void deSelect(JToggleButtonJoinPointer component) {
			boolean hit = false;
			for (Condition item : conditions) {
				JToggleButtonJoinPointer eitherComponent = item
						.getEither(component);
				if (eitherComponent != null) {
					// クリックしたコンポーネントが既にペアを持っている
					if (firsstClickJoinPointer == null) {
						// 選択中でない場合
						firsstClickJoinPointer = eitherComponent;
					} else {
						// 選択中の場合
						eitherComponent.setSelected(false);
					}
					item.getStartComponent().setJoinStatus(
							JoinStatus.OUTER_NONE);
					item.getEndComponent().setJoinStatus(JoinStatus.OUTER_NONE);
					conditions.remove(item);
					hit = true;
					break;
				}
			}
			if (!hit) {
				// 相方が見つからないので、起点の解除である
				firsstClickJoinPointer = null;
			}
		}

		/**
		 * 選択された時
		 * 
		 * @param component
		 */
		private void select(JToggleButtonJoinPointer component) {
			if (firsstClickJoinPointer == null) {
				// 起点の保存
				firsstClickJoinPointer = component;
				return;
			}

			if (firsstClickJoinPointer.getJPanelColumns() == component
					.getJPanelColumns()) {
				// 同一テーブルのカラムなので、リセット
				firsstClickJoinPointer.setSelected(false);
				firsstClickJoinPointer = component;
				return;
			}

			// 既存の同じテーブルの組み合わせと同じマークをつける
			for (Condition pair : new SameTableIterator(firsstClickJoinPointer,
					component, false)) {
				JToggleButtonJoinPointer c1 = pair
						.equalsJPanelColumns(firsstClickJoinPointer);
				firsstClickJoinPointer.setJoinStatus(c1.getJoinStatus());
				JToggleButtonJoinPointer c2 = pair
						.equalsJPanelColumns(component);
				component.setJoinStatus(c2.getJoinStatus());
				break;
			}

			// 新たなペアの作成
			Condition item = new Condition(firsstClickJoinPointer, component);
			conditions.add(item);
			firsstClickJoinPointer = null;
		}
	}

	/**
	 * @author sss
	 * 
	 */
	static class Condition implements Comparable<Condition> {
		private JToggleButtonJoinPointer endComponent = null;

		private String key;

		private JToggleButtonJoinPointer startComponent = null;

		private CubicCurve2D.Double cubicCurve = new CubicCurve2D.Double();

		private Line2D.Double line = new Line2D.Double();

		public Condition(JToggleButtonJoinPointer startComponent,
				JToggleButtonJoinPointer endComponent) {

			String startKey = keyTo(startComponent);
			String endKey = keyTo(endComponent);

			// 昇順になるようにインスタンスへ保存
			if (startKey.compareTo(endKey) < 0) {
				this.startComponent = startComponent;
				this.endComponent = endComponent;
				key = startKey;
			} else {
				this.startComponent = endComponent;
				this.endComponent = startComponent;
				key = endKey;
			}
		}

		private String column(JToggleButtonJoinPointer c) {
			return c.getJPanelColumns().getTableAlias() + "."
					+ c.getColumnInfoTable().getColumnName();
		}

		public int compareTo(Condition o) {
			return this.key.compareTo(o.key);
		}

		/**
		 * 指定されたコンポーネントのテーブルが、開始・終了のいずれかのテーブルと等しい時に、そのコンポーネントを返却します。
		 * 
		 * @param c
		 * @return
		 */
		public JToggleButtonJoinPointer equalsJPanelColumns(
				JToggleButtonJoinPointer c) {

			if (startComponent.getJPanelColumns() == c.getJPanelColumns()) {
				return startComponent;
			}
			if (endComponent.getJPanelColumns() == c.getJPanelColumns()) {
				return endComponent;
			}
			return null;
		}

		/**
		 * @param c1
		 * @param c2
		 * @return
		 */
		public boolean equalsTable(JToggleButtonJoinPointer c1,
				JToggleButtonJoinPointer c2) {
			JToggleButtonJoinPointer p1 = equalsJPanelColumns(c1);

			if (p1 != null) {
				JToggleButtonJoinPointer p2 = equalsJPanelColumns(c2);
				if (p2 != null) {
					return true;
				}
			}
			return false;
		}

		public String getCondition() {
			return column(startComponent) + " = " + column(endComponent);
		}

		/**
		 * 指定されたコンポーネントの反対側のコンポーネントを取得します。 開始・終了のどちらでも無い場合は、nullを返却します。
		 * 
		 * @param c
		 * @return
		 */
		public JToggleButtonJoinPointer getEither(JToggleButtonJoinPointer c) {
			if (startComponent == c) {
				return endComponent;
			}
			if (endComponent == c) {
				return startComponent;
			}
			return null;
		}

		public JToggleButtonJoinPointer getEndComponent() {
			return endComponent;
		}

		public Shape getLine() {
			Point startPoint = startComponent.getConnecterPoint();
			Point endPoint = endComponent.getConnecterPoint();
			if (startPoint.y == endPoint.y) {
				int ctlx = (startPoint.x + (endPoint.x - startPoint.x) / 2);
				int ctly = (endPoint.y + (endPoint.x - startPoint.x) / 10);
				Point ctlPoint = new Point(ctlx, ctly);
				cubicCurve.setCurve(startPoint, ctlPoint, ctlPoint, endPoint);
				return cubicCurve;
			}
			line.setLine(startPoint, endPoint);
			return line;
		}

		public void getOuterTable(PrefixStringBuffer buf, boolean first,
				JToggleButtonJoinPointer c) {
			if (startComponent.getJoinStatus() == JoinStatus.OUTER_NONE
					|| endComponent.getJoinStatus() == JoinStatus.OUTER_NONE) {
				System.out.print("異常です");
				return;
			}

			if (first) {
				buf.append(table(c));
			} else {
				buf.insert(0, "(");
				buf.append(")");
			}
			buf.append(" " + tableEdit() + " JOIN " + table(getEither(c))
					+ " ON");
			return;
		}

		public JToggleButtonJoinPointer getStartComponent() {
			return startComponent;
		}

		public boolean isContain(JToggleButtonJoinPointer c) {
			if (startComponent == c) {
				return true;
			}
			if (endComponent == c) {
				return true;
			}
			return false;
		}

		private String keyTo(JToggleButtonJoinPointer c) {
			return c.getJPanelColumns().getSchemaTableName()
					.getCompleteTableName()
					+ new DecimalFormat("000").format(c.getColumnInfoTable()
							.getOrdinalPosition());
		}

		private String table(JToggleButtonJoinPointer c) {
			return c.getJPanelColumns().getSchemaTableName()
					.getCompleteTableName()
					+ " " + c.getJPanelColumns().getTableAlias();
		}

		private String tableEdit() {
			String joinKey;
			if (startComponent.getJoinStatus() == JoinStatus.OUTER_EQUAL) {
				joinKey = "RIGHT";
			} else if (endComponent.getJoinStatus() == JoinStatus.OUTER_EQUAL) {
				joinKey = "LEFT";
			} else {
				joinKey = "FULL";
			}
			return joinKey;
		}

		@Override
		public String toString() {
			return "Condition[" + "start=" + keyTo(startComponent) + " "
					+ "end=" + keyTo(endComponent) + "]";
		}
	}

	private static final long serialVersionUID = 1L;

	private MouseMotionListener componentMouseMotionListener;

	private JToggleButtonJoinPointer firsstClickJoinPointer = null;

	private JButton jButtonOk = null;

	private JPanel jContentPane = null;

	private MouseListener joinPointerMouseListener; // @jve:decl-index=0:

	private JPanel jPanel = null;

	private JPanel jpanelWhereColumns = null;

	private JScrollPane jScrollPane = null;

	private JToggleButton jToggleButtonOuterJoin = null;

	private Point mousePoint = new Point();

	private JTextPaneSqlStatement sqlStatement;

	private ArrayList<JPanelConditionTable> tableInfos = new ArrayList<JPanelConditionTable>(); // @jve:decl-index=0:

	private ActionListener whereActionListener;

	private ArrayList<Condition> conditions = new ArrayList<Condition>(); // @jve:decl-index=0:

	/**
	 * This is the default constructor
	 */
	public JFrameCreateSqlFromTree() {
		super();
		initialize();
	}

	/**
	 * 
	 */
	protected void actionPerformedOk() {
		Collections.sort(conditions);

		// System.out.println("##debug###ソート後:conditions");
		// for (Condition item : conditions) {
		// System.out.println(item.toString());
		// }

		PrefixStringBuffer psbSelectColumn = new PrefixStringBuffer("", " ,");
		PrefixStringBuffer psbFrom = new PrefixStringBuffer("", " ,");
		PrefixStringBuffer psbWhere = new PrefixStringBuffer("", " AND ");

		ArrayList<SchemaTableName> stnList = new ArrayList<SchemaTableName>();
		for (Condition item : conditions) {
			if (item.getStartComponent().getJoinStatus() == JoinStatus.OUTER_NONE) {
				// 外部結合指定無いので、WHEREに編集
				psbWhere.appendItem(item.getCondition());
			} else {
				// 外部結合条件作成
				// 一度テーブル名を記述したら、もうやらない為の設定済テーブルリストを管理する
				SchemaTableName stn1 = item.getStartComponent()
						.getJPanelColumns().getSchemaTableName();
				SchemaTableName stn2 = item.getEndComponent()
						.getJPanelColumns().getSchemaTableName();
				if (stnList.contains(stn1) && stnList.contains(stn2)) {
					System.out.println("編集済:" + item.toString());
					continue;
				}
				if (!stnList.contains(stn1) && !stnList.contains(stn2)) {
					// 記述済に設定
					stnList.add(stn1);
					stnList.add(stn2);

					psbFrom.appendItem("");
					// OUTER JOINのFROM句を編集
					item.getOuterTable(psbFrom, true, item.getStartComponent());

				}
				if (stnList.contains(stn1) && !stnList.contains(stn2)) {
					// 記述済に設定
					stnList.add(stn2);

					// OUTER JOINのFROM句を編集
					item
							.getOuterTable(psbFrom, false, item
									.getStartComponent());
				}
				if (!stnList.contains(stn1) && stnList.contains(stn2)) {
					// 記述済に設定
					stnList.add(stn1);

					// OUTER JOINのFROM句を編集
					item.getOuterTable(psbFrom, false, item.getEndComponent());
				}
				// 同一テーブルの条件ペアを全て、編集する
				PrefixStringBuffer psb = new PrefixStringBuffer("", " AND ");
				for (Condition pair : new SameTableIterator(item
						.getStartComponent(), item.getEndComponent(), true)) {
					psb.appendItem(pair.getCondition());
				}
				psbFrom.append(" " + psb.getString() + "\n");
			}
		}

		for (Component component : getJPanelWhereColumns().getComponents()) {
			if (!(component instanceof JPanelConditionTable)) {
				continue;
			}

			JPanelConditionTable jPanelConditionTable = (JPanelConditionTable) component;

			SelectColumns sc = jPanelConditionTable.getSelectColumns();
			ArrayList<DBObjectTableColumn> selectColumnlist = sc.columnInfoTables;
			// 選択カラムも無し かつ 条件も無し の場合は何もしない
			if (selectColumnlist.isEmpty()
					&& !jPanelConditionTable.isUseJoinPointer()) {
				continue;
			}

			if (sc.allSelected) {
				psbSelectColumn.appendItem(jPanelConditionTable.getTableAlias()
						+ ".*");
			} else {
				// 選択カラムの編集
				for (DBObjectTableColumn item : selectColumnlist) {
					psbSelectColumn.appendItem(jPanelConditionTable
							.getTableAlias()
							+ "." + item.getColumnName());
				}
				if (selectColumnlist.size() != 0) {
					psbSelectColumn.append("\n");
				}
			}

			// WHEREのみで使用するテーブル名の編集
			if (!stnList.contains(jPanelConditionTable.getSchemaTableName())) {
				psbFrom.appendItem(jPanelConditionTable.getSchemaTableName()
						.getCompleteTableName()
						+ " " + jPanelConditionTable.getTableAlias());
			}
		}

		StringBuilder sb = new StringBuilder();
		sb.append("SELECT " + psbSelectColumn.getStringLastLF());

		sb.append("FROM " + psbFrom.getStringLastLF());

		if (psbWhere.length() != 0) {
			sb.append("WHERE " + psbWhere.getStringLastLF());
		}
		sqlStatement.setText(sb.toString());
	}

	/**
	 * 配下の全Swingコンポーネントにマウスリスナーを登録
	 * 
	 * @param jComponent
	 */
	private void addAllMouseMotionListener(JComponent jComponent,
			MouseMotionListener mouseMotionListener) {
		jComponent.addMouseMotionListener(mouseMotionListener);
		for (Component childComponent : jComponent.getComponents()) {
			if (childComponent instanceof JComponent) {
				addAllMouseMotionListener((JComponent) childComponent,
						mouseMotionListener);
			}
		}
	}

	@Override
	public void frameClosing() {
		save();
	}

	/**
	 * This method initializes jButtonOk
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButtonOk() {
		if (jButtonOk == null) {
			jButtonOk = new JButton();
			jButtonOk.setText("SQL作成");
			jButtonOk.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					actionPerformedOk();
					save();
				}
			});
		}
		return jButtonOk;
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJScrollPane(), BorderLayout.CENTER);
			jContentPane.add(getJPanel(), BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel();
			jPanel.setLayout(new FlowLayout());
			jPanel.add(getJToggleButtonOuterJoin(), null);
			jPanel.add(getJButtonOk());
		}
		return jPanel;
	}

	/**
	 * This method initializes jPanel1
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanelWhereColumns() {
		if (jpanelWhereColumns == null) {

			jpanelWhereColumns = new JPanel();
			jpanelWhereColumns.setLayout(new BoxLayout(jpanelWhereColumns,
					BoxLayout.LINE_AXIS));
		}
		return jpanelWhereColumns;
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getJPanelWhereColumns());
			jScrollPane.getVerticalScrollBar().setUnitIncrement(30);
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jToggleButtonOuterJoin
	 * 
	 * @return javax.swing.JToggleButton
	 */
	private JToggleButton getJToggleButtonOuterJoin() {
		if (jToggleButtonOuterJoin == null) {
			jToggleButtonOuterJoin = new JToggleButton();
			jToggleButtonOuterJoin.setText("全部検索指定(外部結合)");
		}
		return jToggleButtonOuterJoin;
	}

	/**
	 * ＳＱＬ条件の先頭要素（table1-table2-table3..)を取得します
	 * 
	 * @return
	 */
	private Element getSqlEditRoot() {
		PrefixStringBuffer buf = new PrefixStringBuffer("", "-");
		for (JPanelConditionTable item : tableInfos) {
			String tbl = item.getSchemaTableName().getCompleteTableName();
			buf.appendItem(tbl);
		}

		SettingFile sf = SettingFile.getInstance();
		Element topSqlEditElement = sf.getRootElement("sqlEdit");

		Element element = SettingFile.getElementSearchingAttribute(
				topSqlEditElement, "TABLES", "name", buf.getString());

		if (element == null) {
			element = sf.createElement("TABLES");
			element.setAttribute("name", buf.getString());
			topSqlEditElement.appendChild(element);
		}
		return element;
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(500, 400);
		this.setContentPane(getJContentPane());
		this.setTitle("SQL文生成");

		getRootPane().setDefaultButton(jButtonOk);

		JComponent jc = new ConditionLine();
		jc.setOpaque(false);
		this.setGlassPane(jc);
		this.getGlassPane().setVisible(true);

		whereActionListener = new JoinPointerActionListener();

		componentMouseMotionListener = new MouseMotionListener() {
			public void mouseDragged(MouseEvent e) {
			}

			public void mouseMoved(MouseEvent e) {
				Point p = ((Component) e.getSource()).getLocationOnScreen();
				Point rp = getRootPane().getLocationOnScreen();

				mousePoint.setLocation(p.x - rp.x + e.getX(), p.y - rp.y
						+ e.getY());
				// System.out.println(mousePoint);
				if (firsstClickJoinPointer != null) {
					getGlassPane().repaint();
				}
			}
		};

		joinPointerMouseListener = new JoinPointerMouseListener();

		getRootPane().setDefaultButton(jButtonOk);
	}

	/**
	 * 設定ファイルに条件を保存する
	 */
	private void save() {
		Element top = getSqlEditRoot();
		((Element) top.getParentNode()).removeChild(top);
		top = getSqlEditRoot();

		SettingFile sf = SettingFile.getInstance();

		// 選択リストの保存
		for (JPanelConditionTable item : tableInfos) {
			String tbl = item.getSchemaTableName().getCompleteTableName();

			// tableの追加
			SelectColumns sc = item.getSelectColumns();
			Element tableElement = sf.createElement("TABLE");
			tableElement.setAttribute("name", tbl);
			tableElement.setAttribute("allSelect", Boolean
					.toString(sc.allSelected));
			top.appendChild(tableElement);

			if (!sc.allSelected) {
				// selected columnsの保存
				for (DBObjectTableColumn item1 : sc.columnInfoTables) {
					Element columnElement = sf.createElement("SELCOLS");
					columnElement.setAttribute("name", item1.getColumnName());
					tableElement.appendChild(columnElement);
				}
			}
		}

		// where条件の保存
		for (Condition condition : conditions) {
			Element columnElement = sf.createElement("WHERECOLS");
			columnElement.setAttribute("starttbl", condition
					.getStartComponent().getColumnInfoTable()
					.getSchemaTableName().getCompleteTableName());
			columnElement.setAttribute("startcol", condition
					.getStartComponent().getColumnInfoTable().getColumnName());
			columnElement.setAttribute("startcolside", condition
					.getStartComponent().getSide());
			columnElement.setAttribute("startjoin", condition
					.getStartComponent().getJoinStatus().toString());

			columnElement.setAttribute("endtbl", condition.getEndComponent()
					.getColumnInfoTable().getSchemaTableName()
					.getCompleteTableName());
			columnElement.setAttribute("endcol", condition.getEndComponent()
					.getColumnInfoTable().getColumnName());
			columnElement.setAttribute("endcolside", condition
					.getEndComponent().getSide());
			columnElement.setAttribute("endjoin", condition.getEndComponent()
					.getJoinStatus().toString());

			top.appendChild(columnElement);
		}
	}

	/**
	 * 作成したＳＱＬ文を書き込むコンポーネントを設定する
	 * 
	 * @param sqlStatement
	 */
	public void setSqlStatement(JTextPaneSqlStatement sqlStatement) {
		this.sqlStatement = sqlStatement;
	}

	public void setTablemap(
			TreeMap<SchemaTableName, ArrayList<DBObjectTableColumn>> selTablesMap, 
			TreeMap<SchemaTableName, ArrayList<DBObjectTableColumn>> defTablesMap) {

		// 初期化
		tableInfos.clear();
		getJPanelWhereColumns().removeAll();
		getJPanelWhereColumns().repaint();

		conditions.clear();
		firsstClickJoinPointer = null;

		int tableNO = 0;
		// テーブル情報を設定
		for (Entry<SchemaTableName, ArrayList<DBObjectTableColumn>> t : defTablesMap.entrySet()) {
			tableNO++;
			JPanelConditionTable p = new JPanelConditionTable(
					t.getKey(),t.getValue(),
					whereActionListener, "T" + tableNO,
					joinPointerMouseListener);
			getJPanelWhereColumns().add(p);
			tableInfos.add(p);
		}

		// 選択済カラムの設定
		Element topElement = getSqlEditRoot();
		for (JPanelConditionTable item : tableInfos) {
			// 処理対象のテーブル決定
			SchemaTableName stn = item.getSchemaTableName();

//			ArrayList<DBObjectTableColumn> columnInfoTables = selTablesMap.get(stn);

			// ツリー選択でカラムが選択されている時は、ツリー選択を最優先
			if (selTablesMap.get(stn).size() != 0) {
				item.setColumnInfoTables(selTablesMap.get(stn));
				continue;
			}

			// 保存パラメータの情報を取得
			Element tableElement = SettingFile.getElementSearchingAttribute(
					topElement, "TABLE", "name", stn.getCompleteTableName());

			if (tableElement == null) {
				// 保存パラメータが無い時は、全カラム選択
				item.setColumnInfoTables(defTablesMap.get(stn));
				continue;
			}

			// ALLSELECT指定の時は、SELCOLSの内容と無関係に全カラム選択
			boolean allSelect = Boolean.valueOf(tableElement
					.getAttribute("allSelect"));
			if (allSelect) {
				item.setColumnInfoTables(defTablesMap.get(stn));
				continue;
			}

			// 保存されているカラムに対応する分を選択する
			ArrayList<DBObjectTableColumn> list = new ArrayList<DBObjectTableColumn>();
			for (Element selColElement : SettingFile.getElements(tableElement,"SELCOLS")) {
				String colName = selColElement.getAttribute("name");
				for (DBObjectTableColumn defColumns : defTablesMap.get(stn)){
					if(defColumns.getColumnName().equals(colName)){
						list.add(defColumns);
						continue;
					}
				}
			}
			item.setColumnInfoTables(list);
		}

		for (Element whereElement : SettingFile.getElements(topElement,
				"WHERECOLS")) {
			SchemaTableName st = new SchemaTableName(whereElement
					.getAttribute("starttbl"));
			String sc = whereElement.getAttribute("startcol");
			String ss = whereElement.getAttribute("startcolside");
			String sj = whereElement.getAttribute("startjoin");
			JToggleButtonJoinPointer start = null;

			SchemaTableName et = new SchemaTableName(whereElement
					.getAttribute("endtbl"));
			String ec = whereElement.getAttribute("endcol");
			String es = whereElement.getAttribute("endcolside");
			String ej = whereElement.getAttribute("endjoin");
			JToggleButtonJoinPointer end = null;

			for (JPanelConditionTable jPanelConditionTable : tableInfos) {
				if (jPanelConditionTable.getSchemaTableName().equals(st)) {
					start = jPanelConditionTable.getJoinPointer(sc, ss);
					start.setSelected(true);
					start.setJoinStatus(JoinStatus.defaultValueOf(sj));
				}
				if (jPanelConditionTable.getSchemaTableName().equals(et)) {
					end = jPanelConditionTable.getJoinPointer(ec, es);
					end.setSelected(true);
					end.setJoinStatus(JoinStatus.defaultValueOf(ej));
				}
			}
			if (start != null && end != null) {
				// 通常ありえないので保険
				conditions.add(new Condition(start, end));
			}
		}
		addAllMouseMotionListener(getJPanelWhereColumns(),
				componentMouseMotionListener);

	}

}