package jdbcacsess.gui;

import java.awt.BorderLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.TableColumn;

import jdbcacsess.JdbcacsessException;
import jdbcacsess.csv.CsvInfo;
import jdbcacsess.csv.CsvInputDataView;
import jdbcacsess.csv.CsvInputImportToDB;
import jdbcacsess.csv.CsvMode;
import jdbcacsess.csv.ImportHistoryTableModel;
import jdbcacsess.csv.JDialogCsv;
import jdbcacsess.gui.common.ColumnWidth;
import jdbcacsess.gui.common.JDialogMessage;
import jdbcacsess.gui.common.JFrameBase;
import jdbcacsess.gui.common.JPopupMenuCnageUI;
import jdbcacsess.gui.common.PopupKeybordAndMouse;
import jdbcacsess.sql.SqlExec;

public class JFrameImportHistory extends JFrameBase {
	@Override
	public String getAttrKey(){ 
		return null;
	}
	@Override
	public String getAttrValue(){ 
		return null;
	}

	private static final long serialVersionUID = 7126241261349764262L;

	private JPanel jContentPane = null;

	private JScrollPane jScrollPane = null;

	private JTable jTable = null;

	private JPanel jPanel = null;

	private JButton jButtonOK = null;

	private ImportHistoryTableModel importHistoryTableModel;

	private JButton jButtonDelete = null;

	private JButton jButtonDisplayUpdate = null;

	private JButton jButtonEditAndImport = null;

	private JButton jButtonEdit = null;

	private JPopupMenuCnageUI jPopupMenuChangeUI = null;

	private JMenuItem jMenuItemOK = null;

	private JMenuItem jMenuItemEditAndImport = null;

	private JMenuItem jMenuItemEdit = null;

	private JMenuItem jMenuItemDelete = null;

	private JButton jButtonDataView = null;

	private JMenuItem jMenuItemDataView = null;

	private JMenuItem jMenuItemDisplayUpdate = null;

	/**
	 * This is the default constructor
	 * @throws JdbcacsessException 
	 */
	public JFrameImportHistory() throws JdbcacsessException {
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 * @throws JdbcacsessException 
	 */
	private void initialize() throws JdbcacsessException {
		this.setTitle("CSV入力履歴");
		this.setSize(new java.awt.Dimension(800, 400));

		this.setContentPane(getJContentPane());

		importHistoryTableModel = new ImportHistoryTableModel();
		jTable.setModel(importHistoryTableModel);

		jTable
				.setSelectionMode(javax.swing.ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		// 列の並び替えは許可しない。
		jTable.getTableHeader().setReorderingAllowed(false);

		// ポップアップを設定
		new PopupKeybordAndMouse(jTable, getJPopupMenuChangeUI());

		// カラムヘッダがクリックされたらソートする のイベント登録
		jTable.getTableHeader().addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				int columnIndex = jTable.getTableHeader().columnAtPoint(
						e.getPoint());
				try {
					importHistoryTableModel.setSortColumnIndex(columnIndex);
				} catch (JdbcacsessException e1) {
					JDialogMessage.errorDialog(e1);
				}
			}
		});

		// カラム幅を暫定設定してから、復元
		ColumnWidth.setColumnWidth(jTable);
		for (int i = 0; i < jTable.getColumnCount(); i++) {
			TableColumn tc = jTable.getTableHeader().getColumnModel()
					.getColumn(i);
			Integer width = omponentProperty.getToInteger(this.getClass(), "col" + i);
			if (width != null) {
				tc.setPreferredWidth(width);
			}
		}
		getRootPane().setDefaultButton(jButtonDisplayUpdate);
		setVisible(true);
	}

	public void reLoad() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try {
					importHistoryTableModel.reLoad();
				} catch (JdbcacsessException e) {
					JDialogMessage.errorDialog(e);
				}
			}
		});
	}

	@Override
	public void frameClosing() {
		// カラム幅保存
		for (int i = 0; i < jTable.getColumnCount(); i++) {
			TableColumn tc = jTable.getTableHeader().getColumnModel()
					.getColumn(i);
			omponentProperty.put(this.getClass(), "col" + i, tc.getWidth());
		}
	}

	private void actionPerformedExecuteImport() {
		if (!SqlExec.isConnect()) {
			JDialogMessage.notConnectErrorDialog();
			return;
		}
		if (!isSelected()) {
			return;
		}
		// Importはそれぞれを別スレッドで実行
		ExecutorService e = Executors.newSingleThreadExecutor();
		for (int index : jTable.getSelectedRows()) {
			e.execute(new CsvInputImportToDB(importHistoryTableModel
					.getCsvInfo(index)));
		}
		e.shutdown();
	}

	private void actionPerformedDelete() throws JdbcacsessException {
		if (!isSelected()) {
			return;
		}
		// リストの削除はインデックス番号で行うと、
		// １件削除する度に番号が変わるので、ファイル名を削除キーにする
		ArrayList<String> fileNames = new ArrayList<String>();
		for (int idx : jTable.getSelectedRows()) {
			fileNames.add(importHistoryTableModel.getKey(idx));
		}
		for (String key : fileNames) {
			importHistoryTableModel.remove(key);
		}
		importHistoryTableModel.reLoad();
	}

	private void actionPerformedEditAndImport() throws JdbcacsessException {
		if (!SqlExec.isConnect()) {
			JDialogMessage.notConnectErrorDialog();
			return;
		}
		if (!isSelected()) {
			return;
		}
		ExecutorService e = Executors.newSingleThreadExecutor();
		for (int idx : jTable.getSelectedRows()) {
			JDialogCsv dlg = new JDialogCsv(CsvMode.IMPORT,
					importHistoryTableModel.getCsvInfo(idx));
			CsvInfo ci = dlg.getCsvInfo();
			if (ci != null) {
				ci.currentDataSave();
				e.execute(new CsvInputImportToDB(ci));
				importHistoryTableModel.fireTableRowsUpdated(idx, idx);
			}
		}
		e.shutdown();
	}

	private void actionPerformedEdit() throws JdbcacsessException {
		if (!SqlExec.isConnect()) {
			JDialogMessage.notConnectErrorDialog();
			return;
		}
		if (!isSelected()) {
			return;
		}
		for (int idx : jTable.getSelectedRows()) {
			JDialogCsv dlg = new JDialogCsv(CsvMode.EDIT,
					importHistoryTableModel.getCsvInfo(idx));
			CsvInfo ci = dlg.getCsvInfo();
			if (ci != null) {
				ci.currentDataSave();
				importHistoryTableModel.fireTableRowsUpdated(idx, idx);
			}
		}
	}

	protected void actionPerformedDataView() {
		if (!isSelected()) {
			return;
		}
		for (int index : jTable.getSelectedRows()) {
			new CsvInputDataView(importHistoryTableModel.getCsvInfo(index));
		}
	}

	/**
	 * 選択行があるかチェック
	 * 
	 * @return 選択されている時 true
	 */
	private boolean isSelected() {
		if (jTable.getSelectedColumnCount() == 0) {
			JDialogMessage.infoDialog("選択されてません", "エラー");
			return false;
		}
		return true;
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJScrollPane(), java.awt.BorderLayout.CENTER);
			jContentPane.add(getJPanel(), java.awt.BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getJTable());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jTable
	 * 
	 * @return javax.swing.JTable
	 */
	private JTable getJTable() {
		if (jTable == null) {
			jTable = new JTable(10, 10);
		}
		return jTable;
	}

	/**
	 * This method initializes jPanel
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel();
			jPanel.add(getJButtonDisplayUpdate(), null);
			jPanel.add(getJButtonDataView(), null);
			jPanel.add(getJButtonEditAndImport(), null);
			jPanel.add(getJButtonEdit(), null);
			jPanel.add(getJButtonDelete(), null);
			jPanel.add(getJButtonImport(), null);
		}
		return jPanel;
	}

	/**
	 * This method initializes jButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButtonImport() {
		if (jButtonOK == null) {
			jButtonOK = new JButton();
			jButtonOK.setText("Import実行");
			jButtonOK.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					actionPerformedExecuteImport();
				}
			});
		}
		return jButtonOK;
	}

	/**
	 * This method initializes jButton
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButtonDelete() {
		if (jButtonDelete == null) {
			jButtonDelete = new JButton();
			jButtonDelete.setText("条件削除");
			jButtonDelete
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							try {
								actionPerformedDelete();
							} catch (JdbcacsessException e1) {
								JDialogMessage.errorDialog(e1);
							}
						}
					});
		}
		return jButtonDelete;
	}

	/**
	 * This method initializes jButtonDisplayUpdate
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButtonDisplayUpdate() {
		if (jButtonDisplayUpdate == null) {
			jButtonDisplayUpdate = new JButton();
			jButtonDisplayUpdate.setText("一覧を更新");
			jButtonDisplayUpdate
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							reLoad();
						}
					});
		}
		return jButtonDisplayUpdate;
	}

	/**
	 * This method initializes jButtonEdit
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButtonEditAndImport() {
		if (jButtonEditAndImport == null) {
			jButtonEditAndImport = new JButton();
			jButtonEditAndImport.setText("CSV入力...");
			jButtonEditAndImport
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							try {
								actionPerformedEditAndImport();
							} catch (JdbcacsessException e1) {
								JDialogMessage.errorDialog(e1);
							}
						}
					});
		}
		return jButtonEditAndImport;
	}

	/**
	 * This method initializes jButtonEdit
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButtonEdit() {
		if (jButtonEdit == null) {
			jButtonEdit = new JButton();
			jButtonEdit.setText("条件編集...");
			jButtonEdit.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					try {
						actionPerformedEdit();
					} catch (JdbcacsessException e1) {
						JDialogMessage.errorDialog(e1);
					}
				}
			});
		}
		return jButtonEdit;
	}

	/**
	 * This method initializes jButtonDataView
	 * 
	 * @return javax.swing.JButton
	 */
	private JButton getJButtonDataView() {
		if (jButtonDataView == null) {
			jButtonDataView = new JButton();
			jButtonDataView.setText("データ100件参照...");
			jButtonDataView
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							actionPerformedDataView();
						}
					});
		}
		return jButtonDataView;
	}

	/**
	 * This method initializes jPopupMenu
	 * 
	 * @return javax.swing.JPopupMenu
	 */
	private JPopupMenuCnageUI getJPopupMenuChangeUI() {
		if (jPopupMenuChangeUI == null) {
			jPopupMenuChangeUI = new JPopupMenuCnageUI();
			jPopupMenuChangeUI.add(getJMenuItemDisplayUpdate());
			jPopupMenuChangeUI.add(getJMenuItemDataView());
			jPopupMenuChangeUI.addSeparator();
			jPopupMenuChangeUI.add(getJMenuItemEditAndImport());
			jPopupMenuChangeUI.add(getJMenuItemEdit());
			jPopupMenuChangeUI.add(getJMenuItemDelete());
			jPopupMenuChangeUI.addSeparator();
			jPopupMenuChangeUI.add(getJMenuItemOK());
		}
		return jPopupMenuChangeUI;
	}

	/**
	 * This method initializes jMenuItem
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemOK() {
		if (jMenuItemOK == null) {
			jMenuItemOK = new JMenuItem();
			jMenuItemOK.setText("Import実行");
			jMenuItemOK.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					actionPerformedExecuteImport();
				}
			});
		}
		return jMenuItemOK;
	}

	/**
	 * This method initializes jMenuItem1
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemEditAndImport() {
		if (jMenuItemEditAndImport == null) {
			jMenuItemEditAndImport = new JMenuItem();
			jMenuItemEditAndImport.setText("CSV入力...");
			jMenuItemEditAndImport
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							try {
								actionPerformedEditAndImport();
							} catch (JdbcacsessException e1) {
								JDialogMessage.errorDialog(e1);
							}
						}
					});
		}
		return jMenuItemEditAndImport;
	}

	/**
	 * This method initializes jMenuItem1
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemEdit() {
		if (jMenuItemEdit == null) {
			jMenuItemEdit = new JMenuItem();
			jMenuItemEdit.setText("条件編集...");
			jMenuItemEdit
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							try {
								actionPerformedEdit();
							} catch (JdbcacsessException e1) {
								JDialogMessage.errorDialog(e1);
							}
						}
					});
		}
		return jMenuItemEdit;
	}

	/**
	 * This method initializes jMenuItem2
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemDelete() {
		if (jMenuItemDelete == null) {
			jMenuItemDelete = new JMenuItem();
			jMenuItemDelete.setText("条件削除");
			jMenuItemDelete
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							try {
								actionPerformedDelete();
							} catch (JdbcacsessException e1) {
								JDialogMessage.errorDialog(e1);
							}
						}
					});
		}
		return jMenuItemDelete;
	}

	/**
	 * This method initializes jMenuItemDataView
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemDataView() {
		if (jMenuItemDataView == null) {
			jMenuItemDataView = new JMenuItem();
			jMenuItemDataView.setText("データ100件参照...");
			jMenuItemDataView
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							actionPerformedDataView();
						}
					});
		}
		return jMenuItemDataView;
	}

	/**
	 * This method initializes jMenuItem
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemDisplayUpdate() {
		if (jMenuItemDisplayUpdate == null) {
			jMenuItemDisplayUpdate = new JMenuItem();
			jMenuItemDisplayUpdate.setText("一覧を更新");
			jMenuItemDisplayUpdate
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							reLoad();
						}
					});
		}
		return jMenuItemDisplayUpdate;
	}

} // @jve:decl-index=0:visual-constraint="10,10"
