package jdbcacsess.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.sql.SQLException;
import java.text.ParseException;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.SwingUtilities;
import javax.swing.ToolTipManager;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.plaf.synth.SynthLookAndFeel;
import javax.xml.transform.TransformerException;

import jdbcacsess.JdbcacsessException;
import jdbcacsess.LogOutput;
import jdbcacsess.SettingFile;
import jdbcacsess.csv.CsvInfo;
import jdbcacsess.csv.CsvInputImportToDB;
import jdbcacsess.csv.CsvMode;
import jdbcacsess.csv.JDialogCsv;
import jdbcacsess.dbconnect.JDialogConnect;
import jdbcacsess.gui.cell.JFrameViewerHex;
import jdbcacsess.gui.cell.OutputHandler;
import jdbcacsess.gui.common.JDialogMessage;
import jdbcacsess.gui.common.JFrameBase;
import jdbcacsess.gui.common.JPopupMenuCnageUI;
import jdbcacsess.sql.SqlExec;

import org.xml.sax.SAXException;

public class JFrameMain extends JFrameBase {
	@Override
	public String getAttrKey(){ 
		return null;
	}
	@Override
	public String getAttrValue(){ 
		return null;
	}


	private static final long serialVersionUID = -398926768714658758L;

	private JPanel jContentPane = null;

	private JLabel jLabelStatusBar = null;

	private JPanel jPanel = null;

	private JPanelEditButton jPanelEditButton = null;

	private JTreeDataBaseObjects jTreeDataBaseObjects = null;

	private JMenuBar jMenuBarBase = null;

	private JMenu jMenuFile = null;

	private JMenu jMenuHelp1 = null;

	private JMenuItem jMenuItemFileConnect = null;

	private JMenuItem jMenuItemFileDisConnect = null;

	private JMenuItem jMenuItemFileCsvImport = null;

	private JMenuItem jMenuItemFileExit = null;

	private JMenuItem jMenuItemHelpAbout = null;

	private JScrollPane jScrollPane = null;

	private JSplitPane jSplitPaneBase = null;

	private TabWinManager tabWinManager;

	private SqlStatementObserver sqlStatementObserver;

	private JMenu jMenuNew = null;

	private JMenuItem jMenuItemAutoCommit = null;

	private JMenuItem jMenuItemManualCommit = null;

	private JMenu jMenuDisplay = null;

	private JMenuItem jMenuItemChangeWin = null;

	private JMenuItem jMenuItemChangeTab = null;

	private JMenuItem jMenuItemWinSort = null;

	private JMenuItem jMenuItemCsvImportHistory = null;

	private JCheckBoxMenuItem jCheckBoxMenuItemDebug = null;

//	public static String fontFamilyName = "Monospaced";

	// フレームの構築
	public JFrameMain() throws JdbcacsessException, SAXException, IOException {
		super();

		actionPerformedConnect();

		initialize();

		jTreeDataBaseObjects.requestFocusInWindow();
		
	}

	// コンポーネントの初期化
	private void initialize() {
		setSize(800, 600);
		setContentPane(getJContentPane());
		setJMenuBar(getJMenuBarBase());
		setTitle("JdbcAcsess");

		Integer divider = omponentProperty.getToInteger(this.getClass(), "divider");
		if (divider != null) {
			jSplitPaneBase.setDividerLocation(divider);
		}

		sqlStatementObserver = new SqlStatementObserver();
		sqlStatementObserver.addObserver(jTreeDataBaseObjects);
		sqlStatementObserver.addObserver(jPanelEditButton);
		tabWinManager = new TabWinManager(sqlStatementObserver);

		jSplitPaneBase.setRightComponent(tabWinManager.getPane().getJComponentPane());

		// ツールチップの表示時間を設定する
		ToolTipManager.sharedInstance().setDismissDelay(60000);

		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setVisible(true);

		if (SqlExec.isConnect()) {
			jTreeDataBaseObjects.setTree();
		}
		
		setMessage(SqlExec.getConnectString());
		

	}
	
	
	public TabWinManager getTabWinManager() {
		return tabWinManager;
	}

	@Override
	public void frameClosing() throws JdbcacsessException {
		if (SqlExec.isConnect()) {
			SqlExec.disconnect();
		}
		tabWinManager.allPaneClosed();

		omponentProperty.put(this.getClass(), "divider", jSplitPaneBase.getDividerLocation());

		SettingFile xml = SettingFile.getInstance();
		try {
			xml.save();
		} catch (FileNotFoundException e) {
			throw new JdbcacsessException(e);
		} catch (TransformerException e) {
			throw new JdbcacsessException(e);
		} catch (IOException e) {
			throw new JdbcacsessException(e);
		}
//		System.exit(0);
	}

	private void actionPerformedNewWindow(boolean antoCommit) {
		tabWinManager.newPanel(antoCommit);
	}

	private void actionPerformedTabWin(TabWinMode tw) {
		if (tabWinManager.getPane().isTabWinModeEqual(tw)){
			return;
		}
		Component c = jSplitPaneBase.getRightComponent();
		
		int size = jSplitPaneBase.getDividerLocation();

		if (c != null)
			jSplitPaneBase.remove(c);

		jSplitPaneBase.setRightComponent(tabWinManager.changeTabWin(tw));
		jSplitPaneBase.setDividerLocation(size);
		jSplitPaneBase.validate();
	}

	private void actionPerformedSort() {
		tabWinManager.sortWindow();
	}

	/**
	 * DB接続イベント
	 * @throws IOException 
	 * @throws SAXException 
	 * @throws JdbcacsessException 
	 */
	private boolean actionPerformedConnect() throws JdbcacsessException, SAXException, IOException {
		// 接続画面のインスタンス生成
		JDialogConnect jdc = new JDialogConnect();
		if (!jdc.isAnsOK()){
			return false;
		}
		
		setMessage(SqlExec.getConnectString());
		return true;
	}

	/**
	 * DB切断イベント処理
	 */
	private void actionPerformedDisConnect(ActionEvent e) {
		if (!SqlExec.isConnect()) {
			JDialogMessage.notConnectErrorDialog();
			return;
		}
		setMessage(" ");
		tabWinManager.allPaneClosed();

		SqlExec.disconnect();

		jTreeDataBaseObjects.clearTree();
		if (!SqlExec.isConnect()) {
			setMessage("DB切断しました");
		}
	}

	/**
	 * ＣＳＶ入力
	 */
	private void actionPerformedCsvImport(ActionEvent e) {
		if (!SqlExec.isConnect()) {
			JDialogMessage.notConnectErrorDialog();
			return;
		}
		setMessage(" ");

		// Importダイアログの表示
		JDialogCsv dlg = new JDialogCsv(CsvMode.IMPORT);
		CsvInfo csvInfo = dlg.getCsvInfo();
		if (csvInfo == null) {
			// 取得できなかったなので、取消押下
			return;
		}

		new Thread(new CsvInputImportToDB(csvInfo)).start();
	}

	/**
	 * @return jContentPane を戻します。
	 * @uml.property name="jContentPane"
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabelStatusBar = new JLabel();
			jLabelStatusBar.setFont(new java.awt.Font("Dialog",
					java.awt.Font.PLAIN, 14));
			jLabelStatusBar.setText(" ");
			jLabelStatusBar.setName("statusBar");
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJSplitPaneBase(), java.awt.BorderLayout.CENTER);
			jContentPane.add(jLabelStatusBar, java.awt.BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jPanel
	 * 
	 * @return javax.swing.JPanel
	 * @uml.property name="jPanel"
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			jPanel = new JPanel();
			jPanel.setLayout(new BorderLayout());
			jPanel.add(getJPanelEditButton(), java.awt.BorderLayout.NORTH);
			jPanel.add(getJScrollPane(), java.awt.BorderLayout.CENTER);
		}
		return jPanel;
	}

	/**
	 * This method initializes jPanelEditButton
	 * 
	 * @return jdbcacsess.gui.JPanelEditButton
	 * @uml.property name="jPanelEditButton"
	 */
	private JPanelEditButton getJPanelEditButton() {
		if (jPanelEditButton == null) {
			jPanelEditButton = new JPanelEditButton();
		}
		return jPanelEditButton;
	}

	/**
	 * This method initializes jScrollPaneTreeTables
	 * 
	 * @return jdbcacsess.gui.JScrollPaneTreeTables
	 * @uml.property name="jTreeDataBaseObjects"
	 */
	private JTreeDataBaseObjects getJTreeDataBaseObjects() {
		if (jTreeDataBaseObjects == null) {
			jTreeDataBaseObjects = new JTreeDataBaseObjects();
		}
		return jTreeDataBaseObjects;
	}

	/**
	 * This method initializes jSplitPane
	 * 
	 * @return javax.swing.JSplitPane
	 * @uml.property name="jSplitPaneBase"
	 */
	private JSplitPane getJSplitPaneBase() {
		if (jSplitPaneBase == null) {
			jSplitPaneBase = new JSplitPane();
			jSplitPaneBase.setDividerLocation(150);
			jSplitPaneBase.setOneTouchExpandable(true);
			jSplitPaneBase.setDividerSize(7);
			jSplitPaneBase.setLeftComponent(getJPanel());
		}
		return jSplitPaneBase;
	}

	// [ヘルプ|バージョン情報]
	private void actionPerformedHelpAbout(ActionEvent e) {
		JFrameAboutBox dlg =  JFrameAboutBox.getInstance();
		dlg.setVisible(true);
	}

	public void setMessage(String msg) {
		if (jLabelStatusBar != null) {
			jLabelStatusBar.setText(msg);
		}
	}

	/**
	 * This method initializes jJMenuBar
	 * 
	 * @return javax.swing.JMenuBar
	 * @uml.property name="jMenuBarBase"
	 */
	private JMenuBar getJMenuBarBase() {
		if (jMenuBarBase == null) {
			jMenuBarBase = new JMenuBar();
			jMenuBarBase.add(getJMenuFile());
			jMenuBarBase.add(getJMenuDisplay());
			jMenuBarBase.add(getJMenuHelp1());
		}
		return jMenuBarBase;
	}

	/**
	 * This method initializes jMenuFile1
	 * 
	 * @return javax.swing.JMenu
	 * @uml.property name="jMenuFile"
	 */
	private JMenu getJMenuFile() {
		if (jMenuFile == null) {
			jMenuFile = new JMenu();
			jMenuFile.setText("ファイル(F)");
			jMenuFile.setMnemonic(java.awt.event.KeyEvent.VK_F);
			jMenuFile.add(getJMenuNew());
			jMenuFile.add(getJMenuItemFileConnect());
			jMenuFile.add(getJMenuItemFileDisConnect());
			jMenuFile.add(getJMenuItemFileCsvImport());
			jMenuFile.add(getJMenuItemCsvImportHistory());
			jMenuFile.add(getJMenuItemHexViewer());
			jMenuFile.add(getJMenuItemFileExit());
		}
		return jMenuFile;
	}

	/**
	 * This method initializes jMenuHelp1
	 * 
	 * @return javax.swing.JMenu
	 * @uml.property name="jMenuHelp1"
	 */
	private JMenu getJMenuHelp1() {
		if (jMenuHelp1 == null) {
			jMenuHelp1 = new JMenu();
			jMenuHelp1.setText("ヘルプ(H)");
			jMenuHelp1.setMnemonic(java.awt.event.KeyEvent.VK_H);
			jMenuHelp1.add(getJCheckBoxMenuItemDebug());
			jMenuHelp1.add(getJMenuItemHelpAbout());
		}
		return jMenuHelp1;
	}

	/**
	 * This method initializes jMenuItemFileConnect
	 * 
	 * @return javax.swing.JMenuItem
	 * @uml.property name="jMenuItemFileConnect"
	 */
	private JMenuItem getJMenuItemFileConnect() {
		if (jMenuItemFileConnect == null) {
			jMenuItemFileConnect = new JMenuItem();
			jMenuItemFileConnect.setText("接続...(C)");
			jMenuItemFileConnect.setMnemonic(java.awt.event.KeyEvent.VK_C);
			jMenuItemFileConnect.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					boolean rtn;
					try {
						rtn = actionPerformedConnect();
						if (!rtn) {
							return;
						}
					} catch (JdbcacsessException e1) {
						JDialogMessage.errorDialog(e1);
					} catch (SAXException e1) {
						JDialogMessage.errorDialog(e1);
					} catch (IOException e1) {
						JDialogMessage.errorDialog(e1);
					}
					jTreeDataBaseObjects.clearTree();
					tabWinManager.allDisconnect();
					jTreeDataBaseObjects.setTree();
					if (tabWinManager.getExecSqlListCnt() == 0) {
						tabWinManager.newPanel(true);
						tabWinManager.newPanel(true);
						tabWinManager.newPanel(true);
						tabWinManager.newPanel(true);
					}
				}
			});
		}
		return jMenuItemFileConnect;
	}

	/**
	 * This method initializes jMenuItemFileDisConnect
	 * 
	 * @return javax.swing.JMenuItem
	 * @uml.property name="jMenuItemFileDisConnect"
	 */
	private JMenuItem getJMenuItemFileDisConnect() {
		if (jMenuItemFileDisConnect == null) {
			jMenuItemFileDisConnect = new JMenuItem();
			jMenuItemFileDisConnect.setText("切断(D)");
			jMenuItemFileDisConnect.setMnemonic(java.awt.event.KeyEvent.VK_D);
			jMenuItemFileDisConnect
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							actionPerformedDisConnect(e);
						}
					});
		}
		return jMenuItemFileDisConnect;
	}

	/**
	 * This method initializes jMenuItem
	 * 
	 * @return javax.swing.JMenuItem
	 * @uml.property name="jMenuItemFileCsvImport"
	 */
	private JMenuItem getJMenuItemFileCsvImport() {
		if (jMenuItemFileCsvImport == null) {
			jMenuItemFileCsvImport = new JMenuItem();
			jMenuItemFileCsvImport.setText("CSV入力...(I)");
			jMenuItemFileCsvImport.setMnemonic(java.awt.event.KeyEvent.VK_I);
			jMenuItemFileCsvImport
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							actionPerformedCsvImport(e);
						}
					});
		}
		return jMenuItemFileCsvImport;
	}

	/**
	 * This method initializes jMenuItemFileExit
	 * 
	 * @return javax.swing.JMenuItem
	 * @uml.property name="jMenuItemFileExit"
	 */
	private JMenuItem getJMenuItemFileExit() {
		if (jMenuItemFileExit == null) {
			jMenuItemFileExit = new JMenuItem();
			jMenuItemFileExit.setText("終了(X)");
			jMenuItemFileExit.setMnemonic(java.awt.event.KeyEvent.VK_X);
			jMenuItemFileExit
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							try {
								closeAnotherFrame();
								closeFrame();
							} catch (JdbcacsessException e1) {
								JDialogMessage.errorDialog(e1);
							}
						}
					});
		}
		return jMenuItemFileExit;
	}

	/**
	 * This method initializes jMenuItemHelpAbout
	 * 
	 * @return javax.swing.JMenuItem
	 * @uml.property name="jMenuItemHelpAbout"
	 */
	private JMenuItem getJMenuItemHelpAbout() {
		if (jMenuItemHelpAbout == null) {
			jMenuItemHelpAbout = new JMenuItem();
			jMenuItemHelpAbout.setText("このプログラムについて(A)");
			jMenuItemHelpAbout.setMnemonic(java.awt.event.KeyEvent.VK_A);
			jMenuItemHelpAbout
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							actionPerformedHelpAbout(e);
						}
					});
		}
		return jMenuItemHelpAbout;
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 * @uml.property name="jScrollPane"
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getJTreeDataBaseObjects());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jMenu
	 * 
	 * @return javax.swing.JMenu
	 */
	private JMenu getJMenuNew() {
		if (jMenuNew == null) {
			jMenuNew = new JMenu();
			jMenuNew.setText("新規(N)");
			jMenuNew.setMnemonic(java.awt.event.KeyEvent.VK_N);
			jMenuNew.add(getJMenuItemAutoCommit());
			jMenuNew.add(getJMenuItemManualCommit());
		}
		return jMenuNew;
	}

	/**
	 * This method initializes jMenuItem
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemAutoCommit() {
		if (jMenuItemAutoCommit == null) {
			jMenuItemAutoCommit = new JMenuItem();
			jMenuItemAutoCommit.setText("AutoCommit");
			jMenuItemAutoCommit.setMnemonic(java.awt.event.KeyEvent.VK_A);
			jMenuItemAutoCommit
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							actionPerformedNewWindow(true);
						}
					});
		}
		return jMenuItemAutoCommit;
	}

	/**
	 * This method initializes jMenuItem1
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemManualCommit() {
		if (jMenuItemManualCommit == null) {
			jMenuItemManualCommit = new JMenuItem();
			jMenuItemManualCommit.setText("ManualCommit");
			jMenuItemManualCommit.setMnemonic(java.awt.event.KeyEvent.VK_M);
			jMenuItemManualCommit
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							actionPerformedNewWindow(false);
						}
					});
		}
		return jMenuItemManualCommit;
	}

	/**
	 * This method initializes jMenu1
	 * 
	 * @return javax.swing.JMenu
	 */
	private JMenu getJMenuDisplay() {
		if (jMenuDisplay == null) {
			jMenuDisplay = new JMenu();
			jMenuDisplay.setText("表示(V)");
			jMenuDisplay.setMnemonic(java.awt.event.KeyEvent.VK_V);
			jMenuDisplay.add(getJMenuItemChangeTab());
			jMenuDisplay.addSeparator();
			jMenuDisplay.add(getJMenuItemChangeWin());
			jMenuDisplay.add(getJMenuItemWinSort());
			jMenuDisplay.addSeparator();
			jMenuDisplay.add(getJMenuChangeUI());
			jMenuDisplay.add(getJMenuBinaryData());
			jMenuDisplay.add(getJMenuItemSetting());
		}
		return jMenuDisplay;
	}

	/**
	 * This method initializes jMenuItem2
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemChangeTab() {
		if (jMenuItemChangeWin == null) {
			jMenuItemChangeWin = new JMenuItem();
			jMenuItemChangeWin.setText("タブに切替(T)");
			jMenuItemChangeWin.setMnemonic(java.awt.event.KeyEvent.VK_T);
			jMenuItemChangeWin
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							actionPerformedTabWin(TabWinMode.TAB);
						}
					});
		}
		return jMenuItemChangeWin;
	}

	/**
	 * This method initializes jMenuItem3
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemChangeWin() {
		if (jMenuItemChangeTab == null) {
			jMenuItemChangeTab = new JMenuItem();
			jMenuItemChangeTab.setText("ウィンドウに切替(W)");
			jMenuItemChangeTab.setMnemonic(java.awt.event.KeyEvent.VK_W);
			jMenuItemChangeTab
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							actionPerformedTabWin(TabWinMode.WIN);
						}
					});
		}
		return jMenuItemChangeTab;
	}

	/**
	 * This method initializes jMenuItemWinSort
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemWinSort() {
		if (jMenuItemWinSort == null) {
			jMenuItemWinSort = new JMenuItem();
			jMenuItemWinSort.setText("ウィンドウの整列(S)");
			jMenuItemWinSort.setMnemonic(java.awt.event.KeyEvent.VK_S);
			jMenuItemWinSort
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							actionPerformedSort();
						}
					});
		}
		return jMenuItemWinSort;
	}

	/**
	 * This method initializes jMenuItemCsvImportHistory
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemCsvImportHistory() {
		if (jMenuItemCsvImportHistory == null) {
			jMenuItemCsvImportHistory = new JMenuItem();
			jMenuItemCsvImportHistory.setText("CSV入力履歴...(H)");
			jMenuItemCsvImportHistory.setMnemonic(java.awt.event.KeyEvent.VK_H);
			jMenuItemCsvImportHistory
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							try {
								new JFrameImportHistory();
							} catch (JdbcacsessException e1) {
								JDialogMessage.errorDialog(e1);
							}
						}
					});
		}
		return jMenuItemCsvImportHistory;
	}

	/**
	 * This method initializes jCheckBoxMenuItemDebug
	 * 
	 * @return javax.swing.JCheckBoxMenuItem
	 */
	public static boolean debugMode = false;

	public static int binaryDataSize = 100;

	private JMenu jMenuChangeUI = null;

	private JRadioButtonMenuItem jRadioButtonMenuItem = null;

	private JMenuItem jMenuItemHexViewer = null;

	private JMenu jMenuBinaryData = null;

	private JRadioButtonMenuItem jRadioButtonMenuItemNoLimit = null;

	private JRadioButtonMenuItem jRadioButtonMenuItemLimit1KB = null;

	private JRadioButtonMenuItem jRadioButtonMenuItemLimit100KB = null;

	private JRadioButtonMenuItem jRadioButtonMenuItemLimit100B = null;

	private JRadioButtonMenuItem jRadioButtonMenuItemLimit10KB = null;

	private JMenuItem jMenuItemSetting = null;
	
	static private JPanelPrepareStatementLog jPanelPrepareStatementLog;
	static public JPanelPrepareStatementLog getInstanceJPanelPrepareStatementLog(){
		if (jPanelPrepareStatementLog==null){
			jPanelPrepareStatementLog = new JPanelPrepareStatementLog();
		}
		return jPanelPrepareStatementLog;
	}

	static private JPanelResultSetMetaData jPanelResultSetMetaData;
	static public JPanelResultSetMetaData getInstanceJPanelResultSetMetaData(){
		if (jPanelResultSetMetaData==null){
			jPanelResultSetMetaData = new JPanelResultSetMetaData();
		}
		return jPanelResultSetMetaData;
	}

	private JPanelDatabaseMetaData jPanelDatabaseMetaData=null;
	
	private JCheckBoxMenuItem getJCheckBoxMenuItemDebug() {
		if (jCheckBoxMenuItemDebug == null) {
			jCheckBoxMenuItemDebug = new JCheckBoxMenuItem();
			jCheckBoxMenuItemDebug.setText("デバックモード(D)");
			jCheckBoxMenuItemDebug.setMnemonic(java.awt.event.KeyEvent.VK_D);
			jCheckBoxMenuItemDebug
					.addItemListener(new java.awt.event.ItemListener() {
						public void itemStateChanged(java.awt.event.ItemEvent e) {
							if (e.getStateChange() == ItemEvent.SELECTED) {
								debugMode = true;
								tabWinManager.addPanel(getInstanceJPanelPrepareStatementLog());
								tabWinManager.addPanel(getInstanceJPanelResultSetMetaData());
								jPanelDatabaseMetaData = new JPanelDatabaseMetaData();
								tabWinManager.addPanel(jPanelDatabaseMetaData);
								try {
									jPanelDatabaseMetaData.setData(SqlExec.getDatabaseMetaData());
								} catch (SQLException e1) {
									e1.printStackTrace();
								}
							} else {
								debugMode = false;
								tabWinManager.disposePanel(getInstanceJPanelPrepareStatementLog());
								tabWinManager.disposePanel(getInstanceJPanelResultSetMetaData());
								tabWinManager.disposePanel(jPanelDatabaseMetaData);
								jPanelPrepareStatementLog=null;
								jPanelResultSetMetaData=null;
								jPanelDatabaseMetaData=null;
							}
						}
					});
		}
		return jCheckBoxMenuItemDebug;
	}

	/**
	 * This method initializes jMenuChangeUI
	 * 
	 * @return javax.swing.JMenu
	 */
	private JMenu getJMenuChangeUI() {
		if (jMenuChangeUI == null) {
			jMenuChangeUI = new JMenu();
			jMenuChangeUI.setText("UI切替(C)");

			jMenuChangeUI.setMnemonic(java.awt.event.KeyEvent.VK_C);
			ButtonGroup group = new ButtonGroup();

			UIManager.LookAndFeelInfo[] infos = UIManager
					.getInstalledLookAndFeels();
			for (UIManager.LookAndFeelInfo info : infos) {
				JRadioButtonMenuItem item = new JRadioButtonMenuItem();
				item.setText(info.getName());
				item.setActionCommand(info.getClassName());
				item.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						try {
							actionPerformedChnageUI(e);
						} catch (Exception e1) {
							JDialogMessage.errorDialog(e1);
						}
					}
				});
				group.add(item);
				jMenuChangeUI.add(item);

				// 現在のLookAndFeelを選択状態にする。
				if (UIManager.getLookAndFeel().getClass().getName().equals(
						info.getClassName())) {
					item.setSelected(true);
				}
			}
			jMenuChangeUI.add(getJRadioButtonMenuItem());
			group.add(jRadioButtonMenuItem);

		}
		return jMenuChangeUI;
	}

	/**
	 * @param e
	 * @throws UnsupportedLookAndFeelException 
	 * @throws IllegalAccessException 
	 * @throws InstantiationException 
	 * @throws ClassNotFoundException 
	 */
	protected void actionPerformedChnageUI(ActionEvent e) throws ClassNotFoundException, InstantiationException, IllegalAccessException, UnsupportedLookAndFeelException {
		UIManager.setLookAndFeel(e.getActionCommand());
		SwingUtilities.updateComponentTreeUI(this);
		JPopupMenuCnageUI.changeUI();
		JFrameBase.changeUI();

		omponentProperty.put(UIManager.class, "className", UIManager
				.getLookAndFeel().getClass().getName());
	}

	/**
	 * This method initializes jRadioButtonMenuItem
	 * 
	 * @return javax.swing.JRadioButtonMenuItem
	 */
	private JRadioButtonMenuItem getJRadioButtonMenuItem() {
		if (jRadioButtonMenuItem == null) {
			jRadioButtonMenuItem = new JRadioButtonMenuItem();
			jRadioButtonMenuItem.setText("Synth...");
			jRadioButtonMenuItem.setVisible(false);
			jRadioButtonMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							SynthLookAndFeel synth = new SynthLookAndFeel();
							try {
								synth.load(getClass().getResourceAsStream("file1.xml"), getClass());
								UIManager.setLookAndFeel(synth);
							} catch (IllegalArgumentException e1) {
								e1.printStackTrace();
							} catch (ParseException e1) {
								e1.printStackTrace();
							} catch (UnsupportedLookAndFeelException e1) {
								e1.printStackTrace();
							}
						}
					});
		}
		return jRadioButtonMenuItem;
	}

	/**
	 * This method initializes jMenuItem
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemHexViewer() {
		if (jMenuItemHexViewer == null) {
			jMenuItemHexViewer = new JMenuItem();
			jMenuItemHexViewer.setText("バイナリビューワ[File]...(V)");
			jMenuItemHexViewer.setMnemonic(java.awt.event.KeyEvent.VK_V);
			jMenuItemHexViewer
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							actionPerformedHexViewer();
						}
					});
		}
		return jMenuItemHexViewer;
	}

	/**
	 * 
	 */
	protected void actionPerformedHexViewer() {
		JFileChooser chooser = new JFileChooser();
		int returnVal = chooser.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			File f = chooser.getSelectedFile();
			try {
				InputStream is = new BufferedInputStream(new FileInputStream(f));
				OutputHandler oh = JFrameViewerHex.getInstance();
				oh.setInputStream(is);
				oh.setDataSize(f.length());
				oh.setDataSource(f.getAbsolutePath());
				oh.output();
			} catch (FileNotFoundException e) {
				JDialogMessage.errorDialog(e);
			} catch (IOException e) {
				JDialogMessage.errorDialog(e);
			}

		}
	}

	/**
	 * This method initializes jMenuBinaryData
	 * 
	 * @return javax.swing.JMenu
	 */
	private JMenu getJMenuBinaryData() {
		if (jMenuBinaryData == null) {
			jMenuBinaryData = new JMenu();
			jMenuBinaryData.setText("バイナリ検索結果の読込量(R)");
			jMenuBinaryData.setMnemonic(java.awt.event.KeyEvent.VK_R);
			jMenuBinaryData
					.setToolTipText("<html>画面表示時のみ有効です。<br><strong>ＣＳＶ出力時は常に『全て読込』</strong>で行なわれます。</htmb>");
			jMenuBinaryData.add(getJRadioButtonMenuItemNoLimit());
			jMenuBinaryData.add(getJRadioButtonMenuItemLimit100B());
			jMenuBinaryData.add(getJRadioButtonMenuItemLimit1KB());
			jMenuBinaryData.add(getJRadioButtonMenuItemLimit10KB());
			jMenuBinaryData.add(getJRadioButtonMenuItemLimit100KB());
			ButtonGroup bg = new ButtonGroup();
			bg.add(getJRadioButtonMenuItemNoLimit());
			bg.add(getJRadioButtonMenuItemLimit100B());
			bg.add(getJRadioButtonMenuItemLimit1KB());
			bg.add(getJRadioButtonMenuItemLimit10KB());
			bg.add(getJRadioButtonMenuItemLimit100KB());
		}
		return jMenuBinaryData;
	}

	/**
	 * This method initializes jRadioButtonMenuItem1
	 * 
	 * @return javax.swing.JRadioButtonMenuItem
	 */
	private JRadioButtonMenuItem getJRadioButtonMenuItemNoLimit() {
		if (jRadioButtonMenuItemNoLimit == null) {
			jRadioButtonMenuItemNoLimit = new JRadioButtonMenuItem();
			jRadioButtonMenuItemNoLimit.setText("全て読込");
			jRadioButtonMenuItemNoLimit
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							binaryDataSize = 0;
						}
					});
		}
		return jRadioButtonMenuItemNoLimit;
	}

	/**
	 * This method initializes jRadioButtonMenuItem100B
	 * 
	 * @return javax.swing.JRadioButtonMenuItem
	 */
	private JRadioButtonMenuItem getJRadioButtonMenuItemLimit100B() {
		if (jRadioButtonMenuItemLimit100B == null) {
			jRadioButtonMenuItemLimit100B = new JRadioButtonMenuItem();
			jRadioButtonMenuItemLimit100B.setText("100Byte迄読込");
			jRadioButtonMenuItemLimit100B.setSelected(true);
			jRadioButtonMenuItemLimit100B
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							binaryDataSize = 100;
						}
					});
		}
		return jRadioButtonMenuItemLimit100B;
	}

	/**
	 * This method initializes jRadioButtonMenuItem1
	 * 
	 * @return javax.swing.JRadioButtonMenuItem
	 */
	private JRadioButtonMenuItem getJRadioButtonMenuItemLimit1KB() {
		if (jRadioButtonMenuItemLimit1KB == null) {
			jRadioButtonMenuItemLimit1KB = new JRadioButtonMenuItem();
			jRadioButtonMenuItemLimit1KB.setText("1KByte迄読込");
			jRadioButtonMenuItemLimit1KB
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							binaryDataSize = 1 * 1024;
						}
					});
		}
		return jRadioButtonMenuItemLimit1KB;
	}

	/**
	 * This method initializes jRadioButtonMenuItemLimit10KB
	 * 
	 * @return javax.swing.JRadioButtonMenuItem
	 */
	private JRadioButtonMenuItem getJRadioButtonMenuItemLimit10KB() {
		if (jRadioButtonMenuItemLimit10KB == null) {
			jRadioButtonMenuItemLimit10KB = new JRadioButtonMenuItem();
			jRadioButtonMenuItemLimit10KB.setText("10KByte迄読込");
			jRadioButtonMenuItemLimit10KB
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							binaryDataSize = 10 * 1024;
						}
					});
		}
		return jRadioButtonMenuItemLimit10KB;
	}

	/**
	 * This method initializes jRadioButtonMenuItem100KB
	 * 
	 * @return javax.swing.JRadioButtonMenuItem
	 */
	private JRadioButtonMenuItem getJRadioButtonMenuItemLimit100KB() {
		if (jRadioButtonMenuItemLimit100KB == null) {
			jRadioButtonMenuItemLimit100KB = new JRadioButtonMenuItem();
			jRadioButtonMenuItemLimit100KB.setText("100KByte迄読込");
			jRadioButtonMenuItemLimit100KB
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							binaryDataSize = 100 * 1024;
						}
					});
		}
		return jRadioButtonMenuItemLimit100KB;
	}

	/**
	 * This method initializes jMenuItemSetting
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemSetting() {
		if (jMenuItemSetting == null) {
			jMenuItemSetting = new JMenuItem();
			jMenuItemSetting.setText("設定...");
			jMenuItemSetting
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							try {
								new JFrameSetting();
							} catch (JdbcacsessException e1) {
								JDialogMessage.errorDialog(e1);
							}
						}
					});
		}
		return jMenuItemSetting;
	}

	static public Component serachComponent(Component component, Class<?> c) {
		LogOutput.finest(" serachComponent:" + "component=" + component.getClass().getName() + " Class=" + c.getClass().getName() );
		if (component.getClass() == c){
			return component;
		}
		Container parent = component.getParent();
		if (parent==null){
			return null;
		}
		return serachComponent(parent, c);
		
	}
	
}