package jdbcacsess.gui;

import java.awt.Component;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;

import jdbcacsess.gui.JPanelConditionTable.JPanelConditionColumn.JToggleButtonJoinPointer;
import jdbcacsess.sql.SchemaTableName;
import jdbcacsess.sql.DBObjectTableColumns.DBObjectTableColumn;

/**
 * @author sss
 * 
 */
public class JPanelConditionTable extends JPanel {

	private static final long serialVersionUID = 7919683889963770593L;

	private ActionListener whereActionListener;

	private MouseListener joinPointerMouseListener;

	private String tableAlias;

	private SchemaTableName schemaTableName;

	/**
	 * コンストラクタ
	 * 
	 * @param cols
	 * @param columnInfoTables
	 * @param whereActionListener
	 * @param tableAlias
	 * @param joinPointerMouseListener
	 */
	JPanelConditionTable(SchemaTableName stn,ArrayList<DBObjectTableColumn> columns,
			ActionListener whereActionListener, String tableAlias,
			MouseListener joinPointerMouseListener) {

		this.schemaTableName = stn;
		this.whereActionListener = whereActionListener;
		this.tableAlias = tableAlias;
		this.joinPointerMouseListener = joinPointerMouseListener;

		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		// 全体を囲ませるボーダとテーブル名を見出し
		setBorder(new EtchedBorder(EtchedBorder.RAISED));
		JLabel label = new JLabel(stn.getCompleteTableName());
		label.setBorder(new EmptyBorder(2, 1, 3, 1));
		add(label);

		for (DBObjectTableColumn cit : columns) {
			JPanel jpanel = new JPanelConditionColumn(cit);

			this.add(jpanel);
		}
		// 一番最下の隙間を埋める（これで上揃いになる）
		this.add(Box.createVerticalGlue());
	}

	/**
	 * カラム情報を設定する
	 * 
	 * @param arrayList
	 *            選択状態にするカラムのリスト
	 */
	public void setColumnInfoTables(ArrayList<DBObjectTableColumn> arrayList) {
		// コンポーネント分ループ
		for (Component component : getComponents()) {
			if (component instanceof JPanelConditionColumn) {
				// カラムコンポーネントの時
				JPanelConditionColumn jPanelConditionColumn = (JPanelConditionColumn) component;

				boolean hit = false;
				// 選択要求分のループ
				for (DBObjectTableColumn cit : arrayList) {
					if (cit == jPanelConditionColumn.getColumnInfoTable()) {
						hit = true;
						break;
					}
				}
				if (!hit) {
					jPanelConditionColumn.jToggleButtonColumn
							.setSelected(false);
				}
			}
		}
	}

	/**
	 * コンストラクタで指定したスキーマテーブル情報
	 * 
	 * @return スキーマテーブル情報
	 */
	SchemaTableName getSchemaTableName() {
		return schemaTableName;
	}

	String getTableAlias() {
		return tableAlias;
	}

	/**
	 * 選択リストに載せるカラム情報を取得します
	 * 
	 * @param allColumnsAstar
	 *            全て選択状態の時に空リストを返却する(true)。全カラム返却する(false)。
	 * @return カラムリスト
	 */
	static class SelectColumns {
		boolean allSelected = true;

		ArrayList<DBObjectTableColumn> columnInfoTables = new ArrayList<DBObjectTableColumn>();
	}

	SelectColumns getSelectColumns(boolean allColumnsAstar) {
		SelectColumns rtn = new SelectColumns();

		for (Component component : getComponents()) {
			if (component instanceof JPanelConditionColumn) {
				JPanelConditionColumn jPanelConditionColumn = (JPanelConditionColumn) component;
				if (jPanelConditionColumn.isSelectColumn()) {
					rtn.columnInfoTables.add(jPanelConditionColumn
							.getColumnInfoTable());
				} else {
					rtn.allSelected = false;
				}
			}
		}
		if (allColumnsAstar) {
			if (rtn.allSelected) {
				rtn.columnInfoTables.clear();
			}
		}
		return rtn;
	}

	SelectColumns getSelectColumns() {
		SelectColumns rtn = new SelectColumns();

		for (Component component : getComponents()) {
			if (component instanceof JPanelConditionColumn) {
				JPanelConditionColumn jPanelConditionColumn = (JPanelConditionColumn) component;
				if (jPanelConditionColumn.isSelectColumn()) {
					rtn.columnInfoTables.add(jPanelConditionColumn
							.getColumnInfoTable());
				} else {
					rtn.allSelected = false;
				}
			}
		}
		return rtn;
	}

	/**
	 * where条件に使用すべきカラムの存在有無を取得します
	 * 
	 * @return 有無
	 */
	boolean isUseJoinPointer() {
		for (Component component : getComponents()) {
			if (component instanceof JPanelConditionColumn) {
				JPanelConditionColumn jPanelConditionColumn = (JPanelConditionColumn) component;
				if (jPanelConditionColumn.isSelectJoinPointer()) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * @param columnName
	 * @return
	 */
	public JToggleButtonJoinPointer getJoinPointer(String columnName,
			String side) {
		for (Component component : getComponents()) {
			if (component instanceof JPanelConditionColumn) {
				JPanelConditionColumn jPanelConditionColumn = (JPanelConditionColumn) component;
				if (jPanelConditionColumn.getColumnInfoTable().getName()
						.equals(columnName)) {
					return jPanelConditionColumn.searchJoinPointer(side);
				}
			}
		}
		return null;
	}

	public enum JoinStatus {
		OUTER_NONE, OUTER_EQUAL, OUTER_ALL;
		public static JoinStatus defaultValueOf(String str) {
			try {
				return JoinStatus.valueOf(str);
			} catch (IllegalArgumentException e) {
			}
			return JoinStatus.OUTER_NONE;
		}
	}

	/**
	 * １カラム分のボタンを１つにまとめたクラス
	 */
	public class JPanelConditionColumn extends JPanel {

		private static final long serialVersionUID = 1L;

		private DBObjectTableColumn columnInfoTable;

		private JToggleButtonJoinPointer jToggleButtonJoinPointer1;

		private JToggleButtonJoinPointer jToggleButtonJoinPointer2;

		private JToggleButtonColumn jToggleButtonColumn;

		private JToggleButtonJoinPointer jToggleButtonJoinPointer3;

		private JToggleButtonJoinPointer jToggleButtonJoinPointer4;

		/**
		 * コンストラクタ
		 * 
		 * @param _colInfo
		 * @param columnSelectFlg
		 */
		public JPanelConditionColumn(DBObjectTableColumn _colInfo) {
			initialize();

			this.columnInfoTable = _colInfo;
			jToggleButtonColumn.setText(columnInfoTable.getName());

			// 主キーならBOLD表示
			if (columnInfoTable.isPrimaryKey()) {
				jToggleButtonColumn.setFont(jToggleButtonColumn.getFont()
						.deriveFont(java.awt.Font.BOLD));
			} else {
				jToggleButtonColumn.setFont(jToggleButtonColumn.getFont()
						.deriveFont(java.awt.Font.PLAIN));
			}

			// 生成時の選択リストカラムボタン状態を設定
			jToggleButtonColumn.setSelected(true);
		}

		private void initialize() {
			setAlignmentX(Component.LEFT_ALIGNMENT);
			setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));

			jToggleButtonJoinPointer1 = new JToggleButtonJoinPointer("left");
			jToggleButtonJoinPointer2 = new JToggleButtonJoinPointer("left2");
			jToggleButtonColumn = new JToggleButtonColumn();
			jToggleButtonJoinPointer3 = new JToggleButtonJoinPointer("right");
			jToggleButtonJoinPointer4 = new JToggleButtonJoinPointer("right2");

			// 右クリックのドラッグで選択を反転させる為のマウスリスナー
			jToggleButtonColumn.addMouseListener(new MouseListener() {
				public void mouseClicked(MouseEvent arg0) {
				}

				public void mouseEntered(MouseEvent arg0) {
					if ((arg0.getModifiers() & InputEvent.BUTTON3_MASK) != 0) {
						jToggleButtonColumn.setSelected(!jToggleButtonColumn
								.isSelected());
					}
				}

				public void mouseExited(MouseEvent arg0) {
				}

				public void mousePressed(MouseEvent arg0) {
					if ((arg0.getModifiers() & InputEvent.BUTTON3_MASK) != 0) {
						jToggleButtonColumn.setSelected(!jToggleButtonColumn
								.isSelected());
					}
				}

				public void mouseReleased(MouseEvent arg0) {
				}
			});

			this.add(jToggleButtonJoinPointer1);
			this.add(jToggleButtonJoinPointer2);
			this.add(jToggleButtonColumn);
			this.add(jToggleButtonJoinPointer3);
			this.add(jToggleButtonJoinPointer4);
		}

		public JToggleButtonJoinPointer searchJoinPointer(String side) {
			if (jToggleButtonJoinPointer1.getSide().equals(side)) {
				return jToggleButtonJoinPointer1;
			}
			if (jToggleButtonJoinPointer2.getSide().equals(side)) {
				return jToggleButtonJoinPointer2;
			}
			if (jToggleButtonJoinPointer3.getSide().equals(side)) {
				return jToggleButtonJoinPointer3;
			}
			if (jToggleButtonJoinPointer4.getSide().equals(side)) {
				return jToggleButtonJoinPointer4;
			}
			return null;
		}

		/**
		 * @return
		 */
		public DBObjectTableColumn getColumnInfoTable() {
			return columnInfoTable;
		}

		/**
		 * @return
		 */
		public boolean isSelectColumn() {
			return jToggleButtonColumn.isSelected();
		}

		/**
		 * @return
		 */
		public boolean isSelectJoinPointer() {
			return jToggleButtonJoinPointer1.isSelected()
					| jToggleButtonJoinPointer3.isSelected();
		}

		/**
		 * 選択カラムボタン
		 */
		public class JToggleButtonColumn extends JToggleButton {
			private static final long serialVersionUID = 2164596673577865176L;

			JToggleButtonColumn() {
				setMargin(new Insets(1, 1, 1, 1));
			}
		}

		/**
		 * 条件ボタン
		 */
		public class JToggleButtonJoinPointer extends JToggleButton {
			private static final long serialVersionUID = 8576629740563464503L;

			private String side;

			private JoinStatus joinStatus;

			private Point point = new Point();

			JToggleButtonJoinPointer(String side) {
				setJoinStatus(JoinStatus.OUTER_NONE);
				setMargin(new Insets(1, 1, 1, 1));
				addActionListener(whereActionListener);
				this.side = side;
				addMouseListener(joinPointerMouseListener);
				setFont(new java.awt.Font("Monospaced", java.awt.Font.PLAIN, 9));
			}

			public JPanelConditionTable getJPanelColumns() {
				return JPanelConditionTable.this;
			}

			public DBObjectTableColumn getColumnInfoTable() {
				return columnInfoTable;
			}

			/**
			 * Line2Dで使用するための座標系に変換したpointを取得します
			 * 
			 * @return
			 */
			public Point getConnecterPoint() {
				Point p = getLocationOnScreen();
				Point rp = getRootPane().getLocationOnScreen();

				int x = p.x - rp.x;
				int y = p.y - rp.y;

				point.setLocation(x + getWidth() / 2, y + getHeight() / 2);
				return point;

			}

			public String getSide() {
				return side;
			}

			public JoinStatus getJoinStatus() {
				return joinStatus;
			}

			public void setJoinStatus(JoinStatus joinStatus) {
				this.joinStatus = joinStatus;
				switch (this.joinStatus) {
				case OUTER_NONE:
					setText("・");
					break;
				case OUTER_EQUAL:
					setText("○");
					break;
				case OUTER_ALL:
					setText("●");
					break;

				default:
					break;
				}
			}
		}

	}
}
