package jdbcacsess.gui;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.swing.SwingUtilities;
import javax.swing.table.AbstractTableModel;

import jdbcacsess.JdbcacsessException;
import jdbcacsess.LogOutput;
import jdbcacsess.gui.cell.JDialogTextEditer;
import jdbcacsess.gui.cell.ViewColumn;
import jdbcacsess.gui.common.JDialogMessage;
import jdbcacsess.sql.ColumnInfoManageble;
import jdbcacsess.sql.ColumnInfoResult;
import jdbcacsess.sql.PrepareExecute;
import jdbcacsess.sql.PrepareExecuteStatement;
import jdbcacsess.sql.QueryExecuteListener;
import jdbcacsess.sql.QueryExecuteStatus;
import jdbcacsess.sql.SchemaTableName;
import jdbcacsess.sql.SqlAnalyze;
import jdbcacsess.sql.SqlExec;
import jdbcacsess.sql.SqlMappingInfo;

/**
 * 検索結果を表示するJTableに対するデータモデルを、AbstractTableModelを継承して実装する。
 * Vectorでカラム名及びデータを受け取り、ArrayListにそれぞれを編集する。 また、EditResultbleインプリメントのテーブル版である。
 */

/**
 * @author sima
 * 
 */
public class JTableSqlModel extends AbstractTableModel implements QueryExecuteListener, ColumnInfoManageble {

	private static final long serialVersionUID = 1L;

	private List<List<CellInfo>> detailList;

	private List<ColumnInfoResult> columnInfoResults;

	private SqlAnalyze sqlAnalyze;

	private int insertRowIndex = -1;

	private Connection con;

	private int heapMemoryCnt;

	private boolean selectUpdatable;
	
	/**
	 * init()を呼び出すだけ
	 * 
	 * @param con
	 */
	public JTableSqlModel(Connection con) {
		init();
		this.con = con;
		sqlAnalyze = new SqlAnalyze("");  // これは何だ。ひどい
	}

	/**
	 * データモデルの初期化を行う。QueryExecuteListenerの実装。
	 */
	public void init() {
		// syncは意味あるのか？
		detailList = Collections.synchronizedList(new ArrayList<List<CellInfo>>(1000));
	}

	/**
	 * 検索結果の見出しカラムを受け取り編集する。OutputResultbleの実装。
	 */
	public void setResultHeader(final ArrayList<ColumnInfoResult> headerData, final SqlAnalyze sqlAnalyze,final boolean selectUpdatable) {
		columnInfoResults = Collections.synchronizedList(headerData);
		JTableSqlModel.this.sqlAnalyze = sqlAnalyze;
		JTableSqlModel.this.selectUpdatable = selectUpdatable;
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				fireTableStructureChanged();
			}
		});
		heapMemoryCnt = 0;
	}

	/**
	 * 検索結果カラムの定義情報を返却します
	 * 
	 * @param columnIdx
	 *            カラムインデックス
	 * @return 検索結果カラムの定義情報
	 */
	public ColumnInfoResult getColumnInfo(int columnIdx) {
		return columnInfoResults.get(columnIdx);
	}

	/**
	 * 検索結果の行データを受け取り編集する。OutputResultbleの実装。
	 */
	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.sql.OutputResultble#setResultDetail(java.util.Vector)
	 */
	public synchronized void setResultDetail(ArrayList<Object> detailData) {
		final List<CellInfo> columnList = Collections.synchronizedList(new ArrayList<CellInfo>(detailData.size()));

		for (int i = 0; i < detailData.size(); i++) {
			CellInfo cellInfo = new CellInfo();
			cellInfo.setMode(CellMode.CELL_NON);
			cellInfo.setValue(detailData.get(i));
			if (getColumnClass(i) == Object.class) {
				// ColumnClassとしてjava.lang.Objectが設定されているので編集出来ません
				cellInfo.setMode(CellMode.CELL_DENYEDIT_IN_CELL);
			}
			columnList.add(cellInfo);
		}
		detailList.add(columnList);
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				fireTableRowsInserted(detailList.size() - 1, detailList.size() - 1);
			}
		});

		int cnt = detailList.size() % 5000;
		if ((cnt) == 0) {
			LogOutput.fine("waitします");
			try {
				wait();
			} catch (InterruptedException e) {
				LogOutput.fine("再開します");
			}
		}

		Runtime runtime = Runtime.getRuntime();

		final BigDecimal all = new BigDecimal(runtime.totalMemory());
		final BigDecimal used = new BigDecimal(runtime.totalMemory() - runtime.freeMemory());
		final BigDecimal bd = used.divide(all, 10, BigDecimal.ROUND_HALF_DOWN);

//		System.out.println("all=" + all + " used=" + used + "#" + bd + "#" + heapMemoryCnt);
		if (bd.compareTo(new BigDecimal(0.99)) > 0) {
			heapMemoryCnt++;
			if (heapMemoryCnt > 9) {
				heapMemoryCnt = 0;
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						JDialogMessage.infoDialog("ヒープメモリが不足しているようなので、Fetchを一時停止しました。\n"
								+ "「継続取得」か「取得中止」を指示してください。\n"
								+ "恒久的な対応は 起動時に「-Xmx???m」等を指定しヒープメモリを増やしてください", "ヒープメモリ不足で処理遅延");
					}
				});
				LogOutput.fine("waitします");
				try {
					wait();
				} catch (InterruptedException e) {
					LogOutput.fine("再開します");
				}
			}
		} else {
			heapMemoryCnt = 0;
		}
	}

	/**
	 * 検索終了を受信する。OutputResultbleの実装。
	 */
	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.sql.QueryExecuteListener#completeQuery(jdbcacsess.sql.QueryExecuteStatus)
	 */
	public void completeQuery(QueryExecuteStatus breakStatus) {

		if (breakStatus.isErr()) {
			return;
		}

		// 検索結果が０件の場合、見出しカラム数分の新規行を作る
		// if (detailList.isEmpty() && sqlAnalyze.isSelectUpdatable()) {
		if (detailList.isEmpty()) {

			List<CellInfo> newColumnList = Collections.synchronizedList(new ArrayList<CellInfo>());

			// ２行目は更新できない空行を用意する。Insert命令を発行させるには１行目から移動しないと駄目なので
			for (int i = 0; i < columnInfoResults.size(); i++) {
				CellInfo ci = new CellInfo();
				ci.setMode(CellMode.CELL_DELETE);
				ci.setValue("");
				newColumnList.add(ci);
			}
			detailList.add(newColumnList);
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					fireTableDataChanged();
				}
			});
		}
	}

	/**
	 * 更新対象のテーブル名を返却
	 * 
	 * @return 更新対象のテーブル名
	 */
	public SchemaTableName getSchemaTableName() {
		return sqlAnalyze.getSchemaTableName();
	}

	/**
	 * クラスを返却(ColumnsTableModel)
	 */
	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getColumnClass(int)
	 */
	@Override
	public Class<?> getColumnClass(int columnIndex) {
		return columnInfoResults.get(columnIndex).getColumnClass();
	}

	/**
	 * カラム名を返却(ColumnsTableModel)
	 */
	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getColumnName(int)
	 */
	@Override
	public String getColumnName(int column) {
		return columnInfoResults.get(column).getColumnName();
	}

	/**
	 * カラム数を返却(ColumnsTableModel)
	 */
	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	public int getColumnCount() {
		if (columnInfoResults == null) {
			return 0;
		}
		return columnInfoResults.size();
	}

	/**
	 * 行数を返却(ColumnsTableModel)
	 */
	/*
	 * (非 Javadoc)
	 * 
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	public int getRowCount() {
		return detailList.size();
	}

	/**
	 * 指定されたインデック位置のセル情報を返却します。
	 * 
	 * @param rowIndex
	 *            行インデックス
	 * @param columnIndex
	 *            列インデックス
	 * @return セル情報
	 */
	private CellInfo getCellInfo(int rowIndex, int columnIndex) {
		List<CellInfo> columnList = detailList.get(rowIndex);
		return columnList.get(columnIndex);
	}

	/**
	 * 指定されたインデック位置のセル情報のモードを変更します。
	 * 
	 * @param rowIdx
	 *            行インデックス
	 * @param colIdx
	 *            列インデックス
	 * @param cellMode
	 *            モード
	 */
	private void setCellInfo(int rowIdx, int colIdx, CellMode cellMode) {
		CellInfo cellInfo = getCellInfo(rowIdx, colIdx);
		cellInfo.setMode(cellMode);
	}

	/**
	 * 検索結果の指定項目を返却(ColumnsTableModel)
	 */
	public Object getValueAt(int rowIdx, int colIdx) {
		CellInfo cellInfo = getCellInfo(rowIdx, colIdx);

		return cellInfo.getValue();
	}

	/**
	 * 検索結果の指定状態を返却
	 */
	public CellMode getMode(int rowIndex, int columnIndex) {
		return getCellInfo(rowIndex, columnIndex).getMode();
	}

	/**
	 * 検索結果の指定状態を返却
	 */
	public boolean isDefaultInsert(int rowIndex, int columnIndex) {
		return getCellInfo(rowIndex, columnIndex).isDefaultInsert();
	}

	/**
	 * セルが編集可能かを返す(ColumnsTableModel)
	 */
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		if (selectUpdatable == false) {
			return false;
		}
		return getCellInfo(rowIndex, columnIndex).isCellEditable();
	}

	/**
	 * SQL文を解析した結果での、テーブル編集可能かを返す
	 * 
	 * @return テーブルを編集可能か
	 */
	public boolean isTableEditable() {
		return selectUpdatable;
	}

	/**
	 * デフォルトでINSERTするか？を反転させる
	 * 
	 * @param rowIndex
	 * @param columnIndex
	 */
	public void reverseDefaultInsert(int rowIndex, int columnIndex) {
		CellInfo ci = getCellInfo(rowIndex, columnIndex);
		boolean defaultInsert = !ci.isDefaultInsert();
		ci.setDefaultInsert(defaultInsert);

		fireTableCellUpdated(rowIndex, columnIndex);

	}

	/**
	 * セルデータの変更を受け取り、データベース更新コマンド(UPDATE)を発行する。 更新単位は１セル毎にUPDATEを発行する。
	 */
	@Override
	public void setValueAt(final Object aValue, final int rowIndex, final int columnIndex) {
		// ＤＢ排他に備えて別スレッドでＳＱＬ発行
		new Thread() {
			@Override
			public void run() {
				setValueAtUpdate(aValue, rowIndex, columnIndex);
			}
		}.start();
	}

	/**
	 * セルデータの変更を受け取り、データベース更新コマンド(UPDATE)を発行する。 １つの更新値を範囲でPDATEを発行する。
	 * 
	 * @param aValue
	 *            更新値
	 * @param rowIndexFrom
	 *            開始行
	 * @param columnIndexFrom
	 *            開始列
	 * @param rowIndexTo
	 *            終了行
	 * @param columnIndexTo
	 *            終了列
	 */
	public void setValueAt(final Object aValue, final int[] rowIndexs, final int[] columnIndexs) {
		// ＤＢ排他に備えて別スレッドでＳＱＬ発行
		new Thread() {
			@Override
			public void run() {
				for (int rowIndex = 0; rowIndex < rowIndexs.length; rowIndex++) {
					for (int columnIndex = 0; columnIndex < columnIndexs.length; columnIndex++) {
						setValueAtUpdate(aValue, rowIndexs[rowIndex], columnIndexs[columnIndex]);
					}
				}
			}
		}.start();
	}

	private void setValueAtUpdate(Object aValue, int rowIndex, int columnIndex) {
		if (aValue != null) {
			LogOutput.fine("rowIndex:" + rowIndex + " columnIndex:" + columnIndex + " aValue.getClass().getName():"
					+ aValue.getClass().getName());
		} else {
			LogOutput.fine("rowIndex:" + rowIndex + " columnIndex:" + columnIndex + " aValue:#null#");
		}
		// 挿入行データ作成中のデータ変更なら、モデルだけ変更する
		if (getMode(rowIndex, columnIndex) == CellMode.CELL_INSERT) {
			CellInfo cellInfo = getCellInfo(rowIndex, columnIndex);
			cellInfo.setValue(aValue);
			cellInfo.setDefaultInsert(false);
			fireTableCellUpdated(rowIndex, columnIndex);
			return;
		}

		// 変更があったカラムのSQLコマンドを編集。UPDATE句とSET句。
		final PrepareExecuteStatement pes = new PrepareExecuteStatement();
		pes.addStm("UPDATE " + sqlAnalyze.getSchemaTableName());
		pes.addStm(" SET " + (columnInfoResults.get(columnIndex)).getColumnName() + " = ?");
		pes.addParm(aValue, getColumnInfo(columnIndex));
		// 主キーの条件を編集する。
		editWherePhrase(pes, rowIndex);

		// update要求中にする
		setCellInfo(rowIndex, columnIndex, CellMode.CELL_UPDATE);
		fireTableCellUpdated(rowIndex, columnIndex);

		try {
			pes.prepare(con);
			int cnt = pes.executeUpdate();
			if (cnt != 1) {
				throw new Exception("更新件数が１件でありませんでした。件数=" + cnt);
			}
		} catch (SQLException e) {
			JDialogMessage.errorDialog(e, "カラム更新");
			return;
		} catch (Exception e) {
			JDialogMessage.errorDialog(e);
			return;
		}
		// 更新成功したのでデータモデルも更新
		CellInfo cellInfo = getCellInfo(rowIndex, columnIndex);
		cellInfo.setMode(CellMode.CELL_UPDATE_COMPLETE);
		cellInfo.setValue(aValue);
		cellInfo.setDefaultInsert(false);
		fireTableCellUpdated(rowIndex, columnIndex);
	}

	/**
	 * @param peu
	 * @param rowIndex
	 */
	private void editWherePhrase(PrepareExecute peu, int rowIndex) {
		// 主キーカラム名はcolInfoのisIsPrimeKeyを判定し、colInfo.getColumnName()から、
		// 値はgetValueAtメソッド経由で、detailListから取得する。
		peu.addStm(" WHERE ");
		boolean isFirst = true;
		for (int i = 0; i < columnInfoResults.size(); i++) {
			ColumnInfoResult colInfo = columnInfoResults.get(i);
			if (colInfo.isPrimaryKey()) {
				if (isFirst) {
					peu.addStm(" ");
					isFirst = false;
				} else {
					peu.addStm(" AND ");
				}
				peu.addStm(colInfo.getColumnName() + " = ?");
				peu.addParm(getCellInfo(rowIndex, i).getValue(), getColumnInfo(i));
			}
		}
	}

	/**
	 * 行削除を行う。
	 */
	public void rowDelete(final int[] rowIndexs) {
		new Thread() {
			@Override
			public void run() {
				for (int rowIndex : rowIndexs) {
					switch (getMode(rowIndex, 0)) {
					case CELL_INSERT:
						insertRowIndex = -1;
						detailList.remove(rowIndex);
						break;

					case CELL_DELETE:
					case CELL_DELETE_COMPLETE:
						break;

					default:
						for (CellInfo cellInfo : detailList.get(rowIndex)) {
							cellInfo.setMode(CellMode.CELL_DELETE);
						}
						PrepareExecuteStatement pes = new PrepareExecuteStatement();
						pes.addStm("DELETE FROM " + sqlAnalyze.getSchemaTableName());
						// 主キーの条件を編集する。
						editWherePhrase(pes, rowIndex);
						fireTableRowsDeleted(rowIndex, rowIndex);
						try {
							pes.prepare(con);
							int cnt = pes.executeUpdate();
							if (cnt != 1) {
								throw new Exception("削除件数が１件でありませんでした。件数=" + cnt);
							}
						} catch (SQLException e) {
							JDialogMessage.errorDialog(e, "行削除");
							return;
						} catch (Exception e) {
							JDialogMessage.errorDialog(e);
							return;
						}
						for (CellInfo cellInfo : detailList.get(rowIndex)) {
							cellInfo.setMode(CellMode.CELL_DELETE_COMPLETE);
						}
						break;
					}
					fireTableRowsDeleted(rowIndex, rowIndex);
				}
			}
		}.start();
	}

	/**
	 * データモデル上で行の追加を行う
	 * 
	 * @param orgRowIndex
	 * @param insertMode
	 *            追加の方法
	 */
	public void rowCopy(int orgRowIndex, RowInsertValueMode insertMode) {
		// 既にInsert行編集中の場合はなにもしない
		if (insertRowIndex != -1) {
			return;
		}

		List<CellInfo> newColumnList = Collections.synchronizedList(new ArrayList<CellInfo>());
		switch (insertMode) {
		case VALUEROWNEW:
			// 値付き新規行依頼の場合
			createInsertRowData(newColumnList);
			break;

		case ALLNULLROWNEW:
			// null新規行依頼の場合
			createInsertRowNull(newColumnList);
			break;

		case ROWCOPY:
			// 複写行依頼の場合
			for (CellInfo cellInfo : detailList.get(orgRowIndex)) {
				newColumnList.add(cellInfo.copy());
			}
			break;

		default:
			break;
		}
		insertRowIndex = orgRowIndex;
		detailList.add(insertRowIndex, newColumnList);

		fireTableDataChanged();
	}

	private void createInsertRowData(List<CellInfo> newColumnList) {
		for (int i = 0; i < columnInfoResults.size(); i++) {
			CellInfo ci = new CellInfo();
			Class<?> c = getColumnClass(i);
			try {
				if (c == null) {
					setInsertValue(ci, null);
				} else if (java.lang.String.class.isAssignableFrom(c)) {
					setInsertValue(ci, (getColumnInfo(i).isNullable() ? null : "a"));
				} else if (Number.class.isAssignableFrom(c) || Boolean.class.isAssignableFrom(c)) {
					Constructor<?> constructor = c.getConstructor(String.class);
					setInsertValue(ci, constructor.newInstance("0"));
				} else if (Date.class.isAssignableFrom(c) || Time.class.isAssignableFrom(c)
						|| Timestamp.class.isAssignableFrom(c)) {
					Long l = Long.valueOf(new Date().getTime());
					setInsertValue(ci, c.getConstructor(Long.TYPE).newInstance(l));
				} else if (Object.class == c) {
					setInsertValue(ci, null);
				} else {
					// setInsertValue(ci, c.newInstance());
					Object obj = c.newInstance();
					try {
						if (obj.toString() == null) {
							setInsertValue(ci, null);
						} else {
							setInsertValue(ci, obj);
						}
					} catch (NullPointerException e) {
						setInsertValue(ci, null);
					}
				}
			} catch (IllegalArgumentException e) {
				setInsertValue(ci, null);
				e.printStackTrace();
			} catch (SecurityException e) {
				setInsertValue(ci, null);
				e.printStackTrace();
			} catch (InstantiationException e) {
				setInsertValue(ci, null);
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				setInsertValue(ci, null);
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				setInsertValue(ci, null);
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				setInsertValue(ci, null);
				e.printStackTrace();
			}

			newColumnList.add(ci);
		}
	}

	private void createInsertRowNull(List<CellInfo> newColumnList) {
		for (int i = 0; i < columnInfoResults.size(); i++) {
			CellInfo ci = new CellInfo();
			setInsertValue(ci, null);
			newColumnList.add(ci);
		}
	}

	private void setInsertValue(CellInfo ci, Object value) {
		ci.setMode(CellMode.CELL_INSERT);
		ci.setValue(value);
	}

	/**
	 * 挿入編集中行(insertRowIndex)がしめす行でINSERTを発行する
	 * 
	 * @param selectRowIndex
	 */
	public void rowInsert(int selectRowIndex) {
		if (insertRowIndex == -1) {
			return;
		}
		LogOutput.fine("Insert対象行:" + insertRowIndex + " " + "選択行:" + selectRowIndex);
		if (selectRowIndex == insertRowIndex) {
			// カーソル位置行と挿入位置行が同一の場合は何もしない
			return;
		}

		// SQLコマンドを編集
		final PrepareExecuteStatement pes = new PrepareExecuteStatement();
		int cnt;
		pes.addStm("INSERT INTO " + sqlAnalyze.getSchemaTableName() + "(");
		cnt = 0;
		for (int i = 0; i < columnInfoResults.size(); i++) {
			if (!isDefaultInsert(insertRowIndex, i)) {
				// 挿入位置行の値を順次編集
				if (cnt != 0) {
					pes.addStm(",");
				}
				pes.addStm(columnInfoResults.get(i).getColumnName());
				cnt++;
			}
		}
		pes.addStm(") VALUES(");
		List<CellInfo> cellInfoList = detailList.get(insertRowIndex);
		cnt = 0;
		for (int i = 0; i < cellInfoList.size(); i++) {
			if (!isDefaultInsert(insertRowIndex, i)) {
				// 挿入位置行の値を順次編集
				if (cnt != 0) {
					pes.addStm(",");
				}
				pes.addStm("?");
				pes.addParm(cellInfoList.get(i).getValue(), getColumnInfo(i));
				cnt++;
			}
		}
		pes.addStm(")");

		new Thread() {
			@Override
			public void run() {
				try {
					// INSERTコマンドを発行
					pes.prepare(con);
					int cnt = pes.executeUpdate();
					if (cnt != 1) {
						throw new Exception("挿入件数が１件でありませんでした。件数=" + cnt);
					}
				} catch (SQLException e) {
					JDialogMessage.errorDialog(e, "行挿入");
					return;
				} catch (Exception e) {
					JDialogMessage.errorDialog(e);
					return;
				}
				// 挿入位置行の状態を挿入済みにする
				for (CellInfo cellInfo : detailList.get(insertRowIndex)) {
					cellInfo.setMode(CellMode.CELL_INSERT_COMPLETE);
				}
				// 挿入位置行をクリア
				insertRowIndex = -1;
				fireTableDataChanged();
			}
		}.start();
	}

	public synchronized void nextSearch() {
		notify();
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.sql.QueryExecuteListener#status(jdbcacsess.sql.QueryExecuteStatus)
	 */
	public void status(QueryExecuteStatus breakStatus) {
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.sql.QueryExecuteListener#errorException(java.lang.Exception)
	 */
	public void errorException(Exception ex) {
	}

	PrepareExecuteStatement peu;

	private ResultSet selectUniqColumn(int rowIndex, int col) throws NoDataException, SQLException {
		peu = new PrepareExecuteStatement();
		peu.addStm("SELECT " + getColumnName(col) + " FROM " + sqlAnalyze.getSchemaTableName());
		// 主キーの条件を編集する。
		editWherePhrase(peu, rowIndex);
		peu.prepare(con);
		ResultSet rs = peu.executeQuery();
		if (!rs.next()) {
			throw new NoDataException("データを検索できませんでした");
		}
		return rs;
	}

	static class NoDataException extends Exception {
		private static final long serialVersionUID = 1L;

		NoDataException(String s) {
			super(s);
		}
	}

	/**
	 * @param rowIndex
	 * @param colIndex
	 * @param viewColumn
	 */
	public void viewer(int rowIndex, int colIndex, ViewColumn viewColumn) {
		ResultSet rs = null;
		try {
			rs = selectUniqColumn(rowIndex, colIndex);
			viewColumn.setResultSet(rs);
			viewColumn.execute(peu.getStm());
		} catch (SQLException e) {
			JDialogMessage.errorDialog(e, "ビューワ");
		} catch (Exception e) {
			JDialogMessage.errorDialog(e);
		} finally {
			if (rs != null) {
				try {
					rs.close();
				} catch (SQLException e) {
					JDialogMessage.errorDialog(e, "結果セットのクローズ失敗");
				}
			}
		}
	}

	/**
	 * テキスト編集画面を呼び出します
	 * 
	 * @param rowIndex
	 *            データの行
	 * @param colIndex
	 *            データの列
	 */
	public void textEditer(int rowIndex, int colIndex) {
		ResultSet rs = null;
		JDialogTextEditer textEditer = new JDialogTextEditer();
		try {
			// 検索
			rs = selectUniqColumn(rowIndex, colIndex);
			String text = rs.getString(1);
			textEditer.setText(text);

			while (true) {
				// 表示
				textEditer.setVisible(true);

				if (textEditer.isTableUpdate()) {
					setValueAt(textEditer.getText(), rowIndex, colIndex);
				} else {
					textEditer.dialogClosing();
					break;
				}
			}
		} catch (SQLException e) {
			JDialogMessage.errorDialog(e, "テキスト編集");
		} catch (Exception e) {
			JDialogMessage.errorDialog(e);
		} finally {
			if (rs != null) {
				try {
					rs.close();
				} catch (SQLException e) {
					JDialogMessage.errorDialog(e, "結果セットのクローズ失敗");
				}
			}
			textEditer.dispose();
		}
	}

	/**
	 * @param selectedRow
	 * @param selectedCol
	 * @throws JdbcacsessException 
	 * @throws SQLException 
	 */
	public void sqlMapping(int selectedRow, int selectedCol) throws JdbcacsessException, SQLException {
		SqlMappingInfo sqlMappingInfo = new SqlMappingInfo();
		sqlMappingInfo.setDbms(SqlExec.getDatabaseProductName());
		sqlMappingInfo.setType(columnInfoResults.get(selectedCol).getColumnTypeName());
		sqlMappingInfo.keyDataLoad(sqlMappingInfo.getKeyValue());

		JFrameSqlMapping dlg = new JFrameSqlMapping();
		dlg.setSqlMappingInfo(sqlMappingInfo);
		dlg.setVisible(true);
	}

	public ColumnInfoResult getColumnInfo(int row, int column) {
		return columnInfoResults.get(column);
	}
}