/**
 * 
 */
package jdbcacsess.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.text.BadLocationException;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.swing.text.TabSet;
import javax.swing.text.TabStop;
import javax.swing.undo.CompoundEdit;
import javax.swing.undo.UndoManager;
import javax.swing.undo.UndoableEdit;

import jdbcacsess.LogOutput;
import jdbcacsess.gui.common.JDialogMessage;
import jdbcacsess.gui.common.JPopupMenuEdit;

/**
 * @author sima
 * 
 */
public class JTextPaneSqlStatement extends JTextPane {

	private static final long serialVersionUID = -287689674208777989L;

	private SimpleAttributeSet set;

	static Pattern pattern;
	static {
		String SQLPHASE[] = { "ABSOLUTE", "ACTION", "ADD", "ALL", "ALLOCATE",
				"ALTER", "AND", "ANY", "ARE", "AS", "ASC", "ASSERTION", "AT",
				"AUTHORIZATION", "AVG", "BEGIN", "BETWEEN", "BIT",
				"BIT_LENGTH", "BOTH", "BY", "CALL", "CASCADE", "CASCADED", "CASE",
				"CAST", "CATALOG", "CHAR", "CHARACTER", "CHAR_LENGTH",
				"CHARACTER_LENGTH", "CHECK", "CLOSE", "COALESCE", "COLLATE",
				"COLLATION", "COLUMN", "COMMIT", "CONNECT", "CONNECTION",
				"CONSTRAINT", "CONSTRAINTS", "CONTINUE", "CONVERT",
				"CORRESPONDING", "COUNT", "CREATE", "CROSS", "CURRENT",
				"CURRENT_DATE", "CURRENT_TIME", "CURRENT_TIMESTAMP",
				"CURRENT_USER", "CURSOR", "DATE", "DAY", "DEALLOCATE", "DEC",
				"DECIMAL", "DECLARE", "DEFAULT", "DEFERRABLE", "DEFERRED",
				"DELETE", "DESC", "DESCRIBE", "DESCRIPTOR", "DIAGNOSTICS",
				"DISCONNECT", "DISTINCT", "DOMAIN", "DOUBLE", "DROP", "ELSE",
				"END", "END-EXEC", "ESCAPE", "EXCEPT", "EXCEPTION", "EXEC",
				"EXECUTE", "EXISTS", "EXTERNAL", "EXTRACT", "FALSE", "FETCH",
				"FIRST", "FLOAT", "FOR", "FOREIGN", "FOUND", "FROM", "FULL",
				"GET", "GLOBAL", "GO", "GOTO", "GRANT", "GROUP", "HAVING",
				"HOUR", "IDENTITY", "IMMEDIATE", "IN", "INDICATOR",
				"INITIALLY", "INNER", "INPUT", "INSENSITIVE", "INSERT", "INT",
				"INTEGER", "INTERSECT", "INTERVAL", "INTO", "IS", "ISOLATION",
				"JOIN", "KEY", "LANGUAGE", "LAST", "LEADING", "LEFT", "LEVEL",
				"LIKE", "LOCAL", "LOWER", "MATCH", "MAX", "MIN", "MINUTE",
				"MODULE", "MONTH", "NAMES", "NATIONAL", "NATURAL", "NCHAR",
				"NEXT", "NO", "NOT", "NULL", "NULLIF", "NUMERIC",
				"OCTET_LENGTH", "OF", "ON", "ONLY", "OPEN", "OPTION", "OR",
				"ORDER", "OUTER", "OUTPUT", "OVERLAPS", "PAD", "PARTIAL",
				"POSITION", "PRECISION", "PREPARE", "PRESERVE", "PRIMARY",
				"PRIOR", "PRIVILEGES", "PROCEDURE", "PUBLIC", "READ", "REAL",
				"REFERENCES", "RELATIVE", "RESTRICT", "REVOKE", "RIGHT",
				"ROLLBACK", "ROWS", "SCHEMA", "SCROLL", "SECOND", "SECTION",
				"SELECT", "SESSION", "SESSION_USER", "SET", "SIZE", "SMALLINT",
				"SOME", "SPACE", "SQL", "SQLCODE", "SQLERROR", "SQLSTATE",
				"SUBSTRING", "SUM", "SYSTEM_USER", "TABLE", "TEMPORARY",
				"THEN", "TIME", "TIMESTAMP", "TIMEZONE_HOUR",
				"TIMEZONE_MINUTE", "TO", "TRAILING", "TRANSACTION",
				"TRANSLATE", "TRANSLATION", "TRIM", "TRUE", "UNION", "UNIQUE",
				"UNKNOWN", "UPDATE", "UPPER", "USAGE", "USER", "USING",
				"VALUE", "VALUES", "VARCHAR", "VARYING", "VIEW", "WHEN",
				"WHENEVER", "WHERE", "WITH", "WORK", "WRITE", "YEAR", "ZONE" };

		// キーワード検索用の正規表現作成
		StringBuffer sb = new StringBuffer("\\b(");
		for (int i = 0; i < SQLPHASE.length; i++) {
			if (i != 0) {
				sb.append("|");
			}
			sb.append(SQLPHASE[i]);
		}
		sb.append(")\\b");
		pattern = Pattern.compile(sb.toString());
	}

	private SimpleAttributeSet sqlHigh;

	private StyledDocument doc;

	private JPopupMenuEdit jPopupMenuEdit;

	private UndoManager undoManagerPopupMenu;

	public JTextPaneSqlStatement() {
		super();
		setBackground(Color.WHITE);

		doc = getStyledDocument();
		doc.addDocumentListener(new javax.swing.event.DocumentListener() {
			public void removeUpdate(DocumentEvent e) {
				// System.out.println("removeUpdate:" + e.toString());
				addEventEdit(e);
			}

			public void insertUpdate(DocumentEvent e) {
				// System.out.println("insertUpdate:" + e.toString());
				addEventEdit(e);
			}

			public void changedUpdate(DocumentEvent e) {
				// System.out.println("changedUpdate:" + e.toString());
			}

			private void addEventEdit(DocumentEvent e) {
				if (UndoableEdit.class.isAssignableFrom(e.getClass())) {
					UndoableEdit anEdit = (UndoableEdit) e;

					if (compoundEdit != null) {
						LogOutput
								.fine("   compoundEdit.addEdit=>" + anEdit);
						compoundEdit.addEdit(anEdit);
					} else {
						LogOutput.fine("   undoManagerPopupMenu.addEdit=>"
								+ anEdit);
						undoManagerPopupMenu.addEdit(anEdit);
					}
				}// elseになるのは、undo/redoでの更新の時

				setAttr(pattern, sqlHigh);
			}
		});

		// ポップアップメニューの登録
		jPopupMenuEdit = new JPopupMenuEdit(this);

		// ポップアップメニューのUndoManageを取り出し、アンドゥリスナーを削除する
		undoManagerPopupMenu = jPopupMenuEdit.getUndoManager();
		doc.removeUndoableEditListener(undoManagerPopupMenu);

		// キーワード部分以外の属性作成
		set = new SimpleAttributeSet();
		Font font = JFrameSetting.getGuiFont();

		StyleConstants.setFontSize(set, font.getSize());
//		StyleConstants.setLeftIndent(set, 0.5F);
		StyleConstants.setLeftIndent(set, 0F);
		StyleConstants.setForeground(set, Color.BLACK);
		StyleConstants.setBackground(set, Color.WHITE);
		StyleConstants.setFontFamily(set, font.getFamily());
//		StyleConstants.setLineSpacing(set, -0.15f);
		StyleConstants.setLineSpacing(set, 0F);

		int tabLength = getFontMetrics(font).charWidth('M') * 4;
		TabStop[] tabs = new TabStop[100];
		for (int j = 0; j < tabs.length; j++) {
			tabs[j] = new TabStop((j + 1) * tabLength);
		}
		StyleConstants.setTabSet(set, new TabSet(tabs));

		setParagraphAttributes(set, true);

		// キーワード部分の属性作成
		sqlHigh = new SimpleAttributeSet();
		StyleConstants.setForeground(sqlHigh, Color.BLUE);
	}

	public void insertMenuItem(Component menuItem, int index) {
		jPopupMenuEdit.insert(menuItem, index);
	}

	/**
	 * @param pattern
	 * @param sqlHigh
	 */
	private void setAttr(final Pattern pattern, final SimpleAttributeSet sqlHigh) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try {
					doc.setCharacterAttributes(0, doc.getLength(), set, false);
					Matcher m = pattern.matcher(doc.getText(0, doc.getLength())
							.toUpperCase());
					m.reset();
					while (m.find()) {
						for (int i = 0; i <= m.groupCount(); i++) {
							doc.setCharacterAttributes(m.start(i), m.end(i)
									- m.start(i), sqlHigh, false);
						}
					}
				} catch (BadLocationException e) {
					JDialogMessage.errorDialog(e);
				}
			}
		});
	}

	/**
	 * @param str
	 */
	public void insertCurrentPosition(String str) {
		try {
			doc.insertString(getCaretPosition(), str, set);
		} catch (BadLocationException e) {
			JDialogMessage.errorDialog(e);
		}
		requestFocusInWindow();
	}

	private CompoundEdit compoundEdit;

	public void setText(String str) {
		compoundEdit = new CompoundEdit();

		super.setText(str);

		compoundEdit.end();
		requestFocusInWindow();

		undoManagerPopupMenu.addEdit(compoundEdit);
		compoundEdit = null;
	}

	public void undoDiscard() {
		undoManagerPopupMenu.discardAllEdits();
	}
}
