/**
 * @author sss
 *
 */
package jdbcacsess.gui;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.io.IOException;
import java.sql.DatabaseMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.TreeMap;

import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.ToolTipManager;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.ExpandVetoException;
import javax.swing.tree.TreePath;

import jdbcacsess.PrefixStringBuffer;
import jdbcacsess.createdata.JFrameCreateDataSetting;
import jdbcacsess.gui.common.JDialogMessage;
import jdbcacsess.gui.common.JPopupMenuCnageUI;
import jdbcacsess.gui.common.PopupKeybordAndMouse;
import jdbcacsess.sql.ColumnInfoParameter;
import jdbcacsess.sql.ColumnInfoTable;
import jdbcacsess.sql.DBObject;
import jdbcacsess.sql.DBObjectsRoot;
import jdbcacsess.sql.SchemaTableName;
import jdbcacsess.sql.SqlExec;
import jdbcacsess.sql.DBObjectProcedures.DBObjectProcedure;
import jdbcacsess.sql.DBObjectSchemas.DBObjectSchema;
import jdbcacsess.sql.DBObjectTableColumns.DBObjectTableColumn;
import jdbcacsess.sql.DBObjectTables.DBObjectTable;

/**
 * @author sima
 * 
 */
public class JTreeDataBaseObjects extends JTree implements Observer  {

	/**
	 * 
	 */
	private static final long serialVersionUID = -294660305476207747L;

	private DefaultMutableTreeNode rootTreeNode;

	private JTextPaneSqlStatement sqlStatement;

	private JPopupMenuCnageUI jPopupMenuTable = null;

	private JMenuItem jMenuItemOrderPrimeKey = null;

	private JMenuItem jMenuItemNoOrder = null;

	private JMenuItem jMenuItemStored = null;

	private JMenuItem jMenuItemTreeUpdate = null;

	private DefaultTreeModel treeModel;

	private JMenuItem jMenuItemCreateData = null;

	private JMenuItem jMenuItemName = null;

	private JMenuItem jMenuItemEditSql = null;

	public void update(Observable o, Object sqlStatement) {
		this.sqlStatement = (JTextPaneSqlStatement) sqlStatement;
		if (csft != null) {
			csft.setSqlStatement(this.sqlStatement);
		}
	}

	public JTreeDataBaseObjects() {
		super();
		initialize();
	}

	private void initialize() {
//		setRowHeight(getFontMetrics(CellRendererDataBaseObject.treeFontPrim).getHeight());
		setBackground(Color.WHITE);
		setModel(null);
		setToggleClickCount(2);
		
		addTreeWillExpandListener(new javax.swing.event.TreeWillExpandListener() {
			public void treeWillCollapse(TreeExpansionEvent event) throws ExpandVetoException {
			}

			public void treeWillExpand(TreeExpansionEvent event) throws ExpandVetoException {
				DefaultMutableTreeNode current = (DefaultMutableTreeNode) event.getPath().getLastPathComponent();
				DefaultMutableTreeNode child = (DefaultMutableTreeNode) current.getFirstChild();

				if (child.getUserObject()!=null){
					return;
				}
				
				//ダミーがぶら下がってるので、動的にノード作成
				current.removeAllChildren();
				DBObject currentDBObject = (DBObject) current.getUserObject();

				try {
					DatabaseMetaData dmd = SqlExec.getDatabaseMetaData();

					for (DBObject o : currentDBObject.getChildren(dmd).getDBObjects()) {
						DefaultMutableTreeNode node = new DefaultMutableTreeNode(o);
						if (!o.isBottom()){
							node.add(new DefaultMutableTreeNode(null));
						}
						current.add(node);
					}
					treeModel.nodeStructureChanged(current);
					
				} catch (SQLException e) {
					JDialogMessage.errorDialog(e, "ツリー表示の情報取得");
				}
			}

		});

		addMouseListener(new MouseAdapter() {
			@Override
			public void mouseReleased(java.awt.event.MouseEvent e) {
				// 中ボタンのクリック判定
				if (e.getButton() == 2) {
					transColumnName(getSelectionPaths());
				}
			}
		});

		setCellRenderer(new CellRendererDataBaseObject());
		new PopupKeybordAndMouse(this, getJPopupMenuTable());
		ToolTipManager.sharedInstance().registerComponent(this);

		// ポップアップメニューアイテムの可視制御
		jPopupMenuTable.addPopupMenuListener(new PopupMenuListener() {
			public void popupMenuWillBecomeVisible(PopupMenuEvent arg0) {

				setAllMenuItemEnabled(true);

				boolean tblFlg = false;
				boolean colFlg = false;
				boolean prcFlg = false;

				TreePath[] tps = getSelectionPaths();
				if (tps==null){
					setAllMenuItemEnabled(false);
					return;
				}
				
				for (TreePath treePath : getSelectionPaths()) {
					if (isDBObject(treePath, DBObjectTable.class)) {
						tblFlg = true;
					}else if (isDBObject(treePath, DBObjectTableColumn.class)) {
						colFlg = true;
					}else if (isDBObject(treePath, DBObjectProcedure.class)) {
						prcFlg = true;
					}else{
						// それ以外が選択されていたら何も出来なくする
						setAllMenuItemEnabled(false);
						return;
					}
				}
				
				if ( ( tblFlg & colFlg & prcFlg ) || ( !tblFlg & !colFlg & !prcFlg )) {
					// 全種類選択又は全種類未選択
					setAllMenuItemEnabled(false);
					return;
				}
				
				if ( !(tblFlg | colFlg) | prcFlg ) {
					jMenuItemOrderPrimeKey.setEnabled(false);
					jMenuItemNoOrder.setEnabled(false);
					jMenuItemEditSql.setEnabled(false);
				}
				
				if ( tblFlg | colFlg | !prcFlg ) {
					jMenuItemStored.setEnabled(false);
				}else{
					if ( prcFlg && getSelectionCount()!=1) {
						jMenuItemStored.setEnabled(false);
					}
				}
				
				if (!tblFlg | colFlg | prcFlg ) {
					jMenuItemCreateData.setEnabled(false);
				}
				
			}
			private void setAllMenuItemEnabled(boolean enable){
				jMenuItemOrderPrimeKey.setEnabled(enable);
				jMenuItemNoOrder.setEnabled(enable);
				jMenuItemStored.setEnabled(enable);
				jMenuItemEditSql.setEnabled(enable);
				jMenuItemCreateData.setEnabled(enable);
			}

			public void popupMenuWillBecomeInvisible(PopupMenuEvent arg0) {
			}

			public void popupMenuCanceled(PopupMenuEvent arg0) {
			}
		});
	}

	/**
	 * 指定されたツリーパスのオブジェクト名称を転送する
	 * @param treePaths
	 */
	protected void transColumnName(TreePath[] treePaths) {
		if (treePaths == null) {
			return;
		}

		PrefixStringBuffer psb = new PrefixStringBuffer(" ", ", ");

		for (TreePath treePath : treePaths) {
			String selectItem = getLastPathUserObject(treePath).toString();

			psb.appendItem(selectItem);
		}

		sqlStatement.insertCurrentPosition(psb.getString());
	}

	/**
	 * 指定されたツリーパスの末端のオブジェクトを返却します
	 * 
	 * @param treePath
	 */
	private DBObject getLastPathUserObject(TreePath treePath) {
		DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) treePath.getLastPathComponent();
		return (DBObject)treeNode.getUserObject();
	}

	/**
	 * 指定されたツリーパスの末端オブジェクトが指定されたクラスであるか判定します
	 * 
	 * @param treePath
	 *            判定したいツリーパス
	 * @param cls
	 *            比較したいクラス
	 * 
	 * @return 指定されたクラスの場合はtrue
	 */
	private boolean isDBObject(TreePath treePath, Class<?> cls) {
		DefaultMutableTreeNode symbol = (DefaultMutableTreeNode) treePath.getLastPathComponent();
		if (symbol.getUserObject().getClass() == cls) {
			return true;
		}
		return false;
	}

	/**
	 * スキーマ及びテーブルのツリーをＤＢから検索して作成する。
	 * 
	 * @throws SQLException
	 */
	public void setTree(){
		DatabaseMetaData dmd;
		try {
			dmd = SqlExec.getDatabaseMetaData();
			DBObject root = new DBObjectsRoot(dmd).getDBObjects().get(0);
			
			rootTreeNode = new DefaultMutableTreeNode(root);
			
			for (DBObject dbObject : root.getChildren(dmd).getDBObjects()) {
				DefaultMutableTreeNode node = new DefaultMutableTreeNode(dbObject);
				rootTreeNode.add(node);
				if (!dbObject.isBottom()){
					node.add(new DefaultMutableTreeNode(null));
				}
			}

			treeModel = new DefaultTreeModel(rootTreeNode);
			setModel(treeModel);

		} catch (SQLException e) {
			JDialogMessage.errorDialog(e, "ツリー表示の情報取得");
		}
		
	}
	

	public void clearTree() {
		treeModel.setRoot(null);
		treeModel.reload();
	}

	public void actionPerformedReload() {
		TreePath[] treePaths = getSelectionPaths();
		if (treePaths == null) {
			setTree();
			return;
		}
		for (TreePath treePath : treePaths) {
			DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) treePath.getLastPathComponent();
			treeNode.removeAllChildren();
			DefaultMutableTreeNode dummy = new DefaultMutableTreeNode(null);
			treeNode.add(dummy);
			//削除をモデルに通知し。ダミーノードが表示されないように、折りたたむ。
			treeModel.nodeStructureChanged(treeNode);
			collapsePath(treePath);
		}

	}

	/**
	 * ツリーパス内を上位方向へ遡り、指定されたクラスを持つノードを探す。
	 * 
	 * @param treePath
	 *            ツリーパス
	 * @param cls
	 *            DBObjectを継承した検索対象のクラス
	 * @return 見つかったDBObject
	 */
	private Object getDBObject(TreePath treePath, Class<?> cls) {

		DefaultMutableTreeNode symbol = (DefaultMutableTreeNode) treePath.getLastPathComponent();

		Object[] obj = symbol.getUserObjectPath();
		if (obj.length - 2 < 0) {
			return null;
		}
		for (int i = obj.length - 2; i >= 0; i--) {
			if (obj[i].getClass() == cls) {
				return obj[i];
			}
		}
		return null;
	}

	/**
	 * テーブルを示すツリーパスから、スキーマ名・テーブル名を取得し、 SchemaTableNameとして返却する
	 * 
	 * @param tableTreePath
	 * @return スキーマ名・テーブル名
	 */
	private SchemaTableName getSchemaTable(TreePath tableTreePath) {
		//DBObjectSchema schema = (DBObjectSchema) getDBObject(tableTreePath, DBObjectSchema.class);
		DBObjectTable table = (DBObjectTable) getLastPathUserObject(tableTreePath);
		SchemaTableName stn = new SchemaTableName(table.getTableSchema(), table.getTableName());
		return stn;
	}

	/**
	 * 現在選択されている複数ツリーパスから、テーブル名をリストで返却する
	 * 
	 * @return スキーマテーブル名のリスト
	 */
	private ArrayList<SchemaTableName> getSelectSchemaTables() {
		ArrayList<SchemaTableName> list = new ArrayList<SchemaTableName>();
		for (TreePath treePath : getSelectionPaths()) {
			DBObject dbobject = getLastPathUserObject(treePath);

			SchemaTableName stn = dbobject.getSchemaTableName();
			if (list.indexOf(stn) == -1) {
				list.add(stn);
			}
			
		}
		int i=0;
		for (SchemaTableName name : list) {
			name.setAliasName("T" + ++i );
		}
		return list;

	}

	/**
	 * カンマ区切りのカラム名リストを作成する
	 * 
	 * @param list
	 * @return SQLのカラム名句
	 */
	private StringBuffer getSelColumsListPhrase(ArrayList<SchemaTableName> list) {

		// 現在選択されている複数ツリーパスから、カラム名をリストで返却する
		ArrayList<String> columns = new ArrayList<String>();
		for (TreePath treePath : getSelectionPaths()) {
			if (! isDBObject(treePath, DBObjectTableColumn.class)) {
				continue;
			}
			DBObject dbobject = getLastPathUserObject(treePath);

			int index = list.indexOf(dbobject.getSchemaTableName());

			if (index > -1 ){
				columns.add(list.get(index).getAliasName() + "." + dbobject.getName());
			}else{
				columns.add("???" + "." + dbobject.getName());
			}
		}
		
		if (columns.size() == 0) {
			return new StringBuffer("*");
		}

		PrefixStringBuffer psb = new PrefixStringBuffer("",", ");
		for (String column : columns) {
			psb.appendItem(column);
		}
		return psb.getStringBuffer();
	}

	/**
	 * スキーマ・テーブルのリストから、カンマ区切りテーブル名のFrom句を作成する
	 * 
	 * @param list
	 * @return SQLのFrom句
	 */
	private StringBuffer getFromPhrase(ArrayList<SchemaTableName> list) {
		if (list.size() > 2) {
			JOptionPane.showMessageDialog(null, "2つ以上のテーブルが選択されました。\n" + "JOIN条件作成では最初の2テーブルが有効です");
		}
		PrefixStringBuffer psb = new PrefixStringBuffer("\nFROM ",", ");
		for (SchemaTableName stn : list) {
			psb.appendItem(stn.getCompleteTableName() + " " + stn.getAliasName());
		}
		return psb.getStringBuffer();
	}

	/**
	 * スキーマ名・テーブル名のリストから、カンマ区切りテーブル名のWher句を作成する
	 * 
	 * @param list
	 * @return SQLのFrom句
	 */
	private String getWherePhrase(ArrayList<SchemaTableName> list) {
		if (list.size() < 2) {
			return "";
		}
		StringBuffer str = new StringBuffer();
		ArrayList<String> editKey = new ArrayList<String>();
		createPrimaryAndColumn(editKey, str, list.get(0), list.get(1));
		createPrimaryAndColumn(editKey, str, list.get(1), list.get(0));

		if (editKey.size() == 0) {
			return "";
		}

		return "\nWHERE " + str.substring(5);
	}

	/**
	 * @param editKey
	 * @param str
	 * @param primStn
	 * @param colStn
	 */
	private void createPrimaryAndColumn(ArrayList<String> editKey, StringBuffer str, SchemaTableName primStn,
			SchemaTableName colStn) {
		ArrayList<String> primList = null;
		ArrayList<ColumnInfoTable> colList = null;

		try {
			primList = SqlExec.getPrimaryKeys(primStn.getSchemaName(), primStn.getTableName());
			colList = SqlExec.getColumnMames(colStn);
		} catch (SQLException e) {
			JDialogMessage.errorDialog(e, "カラム情報取得で失敗");
			return;
		}

		for (ColumnInfoTable col : colList) {
			for (String prim : primList) {
				if (col.getColumnName().equals(prim)) {
					if (!editKey.contains(col.getColumnName())) {
						str.append(" AND " + primStn.getAliasName() + "." + prim
								+ " = " + colStn.getAliasName() + "." + col.getColumnName());
						editKey.add(col.getColumnName());
					}
				}
			}
		}
	}

	/**
	 * @param list
	 * @return
	 */
	private StringBuffer getOrdeByPhrase(ArrayList<SchemaTableName> list) {
		PrefixStringBuffer psb = new PrefixStringBuffer("\nORDER BY ",", ");
		for (SchemaTableName stn : list) {
			try {
				for (String key : SqlExec.getPrimaryKeys(stn.getSchemaName(), stn.getTableName())) {
					psb.appendItem(stn.getAliasName()+"."+key);
				}
			} catch (Exception ex) {
			}
		}
		return psb.getStringBuffer();
	}

	/**
	 * SELECT文を編集し、ＳＱＬ入力エリアに転送する
	 * 
	 * @param isOrderBy
	 */
	private void actionPerformedSelect(boolean isOrderBy) {
		if (sqlStatement == null) {
			return;
		}

		ArrayList<SchemaTableName> schemaTables = getSelectSchemaTables();
		
		StringBuffer text = new StringBuffer("SELECT ");
		text.append(getSelColumsListPhrase(schemaTables));
		text.append(getFromPhrase(schemaTables));
		text.append(getWherePhrase(schemaTables));

		if (isOrderBy) {
			text.append(getOrdeByPhrase(schemaTables));
		}

		text.append("\n");

		sqlStatement.setText(text.toString());
		sqlStatement.requestFocus();
	}

	/**
	 * ストアド呼出文作成
	 */
	private void actionPerformedStored() {
		if (sqlStatement == null) {
			return;
		}

		TreePath treePath = getSelectionPath();
		DBObjectProcedure p = (DBObjectProcedure)getLastPathUserObject(treePath);
		DBObjectSchema s = (DBObjectSchema)getDBObject(treePath, DBObjectSchema.class);
		StringBuffer text = new StringBuffer("call " + new SchemaTableName(s.getSchemaName(), p.getProcedureName()).getCompleteTableName() + "(");
		PrefixStringBuffer psb = new PrefixStringBuffer("",",");
		try {
			List<ColumnInfoParameter> list = SqlExec.getProcedureColumns(p);
			for (int i=0;i<list.size();i++){
				ColumnInfoParameter cip = list.get(i);
				if (i==0 && cip.isModeReturn()){
					text.insert(0, "?=" );
				}else{
					if (cip.isModeReturn()){
						break;
					}
					psb.appendItem("?");
				}
			}

			text.append(psb.getStringBuffer());
			text.append(")\n");

			sqlStatement.setText(text.toString());
			sqlStatement.requestFocus();
			if (p.getDuplicateCnt() > 1){
				JDialogMessage.infoDialog("オーバロードされているので、引数の数を正しく判断出来ません。\n" +
						"よって引数の ? は、恐らく間違っています。編集し直して下さい。", "引数が誤っているので修正が必要です");
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
	}
	/**
	 * This method initializes jPopupMenu
	 * 
	 * @return javax.swing.JPopupMenu
	 * @uml.property name="jPopupMenu"
	 */
	private JPopupMenuCnageUI getJPopupMenuTable() {
		if (jPopupMenuTable == null) {
			jPopupMenuTable = new JPopupMenuCnageUI();
			jPopupMenuTable.add(getJMenuItemOrderPrimeKey());
			jPopupMenuTable.add(getJMenuItemNoOrder());
			jPopupMenuTable.add(getJMenuItemStored());
			jPopupMenuTable.add(getJMenuItemEditSql());
			jPopupMenuTable.add(getJMenuItemName());
			jPopupMenuTable.addSeparator();
			jPopupMenuTable.add(getJMenuItemCreateData());
			jPopupMenuTable.addSeparator();
			jPopupMenuTable.add(getJMenuItemTreeUpdate());
		}
		return jPopupMenuTable;
	}

	/**
	 * This method initializes jMenuItem
	 * 
	 * @return javax.swing.JMenuItem
	 * @uml.property name="jMenuItemOrderPrimeKey"
	 */
	private JMenuItem getJMenuItemOrderPrimeKey() {
		if (jMenuItemOrderPrimeKey == null) {
			jMenuItemOrderPrimeKey = new JMenuItem();
			jMenuItemOrderPrimeKey.setText("全件検索文（主キー順）");
			jMenuItemOrderPrimeKey.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					actionPerformedSelect(true);
				}
			});
		}
		return jMenuItemOrderPrimeKey;
	}

	/**
	 * This method initializes jMenuItem1
	 * 
	 * @return javax.swing.JMenuItem
	 * @uml.property name="jMenuItemNoOrder"
	 */
	private JMenuItem getJMenuItemNoOrder() {
		if (jMenuItemNoOrder == null) {
			jMenuItemNoOrder = new JMenuItem();
			jMenuItemNoOrder.setText("全件検索文（無指定順）");
			jMenuItemNoOrder.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					actionPerformedSelect(false);
				}
			});
		}
		return jMenuItemNoOrder;
	}

	/**
	 * This method initializes jMenuItem1
	 * 
	 * @return javax.swing.JMenuItem
	 * @uml.property name="jMenuItemNoOrder"
	 */
	private JMenuItem getJMenuItemStored() {
		if (jMenuItemStored == null) {
			jMenuItemStored = new JMenuItem();
			jMenuItemStored.setText("ストアド呼出文");
			jMenuItemStored.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					actionPerformedStored();
				}
			});
		}
		return jMenuItemStored;
	}

	/**
	 * This method initializes jMenuItemTreeUpdate
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemTreeUpdate() {
		if (jMenuItemTreeUpdate == null) {
			jMenuItemTreeUpdate = new JMenuItem();
			jMenuItemTreeUpdate.setText("定義再取込");
			jMenuItemTreeUpdate.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					actionPerformedReload();
				}
			});
		}
		return jMenuItemTreeUpdate;
	}

	/**
	 * This method initializes jMenuItem
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemCreateData() {
		if (jMenuItemCreateData == null) {
			jMenuItemCreateData = new JMenuItem();
			jMenuItemCreateData.setText("データ生成…");
			jMenuItemCreateData.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					actionPerformedCreateData(e);
				}
			});
		}
		return jMenuItemCreateData;
	}

	/**
	 * @param e
	 */
	private void actionPerformedCreateData(ActionEvent e) {
		for (TreePath treePath : getSelectionPaths()) {
			if (isDBObject(treePath, DBObjectTable.class)) {
				try {
					JFrameCreateDataSetting dlg = new JFrameCreateDataSetting(getSchemaTable(treePath));
					dlg.setVisible(true);
				} catch (IOException e1) {
					JDialogMessage.errorDialog(e1);
				}
			}
		}
	}

	/**
	 * This method initializes jMenuItemColumnName
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemName() {
		if (jMenuItemName == null) {
			jMenuItemName = new JMenuItem();
			jMenuItemName.setText("定義名　　　　　中ボタン");
			jMenuItemName.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					transColumnName(getSelectionPaths());
				}
			});
		}
		return jMenuItemName;
	}

	/**
	 * This method initializes jMenuItemEditSql
	 * 
	 * @return javax.swing.JMenuItem
	 */
	private JMenuItem getJMenuItemEditSql() {
		if (jMenuItemEditSql == null) {
			jMenuItemEditSql = new JMenuItem();
			jMenuItemEditSql.setText("検索文生成…");
			jMenuItemEditSql.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					actionPerformedEditSql();
				}
			});
		}
		return jMenuItemEditSql;
	}

	private JFrameCreateSqlFromTree csft = null;

	protected void actionPerformedEditSql() {
		if (csft == null) {
			csft = new JFrameCreateSqlFromTree();
			csft.setSqlStatement(this.sqlStatement);
		}

		TreeMap<SchemaTableName, ArrayList<DBObjectTableColumn>> selTablesMap = new TreeMap<SchemaTableName, ArrayList<DBObjectTableColumn>>();
		TreeMap<SchemaTableName, ArrayList<DBObjectTableColumn>> allTablesMap = new TreeMap<SchemaTableName, ArrayList<DBObjectTableColumn>>();

		for (TreePath treePath : getSelectionPaths()) {
			DBObject dbobject = getLastPathUserObject(treePath);
			SchemaTableName stn = dbobject.getSchemaTableName();

			// 選択されたカラムすべてを作成する。テーブルだけが選択された場合は、全カラムを意味する空で作成する。
			ArrayList<DBObjectTableColumn> selColumnsList = selTablesMap.get(stn);
			if (selTablesMap.get(stn) == null) {
				selColumnsList = new ArrayList<DBObjectTableColumn>();
				selTablesMap.put(stn,selColumnsList);
			}
			if (isDBObject(treePath, DBObjectTableColumn.class)) {
				selColumnsList.add((DBObjectTableColumn) getLastPathUserObject(treePath));
			}

			// 定義されたカラムすべてを作成する
			ArrayList<DBObjectTableColumn> allColumnsList = selTablesMap.get(stn);
			if (allTablesMap.get(stn) == null) {
				allColumnsList = new ArrayList<DBObjectTableColumn>();
				allTablesMap.put(stn,allColumnsList);

			
				TreePath treePathTable=null;
				if (isDBObject(treePath, DBObjectTable.class)) {
					treePathTable = treePath;
				}else{
					treePathTable = treePath.getParentPath();
				}
				//ノードから全ての子ノードを取得し格納
				DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) treePathTable.getLastPathComponent();
			   for (Enumeration<DefaultMutableTreeNode> e = treeNode.children(); e.hasMoreElements();){
				   DefaultMutableTreeNode treeNodeChild = e.nextElement();
					allColumnsList.add((DBObjectTableColumn) treeNodeChild.getUserObject());
			   }
			}
		}


		csft.setTablemap(selTablesMap, allTablesMap);

		csft.setVisible(true);
	}

}
