package jdbcacsess.gui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.event.InternalFrameEvent;

public class TabWinManager {
	abstract class Pane {

		abstract void addJPanel(JPanel panel);

		abstract JComponent getJComponentPane();

		abstract JPanel getSelectedJPanel();

		abstract boolean isTabWinModeEqual(TabWinMode tw);

		abstract JComponent newJComponentPane();

		abstract void removePanel(JPanel panel);

		abstract void setPaneTitleName(JPanel panel, String PaneTitlename);

		abstract void setSelectPanel(JPanel panel);

		abstract void sortWindow();
	}

	class PaneTab extends Pane {

		private JTabbedPane jTabbedPane = null;

		@Override
		void addJPanel(JPanel panel) {
			jTabbedPane.addTab(panel.getName(), panel);
		}

		@Override
		JComponent getJComponentPane() {
			return jTabbedPane;
		}

		@Override
		JPanel getSelectedJPanel() {
			return (JPanel) jTabbedPane.getSelectedComponent();
		}

		@Override
		boolean isTabWinModeEqual(TabWinMode tw) {
			if (tw == TabWinMode.TAB) {
				return true;
			}
			return false;
		}

		@Override
		JComponent newJComponentPane() {
			jTabbedPane = new JTabbedPane();
			jTabbedPane.setFont(new java.awt.Font("Dialog",
					java.awt.Font.PLAIN, 12));

			jTabbedPane.addChangeListener(new javax.swing.event.ChangeListener() {
				public void stateChanged(javax.swing.event.ChangeEvent e) {
					// 選択された時ではなく、状態変化した時なので、削除した時もイベントが発生する
					Component c = jTabbedPane.getSelectedComponent();
					if (c != null) {
						if (c instanceof JPanelExecSql) {
							JPanelExecSql pes = (JPanelExecSql) c;
							sqlStatementObserver.sqlStatementNotifyObservers(pes.getSqlStatement());
						} else {
							sqlStatementObserver.sqlStatementNotifyObservers(null);
						}
					}
				}
			});
			return jTabbedPane;
		}

		@Override
		void removePanel(JPanel panel) {
			if (panel == null) {
				return;
			}
			jTabbedPane.remove(panel);
		}

		@Override
		void setPaneTitleName(JPanel pane, String panelTitlename) {
			int idx = jTabbedPane.indexOfComponent(pane);
			if (idx != -1) {
				jTabbedPane.setTitleAt(idx, panelTitlename);
				jTabbedPane.revalidate();
			}
		}

		@Override
		void setSelectPanel(JPanel panel) {
			if (panel == null) {
				return;
			}
			jTabbedPane.setSelectedComponent(panel);
		}

		@Override
		void sortWindow() {
			// 何もしない
		}

	}

	class PaneWin extends Pane {

		private JDesktopPane jDesktopPane = null;

		private HashMap<JInternalFrame, JPanel> jPanelExecSqlMap = null;

		int x = 0, y = 0;
		@Override
		void addJPanel(JPanel panel) {
			JInternalFrame jif = getJInternalFrame(panel);
			jif.setLocation(x, y);
			x += 15;
			y += 25;
			jDesktopPane.add(jif);
		}

		@Override
		JComponent getJComponentPane() {
			return jDesktopPane;
		}

		private JInternalFrame getJInternalFrame(JPanel panel) {
			JInternalFrame jInternalFrame = new JInternalFrame();
			jInternalFrame.add(panel);
			jInternalFrame.setTitle(panel.getName());
			jInternalFrame.setLocation(new java.awt.Point(0, 0));
			jInternalFrame.setSize(panel.getPreferredSize());
			jInternalFrame.setIconifiable(true);
			jInternalFrame.setResizable(true);
			jInternalFrame.setMaximizable(true);
			jInternalFrame.setVisible(true);
			jPanelExecSqlMap.put(jInternalFrame, panel);

			jInternalFrame.addInternalFrameListener(new javax.swing.event.InternalFrameAdapter() {
				@Override
				public void internalFrameActivated(InternalFrameEvent e) {
					JPanel panel = jPanelExecSqlMap.get(e.getInternalFrame());
					if (panel instanceof JPanelExecSql) {
						JPanelExecSql pes = (JPanelExecSql) panel;
						sqlStatementObserver.sqlStatementNotifyObservers(pes.getSqlStatement());
					} else {
						sqlStatementObserver.sqlStatementNotifyObservers(null);
					}
				}
			});
			
			return jInternalFrame;
		}

		@Override
		JPanel getSelectedJPanel() {
			JInternalFrame jif = jDesktopPane.getSelectedFrame();
			if (jif == null) {
				return null;
			}
			return jPanelExecSqlMap.get(jif);
		}

		@Override
		boolean isTabWinModeEqual(TabWinMode tw) {
			if (tw == TabWinMode.WIN) {
				return true;
			}
			return false;
		}

		@Override
		JComponent newJComponentPane() {
			jDesktopPane = new JDesktopPane();
			jPanelExecSqlMap = new HashMap<JInternalFrame, JPanel>();
			x=0;
			y=0;
			return jDesktopPane;
		}

		@Override
		void removePanel(JPanel panel) {
			if (panel == null) {
				return;
			}
			JInternalFrame jif = searchParentFrame(panel);
			if (jif != null) {
				jif.dispose();
				jPanelExecSqlMap.remove(jif);
			}
		}

		/**
		 * JPanelExecSqlに対応するJInternalFrameをjPanelExecSqlMapにから探して返却する
		 * 存在しなかったらnullを返却
		 */
		private JInternalFrame searchParentFrame(JPanel panel) {
			for (Map.Entry<JInternalFrame, JPanel> map : jPanelExecSqlMap
					.entrySet()) {
				if (map.getValue() == panel) {
					return map.getKey();
				}
			}
			return null;

		}

		@Override
		void setPaneTitleName(JPanel panel, String panelTitlename) {
			JInternalFrame jif = searchParentFrame(panel);
			if (jif != null) {
				jif.setTitle(panelTitlename);
			}
		}

		@Override
		void setSelectPanel(JPanel panel) {
			if (panel == null) {
				return;
			}
			JInternalFrame jif = searchParentFrame(panel);
			if (jif != null) {
				// selectが動かないので、最前面で代用する。その為、selectイベントが発生しないので、notifyが動作しない
				// でも、TAB => WIN の切替時のみ使用なので、選択パネルは変化していない。つまりnotifyも必要ない。
				// よって、とりあえずＯＫ。
				// また動かないのは、上位のcomponentと繋がっていないからかもしれない
				jif.toFront();
			}
		}

		@Override
		void sortWindow() {
			// InternalFrameを縮小するサイズは、現在のDesktopPaneの大きさから求める
			Dimension dim = jDesktopPane.getSize();

			// アイコン化されていないInternalFrame数を数える
			int sizeList = 0;
			for (JInternalFrame jif : jDesktopPane.getAllFrames()) {
				if (!jif.isIcon()) {
					sizeList++;
				}
			}

			// 縮小後のウィンドウを (cntWidth * cntHeight)枚にするかを計算
			int cntWidth = (int) Math.ceil(sizeList
					/ Math.ceil(Math.sqrt(sizeList)));
			int cntHeight = (int) Math.ceil(Math.sqrt(sizeList));

			// 1つあたりのサイズを計算する
			int winWidth = (int) dim.getWidth() / cntWidth;
			int winHeight = (int) dim.getHeight() / cntHeight;

			// DesktopPane内の位置を計算し、Listに格納する。(cntWidth *
			// cntHeight)枚を全て計算してしまうので、
			// sizeList < (cntWidth * cntHeight) の時最後の方は使用されない。
			ArrayList<Point> points = new ArrayList<Point>();
			for (int width = 0; width < cntWidth; width++) {
				for (int height = 0; height < cntHeight; height++) {
					Point p = new Point(winWidth * width, winHeight * height);
					points.add(p);
				}
			}

			// アイコン化されていないInternalFrameを、Listに計算済の位置に従って並び替えする。
			Iterator<Point> it = points.iterator();
			for (JInternalFrame jif : jDesktopPane.getAllFrames()) {
				if (!jif.isIcon()) {
					jif.setSize(new Dimension(winWidth, winHeight));
					jif.setLocation(it.next());
				}
			}
			
			x=0;
			y=0;
		}
	}

	private Vector<JPanel> jPanelList = null;

	private Pane pane;

	private SqlStatementObserver sqlStatementObserver;

	private int tabCount = 0;

	public TabWinManager(SqlStatementObserver sqlStatementObserver) {
		this.sqlStatementObserver = sqlStatementObserver;

		jPanelList = new Vector<JPanel>();
		newJComponentPane(TabWinMode.TAB);

		for (int i = 0; i < 4; i++) {
			newPanel(true);
		}

	}

	/**
	 * Jpanelの土台になるペインを新規作成する
	 * @param tabWin
	 * @return
	 */
	private JComponent newJComponentPane(TabWinMode tabWin) {
		if (tabWin == TabWinMode.TAB) {
			pane = new PaneTab();
		} else {
			pane = new PaneWin();
		}
		return pane.newJComponentPane();

	}

	/**
	 * JPanelExecSqlを新規作成する
	 * @param antoCommit
	 */
	public void newPanel(boolean antoCommit) {
		JPanelExecSql pes = new JPanelExecSql(Integer.toString(++tabCount), antoCommit);
		addPanel(pes);
	}

	/**
	 * 任意のpanelを追加する
	 * @param panel
	 */
	public void addPanel(JPanel panel) {
		jPanelList.add(panel);
		pane.addJPanel(panel);
	}

	/**
	 * 全てのJPanelExecSqlにDB切断を指示する
	 */
	public void allDisconnect() {
		for (JPanel panel : jPanelList) {
			if (panel instanceof JPanelExecSql) {
				JPanelExecSql pes = (JPanelExecSql) panel;
				pes.disconnect();
			}
		}
	}

	/**
	 *　全てのJPanelExecSqlにDB切断を指示し、全てのパネルを閉じる。 
	 */
	public void allPaneClosed() {
		allDisconnect();

		for (int i = jPanelList.size() - 1; i > -1; i--) {
			disposePanel(jPanelList.get(i));
		}
	}

	/**
	 * 指定されたJPanelExecSqlの表示を消し、関連するリストからも削除
	 */
	public void disposePanel(JPanel panel) {
		pane.removePanel(panel);
		jPanelList.remove(panel);
	}

	/**
	 * ペインの切替
	 * @param tw
	 * @return
	 */
	public JComponent changeTabWin(TabWinMode tw) {
		if (pane.isTabWinModeEqual(tw)) {
			return pane.getJComponentPane();
		}

		// 現在選択されれいるものを検索
		JPanel selectedPanel = pane.getSelectedJPanel();

		// 切替先作成
		newJComponentPane(tw);

		// 既存パネルを全て追加する
		for (JPanel item : jPanelList) {
			pane.addJPanel(item);
		}

		// 切替前に選択されてたものを選択させる
		pane.getJComponentPane().setVisible(true);
		pane.setSelectPanel(selectedPanel);

		return pane.getJComponentPane();
	}

	/**
	 * JPanelExecSqlのパネル数を返却する
	 * @return
	 */
	public int getExecSqlListCnt() {
		int cnt=0;
		for (JPanel item : jPanelList) {
			if (item instanceof JPanelExecSql) {
				cnt++;
			}
		}
		return cnt;
	}

	public Pane getPane() {
		return pane;
	}

	/**
	 * InternalFrameを整列させる
	 */
	public void sortWindow() {
		pane.sortWindow();
	}

}
