/**
 * 
 */
package jdbcacsess.gui.cell;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import jdbcacsess.gui.common.ColumnWidth;
import jdbcacsess.gui.common.JDialogMessage;
import jdbcacsess.gui.common.JFrameBase;
import jdbcacsess.gui.common.JPopupMenuNoEdit;

/**
 * @author sima
 * 
 */
public class JFrameViewerHex extends JFrameBase implements OutputHandler,
		ClipboardOwner {
	@Override
	public String getAttrKey(){ 
		return null;
	}
	@Override
	public String getAttrValue(){ 
		return null;
	}


	/**
	 * 
	 */
	private static final long serialVersionUID = -6595776140691157849L;

	private static final String TITLE = "１６進ダンプ";

	private JPanel jContentPane = null;

	private JScrollPane jScrollPane = null;

	private JTable jTableHexViewer = null;

	private static JFrameViewerHex jFrameViewer = null;

	public static JFrameViewerHex getInstance() {
		if (jFrameViewer == null) {
			jFrameViewer = new JFrameViewerHex();
		}
		return jFrameViewer;
	}

	/**
	 */
	private JFrameViewerHex() {
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setContentPane(getJContentPane());
		this.setTitle(TITLE);
		this.setSize(new java.awt.Dimension(600, 400));

		JPopupMenuNoEdit jpmne = new JPopupMenuNoEdit(jTableHexViewer);
		jpmne.selectAllAddActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				jTableHexViewer.selectAll();
			}
		});

		jpmne.copyAddActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				copyToClipbord();
			}
		});

	}

	private void copyToClipbord() {
		String lineSeparator = System.getProperty("line.separator");

		int countCol = jTableHexViewer.getColumnCount();
		int countRow = jTableHexViewer.getSelectedRowCount();

		int selRows[] = jTableHexViewer.getSelectedRows();

		StringBuffer selectString = new StringBuffer();
		for (int idxCol = 0; idxCol < countCol; idxCol++) {
			if (idxCol != 0) {
				selectString.append('\t');
			}
			selectString.append(model.getColumnName(idxCol));
		}
		selectString.append(lineSeparator);

		for (int idxRow = 0; idxRow < countRow; idxRow++) {

			StringBuffer line = new StringBuffer();
			for (int idxCol = 0; idxCol < countCol; idxCol++) {
				Object o = jTableHexViewer.getValueAt(selRows[idxRow], idxCol);
				if (o != null) {
					if (idxCol != 0) {
						line.append('\t');
					}
					line.append(o.toString());
				}
			}
			selectString.append(line).append(lineSeparator);
		}
		getToolkit().getSystemClipboard().setContents(
				new StringSelection(selectString.toString()), null);
	}

	@Override
	public void frameClosing() {
		jFrameViewer = null;
		if (is != null) {
			try {
				is.close();
			} catch (IOException e) {
				JDialogMessage.errorDialog(e);
			}
		}
		if (model != null) {
			model.clear();
		}
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabel = new JLabel();
			jLabel.setText("読み込み中…");
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getJScrollPane(), java.awt.BorderLayout.CENTER);
			jContentPane.add(jLabel, java.awt.BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getJTableHexViewer());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jTable1
	 * 
	 * @return javax.swing.JTable
	 */
	private JTable getJTableHexViewer() {
		if (jTableHexViewer == null) {
			jTableHexViewer = new JTable();
			jTableHexViewer.setCellSelectionEnabled(true);
			jTableHexViewer.setColumnSelectionAllowed(false);
			jTableHexViewer.getTableHeader().setReorderingAllowed(false);
			Font font=jTableHexViewer.getFont();
			jTableHexViewer.setFont(new Font("Monospaced",font.getStyle(),font.getSize()));
		}
		return jTableHexViewer;
	}

	private JTableHexViewerModel model;

	private InputStream is;

	private JLabel jLabel = null;

	private long dataSize = -1;

	public void output() {

		// this.is = new BufferedInputStream(new ProgressMonitorInputStream(new
		// JDialog(),"read...",is));
		model = new JTableHexViewerModel(is, dataSize, this);
		jTableHexViewer.setModel(model);

		jTableHexViewer.getColumnModel().getColumn(0).setCellRenderer(
				jTableHexViewer.getTableHeader().getDefaultRenderer());

		ColumnWidth.setHeaderWidthResizeOff(jTableHexViewer, 7);

		int w = jTableHexViewer.getColumn("0F").getPreferredWidth();
		jTableHexViewer.getColumn("0F").setPreferredWidth((int) (w * 1.5));

		setVisible(true);
	}

	/**
	 * @param dataSize
	 */
	public void setDataSize(long dataSize) {
		this.dataSize = dataSize;

		String msg = "";
		if (dataSize == -1) {
			msg = "全体バイト数は不明です";

		} else {
			long kByte = dataSize / 1024;
			msg = dataSize + "バイト "
					+ ((kByte > 0) ? ("(" + dataSize / 1024 + "KB)") : "");
			msg += "   0x"
					+ Long.toHexString(dataSize).toUpperCase()
					+ "バイト "
					+ ((kByte > 0) ? ("(0x"
							+ Long.toHexString(dataSize / 1024).toUpperCase() + "KB)")
							: "");
		}
		jLabel.setText(msg);
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.gui.cell.OutputHandler#setInputStream(java.io.InputStream)
	 */
	public void setInputStream(InputStream is) {
		if (this.is != null) {
			try {
				this.is.close();
			} catch (IOException e) {
				JDialogMessage.errorDialog(e);
			}
		}
		this.is = is;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.gui.cell.OutputHandler#setReader(java.io.Reader)
	 */
	public void setReader(Reader r) {
		throw new ExceptionNoimplements();
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.gui.cell.OutputHandler#setSqlStatement(java.lang.String)
	 */
	public void setDataSource(String sqlStatement) {
		setTitle(TITLE + " [" + sqlStatement + "]");
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.gui.cell.OutputHandler#setOutputFile(java.io.File)
	 */
	public void setOutputFile(File file) {
		throw new ExceptionNoimplements();
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see java.awt.datatransfer.ClipboardOwner#lostOwnership(java.awt.datatransfer.Clipboard,
	 *      java.awt.datatransfer.Transferable)
	 */
	public void lostOwnership(Clipboard clipboard, Transferable contents) {
	}

} // @jve:decl-index=0:visual-constraint="10,10"
