/**
 * 
 */
package jdbcacsess.gui.cell;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;

import javax.imageio.ImageIO;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;

import jdbcacsess.LogOutput;
import jdbcacsess.gui.common.JDialogMessage;
import jdbcacsess.gui.common.JFrameBase;

/**
 * @author sima
 * 
 */
public class JFrameViewerImage extends JFrameBase implements OutputHandler {
	@Override
	public String getAttrKey(){ 
		return null;
	}
	@Override
	public String getAttrValue(){ 
		return null;
	}

	private static final String FRAMESIZE_WIN = "win";
	private static final String FRAMESIZE_IMAGE = "image";

	/**
	 * 
	 */
	private static final long serialVersionUID = -6212931542335612140L;

	private static final String TITLE = "画像ビューワ";

	private JPanel jContentPane = null;

	private static OutputHandler jFrameViewer = null;

	private InputStream is;

	private JLabel jLabelImage = null;

	private JPopupMenu jPopupMenu = null;

	private JRadioButtonMenuItem jRadioButtonMenuItemWinSize = null;

	private JRadioButtonMenuItem jRadioButtonMenuItemImageSize = null;

	private BufferedImage image;

	public static OutputHandler getInstance() {
		if (jFrameViewer == null) {
			jFrameViewer = new JFrameViewerImage();
		}
		return jFrameViewer;
	}

	/**
	 * This is the default constructor
	 */
	private JFrameViewerImage() {
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setContentPane(getJContentPane());
		this.setTitle(TITLE);
		this.setSize(new java.awt.Dimension(600, 400));

		getJPopupMenu();
		ButtonGroup bg = new ButtonGroup();
		bg.add(jRadioButtonMenuItemImageSize);
		bg.add(jRadioButtonMenuItemWinSize);

		jLabelImage.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent event) {
				popupMenu(event);
			}

			@Override
			public void mouseReleased(MouseEvent event) {
				popupMenu(event);
			}
		});

		frameSize = omponentProperty.getToString(this.getClass(), "frameSize");
		if (frameSize.equals("")) {
			frameSize = FRAMESIZE_IMAGE;
		}
		setSelectedFrameSize(frameSize);
	}

	private void popupMenu(MouseEvent event) {
		if (event.isPopupTrigger()) {
			getJPopupMenu().show(event.getComponent(), event.getX(),
					event.getY());
		}
	}

	private void setSelectedFrameSize(String size) {
		frameSize = size;
		if (size.equals(FRAMESIZE_IMAGE)) {
			jRadioButtonMenuItemImageSize.setSelected(true);
		}
		if (size.equals(FRAMESIZE_WIN)) {
			jRadioButtonMenuItemWinSize.setSelected(true);
		}
		resize();
	}

	private void resize() {
		ImageIcon imageIcon = (ImageIcon) jLabelImage.getIcon();
		if (imageIcon != null) {

			if (frameSize.equals(FRAMESIZE_IMAGE)) {
				jRadioButtonMenuItemImageSize.setSelected(true);
				Insets insets = getInsets();
				setSize(new Dimension(image.getWidth() + insets.left
						+ insets.right, image.getHeight() + insets.top
						+ insets.bottom));
				imageIcon.setImage(image);
			}

			if (frameSize.equals(FRAMESIZE_WIN)) {
				jRadioButtonMenuItemWinSize.setSelected(true);

				double w = (double) image.getWidth()
						/ (double) jLabelImage.getWidth();
				double h = (double) image.getHeight()
						/ (double) jLabelImage.getHeight();

				LogOutput.fine("orgsize:" + image.getWidth() + ","
						+ image.getHeight());
				LogOutput.fine("倍率:" + w + "," + h);
				if (w > 1.0 || h > 1.0) {
					int width = -1;
					int height = -1;
					if (w > h) {
						width = (int) (image.getWidth() / w);
					} else {
						height = (int) (image.getHeight() / h);
					}
					LogOutput.fine("resize:" + width + "," + height);
					imageIcon.setImage(image.getScaledInstance(width, height,
							Image.SCALE_FAST));
				}
			}
		}
		setVisible(true);
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.gui.Viewer#setInputStream(java.io.InputStream, long,
	 *      java.lang.String)
	 */
	public void output() throws IOException {
		image = ImageIO.read(is);
		if (image == null) {
			throw new IOException("javax.imageio.ImageIOで扱えない画像データ形式です");
		}
		jLabelImage.setIcon(new ImageIcon(image));
		resize();
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.gui.cell.OutputHandler#setInputStream(java.io.InputStream)
	 */
	public void setInputStream(InputStream is) {
		if (this.is != null) {
			try {
				this.is.close();
			} catch (IOException e) {
				JDialogMessage.errorDialog(e);
			}
		}
		this.is = is;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.gui.cell.OutputHandler#setReader(java.io.Reader)
	 */
	public void setReader(Reader r) {
		throw new ExceptionNoimplements();
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.gui.cell.OutputHandler#setSqlStatement(java.lang.String)
	 */
	public void setDataSource(String sqlStatement) {
		setTitle(TITLE + " [" + sqlStatement + "]");
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jLabelImage = new JLabel();
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(jLabelImage, java.awt.BorderLayout.CENTER);
		}
		return jContentPane;
	}

	/**
	 * This method initializes jPopupMenu
	 * 
	 * @return javax.swing.JPopupMenu
	 */
	private JPopupMenu getJPopupMenu() {
		if (jPopupMenu == null) {
			jPopupMenu = new JPopupMenu();
			jPopupMenu.add(getJRadioButtonMenuItemWinSize());
			jPopupMenu.add(getJRadioButtonMenuItemImageSize());
		}
		return jPopupMenu;
	}

	String frameSize;

	/**
	 * This method initializes jRadioButtonMenuItem
	 * 
	 * @return javax.swing.JRadioButtonMenuItem
	 */
	private JRadioButtonMenuItem getJRadioButtonMenuItemWinSize() {
		if (jRadioButtonMenuItemWinSize == null) {
			jRadioButtonMenuItemWinSize = new JRadioButtonMenuItem();
			jRadioButtonMenuItemWinSize.setText("Windoサイズに合わせる");
			jRadioButtonMenuItemWinSize
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							setSelectedFrameSize(FRAMESIZE_WIN);
						}
					});
		}
		return jRadioButtonMenuItemWinSize;
	}

	/**
	 * This method initializes jRadioButtonMenuItem1
	 * 
	 * @return javax.swing.JRadioButtonMenuItem
	 */
	private JRadioButtonMenuItem getJRadioButtonMenuItemImageSize() {
		if (jRadioButtonMenuItemImageSize == null) {
			jRadioButtonMenuItemImageSize = new JRadioButtonMenuItem();
			jRadioButtonMenuItemImageSize.setText("画像サイズに合わせる");
			jRadioButtonMenuItemImageSize
					.addActionListener(new java.awt.event.ActionListener() {
						public void actionPerformed(java.awt.event.ActionEvent e) {
							setSelectedFrameSize(FRAMESIZE_IMAGE);
						}
					});
		}
		return jRadioButtonMenuItemImageSize;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.gui.cell.OutputHandler#setDataSize(long)
	 */
	public void setDataSize(long size) {
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.gui.cell.OutputHandler#setOutputFile(java.io.File)
	 */
	public void setOutputFile(File file) {
		throw new ExceptionNoimplements();
	}

	@Override
	public void frameClosing() {
		omponentProperty.put(this.getClass(), "frameSize", frameSize);

		jFrameViewer = null;
		if (is != null) {
			try {
				is.close();
			} catch (IOException e) {
				JDialogMessage.errorDialog(e);
			}
		}
	}

}
