package jdbcacsess.gui.cell;

import java.awt.BorderLayout;
import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

public class JPanelSqlData extends JPanel {
	
	static Map<String,String> arrayPrimitiveClassNameMap ;
	static {
		arrayPrimitiveClassNameMap = new HashMap<String, String>();
		arrayPrimitiveClassNameMap.put("[Z", "boolean[]");
		arrayPrimitiveClassNameMap.put("[B", "byte[]");
		arrayPrimitiveClassNameMap.put("[C", "char[]");
		arrayPrimitiveClassNameMap.put("[D", "double[]");
		arrayPrimitiveClassNameMap.put("[F", "float[]");
		arrayPrimitiveClassNameMap.put("[I", "int[]");
		arrayPrimitiveClassNameMap.put("[J", "long[]");
		arrayPrimitiveClassNameMap.put("[S", "short[]");
	}

	class SqlData {
		String fieldName;
		Object object;

		public SqlData(String fieldName, Object object) {
			this.fieldName = fieldName;
			this.object = object;
		}

		@Override
		public String toString() {
			StringBuilder sb = new StringBuilder();

			sb.append(fieldName);
			sb.append(" ");

			String className = object.getClass().getName();
			String value = arrayPrimitiveClassNameMap.get(className);
			sb.append((value == null ? className : value));
			
			try {
				if (object.getClass().getMethod("toString", (Class[])null).getDeclaringClass() != Object.class) {
					sb.append("  : ");
					sb.append(object.toString());
				}
			} catch (SecurityException e) {
			} catch (NoSuchMethodException e) {
			}

			return sb.toString();
		}

		public Object getObject() {
			return object;
		}
	}

	class SqlDataPrimitive extends SqlData {
		public SqlDataPrimitive(String fieldName, Object object) {
			super(fieldName, object);
		}

		@Override
		public String toString() {
			StringBuilder sb = new StringBuilder();

			sb.append(fieldName);
			sb.append("  : ");
			sb.append(object);

			return sb.toString();
		}
	}

	class SqlDataNull extends SqlData {
		Class<?> cls;

		public SqlDataNull(String fieldName, Class<?> cls) {
			super(fieldName, null);
			this.cls = cls;
		}

		@Override
		public String toString() {
			StringBuilder sb = new StringBuilder();

			sb.append(fieldName);
			sb.append(" " + cls.getName() );
			sb.append("  : #NULL#");

			return sb.toString();
		}
	}

	private static final long serialVersionUID = 1L;

	private JScrollPane jScrollPane = null;

	private JTree jTree = null;

	private DefaultMutableTreeNode rootTreeNode;

	private DefaultTreeModel treeModel;

	/**
	 * This is the default constructor
	 */
	public JPanelSqlData() {
		super();
		initialize();

		jTree.setModel(null);
		jTree.setToggleClickCount(1);

		jTree.addTreeExpansionListener(new javax.swing.event.TreeExpansionListener() {
			public void treeExpanded(javax.swing.event.TreeExpansionEvent e) {
				DefaultMutableTreeNode node = (DefaultMutableTreeNode) e.getPath().getLastPathComponent();

				DefaultMutableTreeNode dummyChild = (DefaultMutableTreeNode) node.getFirstChild();
				if (dummyChild.getUserObject() != null) {
					// 既に取得しているので、何もせず終了
					return;
				}
				addObject(node);
				node.remove(dummyChild);
				treeModel.reload(node);
			}

			public void treeCollapsed(javax.swing.event.TreeExpansionEvent e) {
			}
		});

	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(300, 200);
		this.setLayout(new BorderLayout());
		this.add(getJScrollPane(), BorderLayout.CENTER);
	}

	public void setTopObject(String fieldName, Class<?> cls, Object obj) {
		rootTreeNode = addChileNode(fieldName, cls, obj);
		
		// expand用に作成されているダミーノードを削除する
		if (rootTreeNode.getChildCount()!=0){
			DefaultMutableTreeNode dummyChild = (DefaultMutableTreeNode) rootTreeNode.getFirstChild();
			if (dummyChild.getUserObject() == null) {
				rootTreeNode.remove(dummyChild);
			}
		}
		
		addObject(rootTreeNode);

		treeModel = new DefaultTreeModel(rootTreeNode);
		jTree.setModel(treeModel);
	}

	private void addObject(DefaultMutableTreeNode node) {
		Object obj = ((SqlData) node.getUserObject()).getObject();
		if (obj == null) {
			return;
		}
		Class<?> cls = obj.getClass();
		while (cls != Object.class) {
			for (Field field : cls.getDeclaredFields()) {
				field.setAccessible(true);

				try {
					node.add(addChileNode(field.getName(), field.getType(), field.get(obj)));
				} catch (IllegalArgumentException e) {
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					e.printStackTrace();
				}
			}
			cls = cls.getSuperclass();
		}
	}

	private DefaultMutableTreeNode addChileNode(String fieldName, Class<?> cls, Object obj) {
		if (obj == null) {
			return new DefaultMutableTreeNode(new SqlDataNull(fieldName, cls));
		}
		if (cls.isPrimitive()) {
			return new DefaultMutableTreeNode(new SqlDataPrimitive(fieldName, obj));
		}

		DefaultMutableTreeNode node = new DefaultMutableTreeNode(new SqlData(fieldName, obj));
		
		if (!cls.isArray()) {
			// expand用にダミーを追加する
			node.add(new DefaultMutableTreeNode(null));
			return node;
		} 

		// Arrayの時
		boolean primitive = arrayPrimitiveClassNameMap.containsKey(obj.getClass().getName());
		
		for (int j = 0; j < java.lang.reflect.Array.getLength(obj); j++) {
			Object arrayObj = java.lang.reflect.Array.get(obj, j);

			if (arrayObj == null) {

				DefaultMutableTreeNode arrayNode = new DefaultMutableTreeNode(new SqlDataNull("(" + j + ")", cls.getComponentType()));
				node.add(arrayNode);
				
			} else {

				String arrayStringName = "(" + j + ")";
				DefaultMutableTreeNode arrayNode = null;

				if (primitive) {
					arrayNode = new DefaultMutableTreeNode(new SqlDataPrimitive(arrayStringName, arrayObj));
					node.add(arrayNode);
				} else {
					node.add(addChileNode(arrayStringName, cls.getComponentType(), arrayObj));
				}
			}
			
			// 打ち切る
			if (j > 999) {
				break;
			}
		}
		return node;
	}

	/**
	 * This method initializes jScrollPane
	 * 
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getJScrollPane() {
		if (jScrollPane == null) {
			jScrollPane = new JScrollPane();
			jScrollPane.setViewportView(getJTree());
		}
		return jScrollPane;
	}

	/**
	 * This method initializes jTree
	 * 
	 * @return javax.swing.JTree
	 */
	private JTree getJTree() {
		if (jTree == null) {
			jTree = new JTree();
		}
		return jTree;
	}

}
