/**
 * 
 */
package jdbcacsess.gui.common;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;

import jdbcacsess.SettingFile;

import org.w3c.dom.Element;

/**
 * @author sima ウィンドウ情報をxmlプロパティに保存と復元を行う。
 */
public class ComponentProperty {

	private String attrKey = null;

	private String attrValue = null;



	private static Element element(){
		return SettingFile.getInstance().getRootElement("component2");
	}
	
	public void appendCondition(String attrKey, String attrValue) {
		this.attrKey = attrKey;
		this.attrValue = attrValue;
	}

	@SuppressWarnings("unchecked")
	public void remove(Class c) {
		Element guiElement = findAttr(c);
		if (guiElement != null) {
			guiElement.getParentNode().removeChild(guiElement);
		}
	}

	/**
	 * Stringの値を持つプロパティを出力する。 第一階層のキーとしてObjectのクラス名を使用する。
	 * 第二階層のキーとして指定されたkeyを使用する。 <br>
	 * xmlファイル上は、第一階層のキーはタグ名、第二階層のキーはアトリビュート名になる。
	 * 
	 * @param c
	 *            第一階層のキーとして使用されるオブジェクトのクラス
	 * @param key
	 *            第二階層のキー
	 * @param value
	 *            格納する文字列
	 */
	@SuppressWarnings("unchecked")
	public void put(Class c, String key, String value) {
		Element guiElement = findAttr(c);
		if (guiElement == null) {
			guiElement = SettingFile.getInstance().createElement("window");
			guiElement.setAttribute("name", c.getName());
			if (attrKey != null) {
				guiElement.setAttribute(attrKey, attrValue);
			}
			element().appendChild(guiElement);
		}
		guiElement.setAttribute(key, value);
	}

	/**
	 * Integerの値をStringに変換してプロパティを出力する。
	 * 
	 * @param c
	 *            第一階層のキーとして使用されるオブジェクトのクラス
	 * @param key
	 *            第二階層のキー
	 * @param value
	 *            格納する数値
	 */
	@SuppressWarnings("unchecked")
	public void put(Class c, String key, Integer value) {
		put(c, key, value.toString());
	}

	/**
	 * Componentからウィンドウのサイズと位置取得しプロパティとして出力する。
	 * xmlのタグ名は、Componentからクラス名から自動的に決定される。
	 * 
	 * @param component
	 *            対象となるComponent
	 */
	@SuppressWarnings("unchecked")
	public void storeWindowPosition(Component component) {
		Class c = component.getClass();
		put(c, "height", component.getSize().height);
		put(c, "width", component.getSize().width);
		put(c, "x", component.getLocation().x);
		put(c, "y", component.getLocation().y);
	}

	/**
	 * xmlプロパティから値を取得する
	 * 
	 * @param class1
	 *            第一階層のキーとして使用されるオブジェクトのクラス
	 * @param key
	 *            第二階層のキー
	 * @return 取得した文字列
	 */
	@SuppressWarnings("unchecked")
	public String getToString(Class class1, String key) {
		Element guiElement = findAttr(class1);

		if (guiElement == null) {
			return "";
		}
		return guiElement.getAttribute(key);
	}

	/**
	 * xmlプロパティから値を数値に変換して取得する。 キーが存在しなかった場合や数値以外の文字を含んでいない場合はnullを返却する
	 * 
	 * @param c
	 *            第一階層のキーとして使用されるオブジェクトのクラス
	 * @param key
	 *            第二階層のキー
	 * @return 取得した数値
	 */
	@SuppressWarnings("unchecked")
	public Integer getToInteger(Class c, String key) {
		Integer i = null;
		try {
			i = new Integer(getToString(c, key));
		} catch (Exception e) {
		}
		return i;
	}

	/**
	 * ウィンドウ位置とサイズをxmlプロパティから取得し、Componentに設定する。
	 * xmlのタグ名は、Componentからクラス名から自動的に決定される。
	 * 
	 * @param component
	 *            設定したいComponentオブジェクト
	 * @param width
	 *            xmlプロパティに値が存在しなかった時の幅
	 * @param height
	 *            xmlプロパティに値が存在しなかった時の高さ
	 */
	public void restoreWindowPosition(Component component, Integer width,
			Integer height) {
		Class<? extends Component> c = component.getClass();
		Integer widthInt = getToInteger(c, "width");
		Integer heightInt = getToInteger(c, "height");
		Integer xInt = getToInteger(c, "x");
		Integer yInt = getToInteger(c, "y");

		setWin(component, width, height, widthInt, heightInt, xInt, yInt);
	}

	/**
	 * @param c
	 * @return
	 */
	private Element findAttr(Class<? extends Component> c) {
		Element guiElement = null;
		if (attrKey == null) {
			guiElement = SettingFile.getElementSearchingAttribute(element(),
					"window", "name", c.getName());
		} else {
			guiElement = SettingFile.getElementSearchingAttribute2(element(),
					"window", "name", c.getName(), attrKey, attrValue);
		}
		return guiElement;
	}

	/**
	 * @param component
	 *            設定したいComponentオブジェクト
	 * @param width
	 *            xmlプロパティに値が存在しなかった時の幅
	 * @param height
	 *            xmlプロパティに値が存在しなかった時の高さ
	 * @param widthInt
	 *            xmlプロパティの幅
	 * @param heightInt
	 *            xmlプロパティの高さ
	 * @param xInt
	 *            xmlプロパティの横位置
	 * @param yInt
	 *            xmlプロパティの縦位置
	 */
	private void setWin(Component component, Integer width, Integer height,
			Integer widthInt, Integer heightInt, Integer xInt, Integer yInt) {

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

		if (heightInt == null || widthInt == null || xInt == null
				|| yInt == null) {
			// 設計上のサイズが画面に収まる様に変更する。そんな大きな画面を設計する事はないと思うけど。。。
			if (height > screenSize.height) {
				height = screenSize.height;
			}
			if (width > screenSize.width) {
				width = screenSize.width;
			}
			// 画面サイズの設定
			component.setSize(new Dimension(width, height));
			component.setLocation(0, 0);
		} else {
			// 画面サイズの設定
			component.setSize(new Dimension(widthInt, heightInt));
			// 画面位置の設定
			component.setLocation(xInt, yInt);
		}
	}

}
