package jdbcacsess.gui.common;

import java.sql.SQLException;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import jdbcacsess.LogOutput;
import jdbcacsess.gui.JFrameMain;

public class JDialogMessage {
	
	public static void infoDialog(String message, String title) {
		showDialog(message, title, JOptionPane.INFORMATION_MESSAGE);
	}

	/**
	 * DB未接続エラーダイアログ表示
	 */
	public static void notConnectErrorDialog() {
		showDialog("データベース未接続", 
				"DB接続して下さい", 
				JOptionPane.ERROR_MESSAGE);
	}

	/**
	 * エラーダイアログ表示
	 */
	public static void errorDialog(Throwable t) {
		t.printStackTrace();
		if (t instanceof SQLException){
			SQLException e = (SQLException)t;
			String outMessage = e.getErrorCode() + "[" + e.getSQLState() + "]" + e.toString();

			LogOutput.severe("SQLエラーが発生:" + outMessage );
			
			showDialog( outMessage, 
					"SQLエラー発生",
					JOptionPane.ERROR_MESSAGE);
		}else{
			LogOutput.severe("障害が発生:" + t.toString());

			showDialog( t.toString(), 
					"エラー発生",
					JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * エラーダイアログ表示
	 */
	public static void errorDialog(Throwable t, String message) {
		t.printStackTrace();
		if (t instanceof SQLException){
			SQLException e = (SQLException)t;
			String outMessage = e.getErrorCode() + "[" + e.getSQLState() + "]" + e.toString();

			LogOutput.severe("SQLエラーが発生:"+ message + ":" + outMessage );
			
			showDialog(message + "\n" + outMessage, 
					"SQLエラー発生",
					JOptionPane.ERROR_MESSAGE);
		}else{
			LogOutput.severe("エラーが発生:" + message +":" + t.toString());
			
			showDialog( message +"\n" + t.toString(), 
					"エラー発生",
					JOptionPane.ERROR_MESSAGE);
		}
	}

	private static void showDialog(final String message, 
			final String title,
			final int messageType) {
		
		if ( JFrameMain.debugMode ){
			JFrameMain.getInstanceJPanelPrepareStatementLog().appendLog("###"+title+"###"+message);
		}
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
		        JOptionPane.showMessageDialog(null, message, title, messageType);
			}
		});

	}
}