package jdbcacsess.gui.common;

import java.awt.Component;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.AbstractAction;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import javax.swing.tree.TreePath;

public class PopupKeybordAndMouse {

	PopupComponent pc;

	JPopupMenu jpm;

	public PopupKeybordAndMouse(JTree jtree, JPopupMenu jpopupMenu) {
		pc = new PopupComponentJTree(jtree);
		jpm = jpopupMenu;
		init();
	}

	public PopupKeybordAndMouse(JTable jtable, JPopupMenu jpopupMenu) {
		pc = new PopupComponentJTable(jtable);
		jpm = jpopupMenu;
		init();
	}

	private interface PopupComponent {
		public JComponent getJComponent();

		public void cellSelect(MouseEvent e);

		public Rectangle getPopupLocation();

	}

	private static class PopupComponentJTree implements PopupComponent {

		JTree tree;

		public PopupComponentJTree(JTree component) {
			this.tree = component;
		}

		public JTree getJComponent() {
			return tree;
		}

		public void cellSelect(MouseEvent e) {
			TreePath mouseTreePath = tree
					.getPathForLocation(e.getX(), e.getY());

			if (mouseTreePath == null) {
				return;
			}

			TreePath[] selectTreePaths = tree.getSelectionPaths();
			if (selectTreePaths == null) {
				tree.setSelectionPath(mouseTreePath);
				return;
			}
			boolean hitFlg = false;
			for (TreePath path : selectTreePaths) {
				if (path == mouseTreePath) {
					hitFlg = true;
					break;
				}
			}
			if (!hitFlg) {
				tree.setSelectionPath(mouseTreePath);
			}
		}

		public Rectangle getPopupLocation() {
			return tree.getPathBounds(tree.getSelectionPath());
		}
	}

	private static class PopupComponentJTable implements PopupComponent {

		JTable table;

		public PopupComponentJTable(JTable table) {
			this.table = table;
		}

		public JComponent getJComponent() {
			return table;
		}

		public void cellSelect(MouseEvent e) {
			int row = table.rowAtPoint(e.getPoint());
			int col = table.columnAtPoint(e.getPoint());

			if (row < 0 || col < 0) {
				return;
			}
			if (!table.isCellSelected(row, col)) {
				table.setRowSelectionInterval(row, row);
				table.setColumnSelectionInterval(col, col);
			}
		}

		public Rectangle getPopupLocation() {
			return table.getCellRect(table.getSelectedRow(), table
					.getSelectedColumn(), false);
		}

	}

	private void init() {
		JComponent component = pc.getJComponent();
		component.addMouseListener(new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				if (e.isPopupTrigger()) {
					pc.cellSelect(e);
					jpm.show(e.getComponent(), e.getX(), e.getY());
				}
			}

			@Override
			public void mouseReleased(MouseEvent e) {
				if (e.isPopupTrigger()) {
					pc.cellSelect(e);
					jpm.show(e.getComponent(), e.getX(), e.getY());
				}
			}

		});

		// ポップアップキーの設定
		component.addAncestorListener(new AncestorListener() {
			// rootpaneが必要なので、上位コンポーネントに追加された時に行う
			public void ancestorAdded(AncestorEvent event) {
				JComponent component = pc.getJComponent();
				// popupのキーストロークは、rootpaneのinputmapにしか無いので、
				// jtableのinputmapにコピーする
				InputMap im = component.getRootPane().getInputMap(
						JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
				InputMap myim = component
						.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

				for (KeyStroke keyStroke : im.allKeys()) {
					myim.put(keyStroke, im.get(keyStroke));
				}

				// actionの登録
				component.getActionMap().put("postPopup", new AbstractAction() {
					private static final long serialVersionUID = 8315610383335629222L;

					public void actionPerformed(ActionEvent e) {
						if (e.getSource() == null) {
							return;
						}
						Rectangle r = pc.getPopupLocation();
						// popup表示する
						jpm.show((Component) e.getSource(), r.x + r.width, r.y
								+ r.height);
					}
				});
			}

			public void ancestorMoved(AncestorEvent event) {
			}

			public void ancestorRemoved(AncestorEvent event) {
			}
		});

	}

}
