/**
 * 
 */
package jdbcacsess.sql;

import java.sql.DatabaseMetaData;
import java.util.Comparator;


/**
 * @author sima
 * 
 */
public class ColumnInfo {

	protected int columnIndex = 0;

	protected String columnName = "";

	protected ConstSqlTypes sqlType;
	
	private int sqlTypeInt;

	private ConstSqlTypes sqlMappingSqlType;

	protected String columnTypeName = "";

	protected boolean nullable = false;

	private boolean primaryKey = false;

	private String remarks = "";

	/**
	 * @param index
	 *            １から始まるカラムインデックス番号 (ColumnInfoで保持します)
	 * @param name
	 *            カラム名 (ColumnInfoで保持します)
	 * @param sqlTypes
	 *            カラムのSqlType (ColumnInfoで保持します)
	 * @param typeName
	 *            データベース独自のデータ型名 (ColumnInfoで保持します)
	 * @param nullable
	 *            NULLを許すか (ColumnInfoで保持します)
	 */
	public ColumnInfo(int index, String name, int sqlTypes, String typeName,
			int nullable) {
		columnIndex = index;
		columnName = name;
		sqlType = ConstSqlTypes.searchTypes(sqlTypes);
		sqlTypeInt = sqlTypes;
		sqlMappingSqlType = ConstSqlTypes.searchTypes(sqlTypes);
		columnTypeName = typeName;
		this.nullable = (nullable == DatabaseMetaData.columnNullable ? true
				: false);
	}

	public String getColumnComment() {
		return " #" + columnIndex + " " + (primaryKey ? "<STRONG>" : "")
				+ columnName + (primaryKey ? "</STRONG>" : "") + " : "
				+ columnTypeName;
	}

	public String getColumnName() {
		return columnName;
	}

	/**
	 * JDBCから受取った値の ConstSqlTypes
	 * ベンダー固有の場合は、int getSqlType()で取得して下さい
	 * @return sqlType を戻します。
	 */
	public ConstSqlTypes getSqlType() {
		return sqlType;
	}
	
	/**
	 * jdbcから受取ったsqlTypesのそのままの値
	 * enumを導入したが、ベンダー固有の値に対応できないので、こうれが必要になった。
	 * @return
	 */
	public int getSqlTypeInt() {
		return sqlTypeInt;
	}

	/**
	 * sqlmapping後の値を戻します。
	 * @return
	 */
	public ConstSqlTypes getSqlMappingSqlType() {
		return sqlMappingSqlType;
	}
	/**
	 * sqlmapping後の値を設定します
	 * @return
	 */
	public void setSqlMappingSqlType(ConstSqlTypes targetSqlType) {
		this.sqlMappingSqlType = targetSqlType;
	}

	/**
	 * @return columnTypeName を戻します。
	 */
	public String getColumnTypeName() {
		return columnTypeName;
	}

	/**
	 * @return nullable を戻します。
	 */
	public boolean isNullable() {
		return nullable;
	}

	/**
	 * @return primaryKey を戻します。
	 */
	public boolean isPrimaryKey() {
		return primaryKey;
	}

	/**
	 * @param primaryKey
	 *            設定する primaryKey。
	 */
	public void setPrimaryKey(boolean primKey) {
		this.primaryKey = primKey;
	}

	@Override
	public String toString() {
		return columnName;
	}

	/**
	 * @return columnIndex を戻します。
	 */
	public int getColumnIndex() {
		return columnIndex;
	}

	static public Comparator<ColumnInfo> getComparatorByPosition(){
		Comparator<ColumnInfo> rtn = new Comparator<ColumnInfo>(){

			public int compare(ColumnInfo o1, ColumnInfo o2) {
				return o1.getColumnIndex() - o2.getColumnIndex();
			}

		};
		return rtn;
	}
	static public Comparator<ColumnInfo> getComparatorByColumnName(){
		Comparator<ColumnInfo> rtn = new Comparator<ColumnInfo>(){

			public int compare(ColumnInfo o1, ColumnInfo o2) {
				return o1.getColumnName().compareTo(o2.getColumnName());
			}

		};
		return rtn;
	}

	public String getRemarks() {
		return remarks;
	}

	public void setRemarks(String remarks) {
		this.remarks = (remarks == null ? "" : remarks);
	}
}
