package jdbcacsess.sql;

import java.sql.DatabaseMetaData;
import java.sql.ParameterMetaData;



/**
 * １項目分のカラム名情報を保持する
 */

public class ColumnInfoParameter extends ColumnInfoResult {



	private int parameterMode ;

	/**
	 * @param columnIndex
	 *            １から始まるカラムインデックス番号 (ColumnInfoで保持します)
	 * @param columnName
	 *            カラム名 (ColumnInfoで保持します)
	 * @param sqlType
	 *            カラムのSqlType (ColumnInfoで保持します)
	 * @param columnTypeName
	 *            データベース独自のデータ型名 (ColumnInfoで保持します)
	 * @param nullable
	 *            NULLを許すか (ColumnInfoで保持します)
	 * 
	 * @param columnClass 
	 *            カラムデータのクラス型 (ColumnInfoResultで保持します)
	 * @param precision
	 *            精度　(ColumnInfoResultで保持します)
	 * @param scale
	 *            小数点以下の桁数　(ColumnInfoResultで保持します)
	 *            
	 * @param parameterMode
	 *            指定されたパラメータのモード。ParameterMetaData.parameterModeIn、ParameterMetaData.parameterModeOut、ParameterMetaData.parameterModeInOut、または ParameterMetaData.parameterModeUnknown のうちの 1 つ
	 *            
	 */
	public ColumnInfoParameter(int columnIndex, 
			String columnName, 
			int columnType,
			String columnTypeName, 
			int nullable, 
			String columnClassName,
			int precision, 
			int scale,
			int parameterMode) {

		super(columnIndex, columnName, columnType, columnTypeName, nullable, columnClassName, precision, scale);

		this.parameterMode = parameterMode;
	}

	public String getParameterMode() {
		if (parameterMode == ParameterMetaData.parameterModeIn){
			return "I";
		}else if (parameterMode == ParameterMetaData.parameterModeOut){
			return "O";
		}else if (parameterMode == ParameterMetaData.parameterModeInOut){
			return "I/O";
		}else if (parameterMode == ParameterMetaData.parameterModeUnknown){
			return "?";
		}else if (parameterMode == DatabaseMetaData.procedureColumnResult){
			return "RS";
		}else if (parameterMode == DatabaseMetaData.procedureColumnReturn){
			return "RV";
		}
		return "-";
	}

	public boolean isModeInput() {
		if (parameterMode == ParameterMetaData.parameterModeIn){
			return true;
		}else if (parameterMode == ParameterMetaData.parameterModeInOut){
			return true;
		}
		return false;
	}
	
	public boolean isModeOutput() {
		if (parameterMode == ParameterMetaData.parameterModeOut){
			return true;
		}else if (parameterMode == ParameterMetaData.parameterModeInOut){
			return true;
		}else if (parameterMode == DatabaseMetaData.procedureColumnResult){
			return true;
		}else if (parameterMode == DatabaseMetaData.procedureColumnReturn){
			return true;
		}
		return false;
	}
	
	public boolean isModeReturn() {
		if (parameterMode == DatabaseMetaData.procedureColumnResult){
			return true;
		}else if (parameterMode == DatabaseMetaData.procedureColumnReturn){
			return true;
		}
		return false;
	}
	
	@Override
	public String toString(){
		return getParameterMode() + ":" + getColumnName() + " " + getColumnTypeName();
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.sql.ColumnInfo#getColumnComment()
	 */
	@Override
	public String getColumnComment() {
		if (columnTypeName == null) {
			// どういう時か忘れた
			return "";
		}
		return super.getColumnComment() + " " + getRemarks();
		// columnSizeが精度でなくバイト数なので、型名だけにする
	}
	
}