/**
 * 
 */
package jdbcacsess.sql;

/**
 * テーブル定義のカラム属性を保持する。
 * 
 * @author sima
 * 
 */
public class ColumnInfoTable extends ColumnInfo {

	private int columnSize = 0;

	private int decimalDigits = 0;

	private SchemaTableName schemaTableName = new SchemaTableName();

	/**
	 * @param columnIndex
	 *            １から始まるカラムインデックス番号 (ColumnInfoで保持します)
	 * @param columnName
	 *            カラム名 (ColumnInfoで保持します)
	 * @param sqlType
	 *            カラムのSqlType (ColumnInfoで保持します)
	 * @param columnTypeName
	 *            データベース独自のデータ型名 (ColumnInfoで保持します)
	 * @param nullable
	 *            NULLを許すか (ColumnInfoで保持します)
	 * @param schemaTableName
	 *            スキーマテーブル名
	 * @param columnSize
	 *            カラムバイト数(?)
	 * @param decimalDigits
	 *            小数点以下桁数
	 */
	public ColumnInfoTable(int columnIndex, String columnName, int columnType,
			String columnTypeName, int nullAble,

			SchemaTableName schemaTableName, int columnSize, int decimalDigits) {
		super(columnIndex, columnName, columnType, columnTypeName, nullAble);

		this.schemaTableName = schemaTableName;

		this.columnSize = columnSize;
		this.decimalDigits = decimalDigits;

	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.sql.ColumnInfo#getColumnComment()
	 */
	@Override
	public String getColumnComment() {
		if (columnTypeName == null) {
			// どういう時か忘れた
			return "";
		}
		return super.getColumnComment() + " " + getRemarks();
		// columnSizeが精度でなくバイト数なので、型名だけにする
	}

	/**
	 * @return columnSize を戻します。
	 */
	public int getColumnSize() {
		return columnSize;
	}

	/**
	 * @return decimalDigits を戻します。
	 */
	public int getDecimalDigits() {
		return decimalDigits;
	}

	/**
	 * @return schemaTableName を戻します。
	 */
	public SchemaTableName getSchemaTableName() {
		return schemaTableName;
	}

}
