package jdbcacsess.sql;

import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import javax.swing.Icon;

import jdbcacsess.gui.icons.IconImage;

public class DBObjectCatalogs extends DBObjects {

	static final String[] header = { "TABLE_CAT" };

	private ArrayList<DBObject> catalogs = new ArrayList<DBObject>();

	public DBObjectCatalogs(DatabaseMetaData dmd) throws SQLException {
		ResultSet rs = dmd.getCatalogs();
		while (rs.next()) {
			catalogs.add(new DBObjectCatalog(rs));
		}
		rs.close();
	}

	@Override
	public ArrayList<DBObject> getDBObjects() {
		return catalogs;
	}

	public static class DBObjectCatalog extends DBObject {

		private String catalogName;

		/**
		 * 
		 * @param rs
		 * @throws SQLException
		 */
		public DBObjectCatalog(ResultSet rs) throws SQLException {
			int colCnt = rs.getMetaData().getColumnCount();
			if (colCnt > 0)
				catalogName = rs.getString(1);
		}

		@Override
		public String[] getDetail() {
			String[] rtn = new String[1];
			rtn[0] = catalogName;
			return rtn;
		}

		@Override
		public String toString() {
			if (catalogName == null) {
				return "(無し)";
			}
			return catalogName;
		}

		@Override
		public String[] getHeader() {
			return header;
		}

		/**
		 * 
		 * @return
		 */
		public String getCatalogName() {
			return catalogName;
		}

		@Override
		public DBObjects getChildren( DatabaseMetaData dmd ) throws SQLException {
			DBObjects rtn = new DBObjectSchemas(this,dmd);
			
			if ( rtn.getDBObjects().size() == 0 ){
				rtn = new DBObjectTableTypes(dmd);				
			}
			
			return rtn;
		}

		@Override
		public String getName() {
			return catalogName;
		}

		@Override
		public Icon getIconImage() {
			return IconImage.CATALOG.getValue();
		}

		@Override
		public boolean isStrong() {
			return false;
		}

		@Override
		public boolean isBottom() {
			return false;
		}

		@Override
		public String getSummary() {
			return "";
		}

	}

	@Override
	public String toString() {
		return "CATALOG";
	}

}
