package jdbcacsess.sql;

import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import javax.swing.Icon;

import jdbcacsess.gui.icons.IconImage;
import jdbcacsess.sql.DBObjectCatalogs.DBObjectCatalog;
import jdbcacsess.sql.DBObjectSchemas.DBObjectSchema;

public class DBObjectProcedures extends DBObjects {

	static final String[] header = { "PROCEDURE_CAT", "PROCEDURE_SCHEM",
			"PROCEDURE_NAME", "REMARKS", "PROCEDURE_TYPE" };

	private ArrayList<DBObject> procedures = new ArrayList<DBObject>();

	public DBObjectProcedures(DBObjectCatalog catalog, DBObjectSchema schema,
			DatabaseMetaData dmd) throws SQLException {
		
		ResultSet rs = dmd.getProcedures(catalog.getCatalogName(), schema
				.getSchemaName(), null);
		
		DBObjectProcedure prev = new DBObjectProcedure();
		while (rs.next()) {
			// 同一のプロシジャ名は捨てる（オーバロード分）
			DBObjectProcedure procedure = new DBObjectProcedure(rs);
			if (!prev.getKey().equals(procedure.getKey())){
				procedures.add(procedure);
				prev = procedure;
			}else{
				prev.incrementDuplicateCnt();
			}
		}
		rs.close();
	}

	@Override
	public ArrayList<DBObject> getDBObjects() {
		return procedures;
	}

	public class DBObjectProcedure extends DBObject {

		private String procedureCat="";

		private String procedureSchem="";

		private String procedureName="";

		private String remarks="";

		private short procedureType=0;
		
		private int duplicateCnt=1;

		public DBObjectProcedure(){
		}
		
		public DBObjectProcedure(ResultSet rs) throws SQLException {
			int colCnt = rs.getMetaData().getColumnCount();
			if (colCnt>0) procedureCat = rs.getString(1);
			if (colCnt>1) procedureSchem = rs.getString(2);
			if (colCnt>2) procedureName = rs.getString(3);
			if (colCnt>6) remarks = rs.getString(7);
			if (colCnt>7) procedureType = rs.getShort(8);
		}

		@Override
		public String[] getDetail() {
			String[] rtn = new String[5];
			rtn[0] = procedureCat;
			rtn[1] = procedureSchem;
			rtn[2] = procedureName;
			rtn[3] = remarks;
			rtn[4] = Short.toString(procedureType);
			return rtn;
		}

		@Override
		public String toString() {
			return procedureName + (duplicateCnt>1 ? "  (*"+duplicateCnt+")" : "");
		}

		@Override
		public String[] getHeader() {
			return header;
		}

		public String getProcedureCat() {
			return procedureCat;
		}

		public String getProcedureName() {
			return procedureName;
		}

		public String getProcedureSchem() {
			return procedureSchem;
		}

		public short getProcedureType() {
			return procedureType;
		}

		public String getRemarks() {
			return remarks;
		}

		public int getDuplicateCnt() {
			return duplicateCnt;
		}

		public void setDuplicateCnt(int duplicateCnt) {
			this.duplicateCnt = duplicateCnt;
		}

		public void incrementDuplicateCnt() {
			this.duplicateCnt++;
		}
		
		public String getKey(){
			return procedureCat+procedureSchem+procedureName;
		}

		@Override
		public DBObjects getChildren(DatabaseMetaData dmd) throws SQLException {
			// TODO 自動生成されたメソッド・スタブ
			return null;
		}

		@Override
		public String getName() {
			return procedureName;
		}

		@Override
		public Icon getIconImage() {
			return IconImage.PROCEDURE.getValue();
		}

		@Override
		public boolean isStrong() {
			return false;
		}

		@Override
		public boolean isBottom() {
			return false;
		}

		@Override
		public String getSummary() {
			return getRemarks();
		}
		
	}

	@Override
	public String toString() {
		return "PROCEDURE";
	}

}
