package jdbcacsess.sql;

import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import javax.swing.Icon;

import jdbcacsess.gui.icons.IconImage;
import jdbcacsess.sql.DBObjectCatalogs.DBObjectCatalog;

public class DBObjectSchemas extends DBObjects {

	static final String[] header = { "TABLE_SCHEM", "TABLE_CATALOG" };

	DBObjectCatalog dBObjectCatalog;
	
	private ArrayList<DBObject> schemas = new ArrayList<DBObject>();

	public DBObjectSchemas(DBObjectCatalog dBObjectCatalog, DatabaseMetaData dmd) throws SQLException {
		this.dBObjectCatalog = dBObjectCatalog;
		
		addSceham(dmd);
	}

	public DBObjectSchemas(DatabaseMetaData dmd)  throws SQLException 	{	
		this.dBObjectCatalog = null;
		
		addSceham(dmd);
	}
	
	private void addSceham(DatabaseMetaData dmd) throws SQLException {
		ResultSet rs = dmd.getSchemas();
		while (rs.next()) {
			schemas.add(new DBObjectSchema(rs));
		}
		rs.close();
	}

	@Override
	public ArrayList<DBObject> getDBObjects() {
		return schemas;
	}

	public class DBObjectSchema extends DBObject {

		private String schemaName;

		private String catalogName;

		public DBObjectSchema(ResultSet rs) throws SQLException {
			int colCnt = rs.getMetaData().getColumnCount();
			if (colCnt > 0)
				schemaName = rs.getString(1);
			if (colCnt > 1)
				catalogName = rs.getString(2);
		}

		public DBObjectSchema() {
		}

		@Override
		public String[] getDetail() {
			String[] rtn = new String[2];
			rtn[0] = catalogName;
			rtn[1] = schemaName;
			return rtn;
		}

		@Override
		public String toString() {
			if (schemaName == null) {
				return "(無し)";
			}
			return schemaName;
		}

		@Override
		public String[] getHeader() {
			return header;
		}

		public String getCatalogName() {
			return catalogName;
		}

		public String getSchemaName() {
			return schemaName;
		}

		public DBObjectCatalog getDBObjectCatalog(){
			return dBObjectCatalog;
		}

		@Override
		public DBObjects getChildren(DatabaseMetaData dmd) throws SQLException {
			return new DBObjectTableTypes(this,dmd);
		}

		@Override
		public String getName() {
			return schemaName;
		}

		@Override
		public Icon getIconImage() {
			return IconImage.SCHEMA.getValue();
		}

		@Override
		public boolean isStrong() {
			return false;
		}

		@Override
		public boolean isBottom() {
			return false;
		}

		@Override
		public String getSummary() {
			return "";
		}

	}

	@Override
	public String toString() {
		return "SCHEMA";
	}
}
