package jdbcacsess.sql;

import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import javax.swing.Icon;

import jdbcacsess.gui.icons.IconImage;
import jdbcacsess.sql.ConstSqlTypes.Categoly;
import jdbcacsess.sql.DBObjectTables.DBObjectTable;


public class DBObjectTableColumns extends DBObjects {

	static final String[] header = { "TABLE_CAT", "TABLE_SCHEM", "TABLE_NAME", "COLUMN_NAME", "DATA_TYPE", "TYPE_NAME",
			"COLUMN_SIZE", "BUFFER_LENGTH", "DECIMAL_DIGITS", "NUM_PREC_RADIX", "NULLABLE", "REMARKS", "COLUMN_DEF",
			"SQL_DATA_TYPE", "SQL_DATETIME_SUB", "CHAR_OCTET_LENGTH", "ORDINAL_POSITION", "IS_NULLABLE",
			"SCOPE_CATLOG", "SCOPE_SCHEMA", "SCOPE_TABLE", "SOURCE_DATA_TYPE", "IS_AUTOINCREMENT" };	
	
	
	private ArrayList<DBObject> tableColumns = new ArrayList<DBObject>();

	DBObjectTable dbObjectTable;

	public DBObjectTableColumns(
			DBObjectTable table, DatabaseMetaData dmd)
			throws SQLException {

//		dbObjectTable = table;
		
		Map<String, Short> pkmap = new HashMap<String, Short>();
		ResultSet pkrs = dmd.getPrimaryKeys(table.getTableCatalog(), table.getTableSchema(),table.getTableName());
		while (pkrs.next()) {
			pkmap.put(pkrs.getString(4),pkrs.getShort(5));
		}
		pkrs.close();
		

		
		ResultSet rs = dmd.getColumns(table.getTableCatalog(), table.getTableSchema(),table.getTableName(), null);	
		while (rs.next()) {
			tableColumns.add(new DBObjectTableColumn(rs,pkmap));
		}
		rs.close();
		
	}

	@Override
	public ArrayList<DBObject> getDBObjects() {
		return tableColumns;
	}
	public class DBObjectTableColumn extends DBObject {
		

		public String getTableCatalog() {
			return tableCatalog;
		}

		public String getTableSchema() {
			return tableSchema;
		}

		public String getTableName() {
			return tableName;
		}

		public String getColumnName() {
			return columnName;
		}

		public int getDataType() {
			return dataType;
		}

		public String getDataName() {
			return dataName;
		}

		public int getColumnSize() {
			return columnSize;
		}

		public int getDecimalDigits() {
			return decimalDigits;
		}

		public int getNumPrecRadix() {
			return numPrecRadix;
		}

		public int getNullable() {
			return nullable;
		}

		public String getRemarks() {
			if (remarks==null){
				return "";
			}
			return remarks;
		}

		public String getColumnDef() {
			return columnDef;
		}

		public int getSqlDataType() {
			return sqlDataType;
		}

		public int getSqlDatetimeSub() {
			return sqlDatetimeSub;
		}

		public int getCharOctetLength() {
			return charOctetLength;
		}

		public int getOrdinalPosition() {
			return ordinalPosition;
		}

		public String getIsNullable() {
			return isNullable;
		}

		public String getScopeCatalog() {
			return scopeCatalog;
		}

		public String getScopeSchema() {
			return scopeSchema;
		}

		public String getScopeTable() {
			return scopeTable;
		}

		public short getSourceDataType() {
			return sourceDataType;
		}

		public String getIsAutoincrement() {
			return isAutoincrement;
		}

		String	tableCatalog;
		String	tableSchema;
		String	tableName;
		String	columnName;

		int		dataType;
		String	dataName;
		int		columnSize;
//		-	BUFFER_LENGTH;

		int		decimalDigits;
		int		numPrecRadix;
		int	    nullable;
		
		String	remarks;
		String	columnDef;
		int		sqlDataType;
		int		sqlDatetimeSub;
		int		charOctetLength;
		int		ordinalPosition;

		String	isNullable;
		String	scopeCatalog;
		String	scopeSchema;
		String	scopeTable;
		short	sourceDataType;
		String	isAutoincrement;
		
		Short primaryKeySeqNo;
		
		StringBuffer summary;

		public DBObjectTableColumn(ResultSet rs, Map<String, Short> pkmap) throws SQLException {
			int colCnt = rs.getMetaData().getColumnCount();
			if (colCnt > 0)
				tableCatalog = rs.getString(1);
			if (colCnt > 1)
				tableSchema = rs.getString(2);
			if (colCnt > 2)
				tableName = rs.getString(3);
			if (colCnt > 3)
				columnName = rs.getString(4);
			
			if (colCnt > 4)
				dataType = rs.getInt(5);
			if (colCnt > 5)
				dataName = rs.getString(6);
			if (colCnt > 6)
				columnSize = rs.getInt(7);

//			-	BUFFER_LENGTH;
			
			if (colCnt > 8)
				decimalDigits = rs.getInt(9);
			if (colCnt > 9)
				numPrecRadix = rs.getInt(10);
			if (colCnt > 10)
				nullable = rs.getInt(11);

			if (colCnt > 11)
				remarks = rs.getString(12);
			if (colCnt > 12)
				columnDef = rs.getString(13);

			if (colCnt > 13)
				sqlDataType = rs.getInt(14);
			if (colCnt > 14)
				sqlDatetimeSub = rs.getInt(15);
			if (colCnt > 15)
				charOctetLength = rs.getInt(16);
			if (colCnt > 16)
				ordinalPosition = rs.getInt(17);

			if (colCnt > 17)
				isNullable = rs.getString(18);
			if (colCnt > 18)
				scopeCatalog = rs.getString(19);
			if (colCnt > 19)
				scopeSchema = rs.getString(20);
			if (colCnt > 20)
				scopeTable = rs.getString(21);

			if (colCnt > 21)
				sourceDataType = rs.getShort(22);
			if (colCnt > 22)
				isAutoincrement = rs.getString(23);
			
			primaryKeySeqNo = pkmap.get(columnName);
			
			summary = new StringBuffer();
			if ( primaryKeySeqNo!=null ){
				summary.append(primaryKeySeqNo);
				summary.append(' ');
			}
			summary.append(columnName);
			summary.append(' ');
			ConstSqlTypes constSqlTypes = ConstSqlTypes.searchTypes(dataType);
			summary.append(constSqlTypes.name());
			summary.append('(');
			summary.append(columnSize);
			if ( constSqlTypes.getCategoly() == Categoly.NUM ){
				if ( decimalDigits!=0 ){
					summary.append('.');
					summary.append(decimalDigits);
				}
			}
			summary.append(')');
			summary.append( (remarks!=null? remarks:"") );
		}

		@Override
		public String[] getDetail() {
			String[] rtn = new String[10];
			rtn[0] = tableCatalog;
			rtn[1] = tableSchema;
			rtn[2] = tableName;
			rtn[3] = columnName;
			
			rtn[4] = Integer.toString(dataType);
			rtn[5] = dataName;
			rtn[6] = Integer.toString(columnSize);
			
			rtn[7] = "";
			rtn[8] = Integer.toString(decimalDigits);
			rtn[9] = Integer.toString(numPrecRadix);
			rtn[10] = Integer.toString(nullable);
			
			rtn[11] = remarks;
			rtn[12] = columnDef;
			
			rtn[13] = Integer.toString(sqlDataType);
			rtn[14] = Integer.toString(sqlDatetimeSub);
			rtn[15] = Integer.toString(charOctetLength);
			rtn[16] = Integer.toString(ordinalPosition);
			
			rtn[17] = isNullable;
			rtn[18] = scopeCatalog;
			rtn[19] = scopeSchema;
			rtn[20] = scopeTable;
			rtn[21] = Short.toString(sourceDataType);
			rtn[22] = isAutoincrement;

			return rtn;
		}

		@Override
		public DBObjects getChildren(DatabaseMetaData dmd) throws SQLException {
			return null;
		}

		@Override
		public String[] getHeader() {
			return header;
		}

		@Override
		public String toString() {
			return columnName;
		}

		@Override
		public SchemaTableName getSchemaTableName() {
			return new SchemaTableName(tableSchema,tableName);
		}

		@Override
		public String getName() {
			return columnName;
		}

		@Override
		public Icon getIconImage() {
			return IconImage.TABLECOLUMN.getValue();
		}

		@Override
		public boolean isStrong() {
			return isPrimaryKey();
		}

		public boolean isPrimaryKey() {
			return (primaryKeySeqNo!=null);
		}

		@Override
		public boolean isBottom() {
			return true;
		}

		@Override
		public String getSummary() {
			return summary.toString();
		}

	}

	@Override
	public String toString() {
		return "TABLE_COLUMNS";
	}

}
