package jdbcacsess.sql;

import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import javax.swing.Icon;

import jdbcacsess.gui.icons.IconImage;
import jdbcacsess.sql.DBObjectSchemas.DBObjectSchema;
import jdbcacsess.sql.DBObjectTableTypes.DBObjectTableType;

public class DBObjectTables extends DBObjects {

	static final String[] header = { "TABLE_CAT", "TABLE_SCHEM", "TABLE_NAME",
			"TABLE_TYPE", "REMARKS", "TYPE_CAT", "TYPE_SCHEM", "TYPE_NAME",
			"SELF_REFERENCING_COL_NAME", "REF_GENERATION" };

	private ArrayList<DBObject> tables = new ArrayList<DBObject>();

	DBObjectSchema dbObjectSchema;
	DBObjectTableType dbObTableType;
	public DBObjectTables(DBObjectSchema schema,
			DBObjectTableType tableType, DatabaseMetaData dmd)
			throws SQLException {
		dbObjectSchema = schema;
		dbObTableType = tableType;
		
		String[] type = new String[1];
		type[0] = tableType.getTableTypeName();
		ResultSet rs;
		if (schema==null){
			rs = dmd.getTables(null, null, null, type);
		}else{
			rs = dmd.getTables(schema.getCatalogName(), schema.getSchemaName(), null, type);	
		}
		while (rs.next()) {
			tables.add(new DBObjectTable(rs));
		}
		rs.close();
	}

	@Override
	public ArrayList<DBObject> getDBObjects() {
		return tables;
	}

	public class DBObjectTable extends DBObject {

		String tableCatalog;

		String tableSchema;

		String tableName;

		String tableType;

		String remarks;

		String typeCatalog;

		String typeSchema;

		String typeName;

		String selfReferencingColName;

		String refGeneration;

		public DBObjectTable(ResultSet rs) throws SQLException {
			int colCnt = rs.getMetaData().getColumnCount();
			if (colCnt > 0)
				tableCatalog = rs.getString(1);
			if (colCnt > 1)
				tableSchema = rs.getString(2);
			if (colCnt > 2)
				tableName = rs.getString(3);
			if (colCnt > 3)
				tableType = rs.getString(4);
			if (colCnt > 4)
				remarks = rs.getString(5);
			if (colCnt > 5)
				typeCatalog = rs.getString(6);
			if (colCnt > 6)
				typeSchema = rs.getString(7);
			if (colCnt > 7)
				typeName = rs.getString(8);
			if (colCnt > 8)
				selfReferencingColName = rs.getString(9);
			if (colCnt > 9)
				refGeneration = rs.getString(10);
		}

		@Override
		public String[] getDetail() {
			String[] rtn = new String[10];
			rtn[0] = tableCatalog;
			rtn[1] = tableSchema;
			rtn[2] = tableName;
			rtn[3] = tableType;
			rtn[4] = remarks;
			rtn[5] = typeCatalog;
			rtn[6] = typeSchema;
			rtn[7] = typeName;
			rtn[8] = selfReferencingColName;
			rtn[9] = refGeneration;

			return rtn;
		}

		@Override
		public String toString() {
			return tableName;
		}

		@Override
		public String[] getHeader() {
			return header;
		}

		public String getRefGeneration() {
			return refGeneration;
		}

		public String getRemarks() {
			if (remarks==null){
				return "";
			}
			return remarks;
		}

		public String getSelfReferencingColName() {
			return selfReferencingColName;
		}

		public String getTableCatalog() {
			return tableCatalog;
		}

		public String getTableName() {
			return tableName;
		}

		public String getTableSchema() {
			return tableSchema;
		}

		public String getTableType() {
			return tableType;
		}

		public String getTypeCatalog() {
			return typeCatalog;
		}

		public String getTypeName() {
			return typeName;
		}

		public String getTypeSchema() {
			return typeSchema;
		}

		@Override
		public DBObjects getChildren(DatabaseMetaData dmd) throws SQLException {
			return new DBObjectTableColumns(this, dmd);
		}
		
		@Override
		public SchemaTableName getSchemaTableName() {
			return new SchemaTableName(tableSchema,tableName);
		}

		@Override
		public String getName() {
			return tableName;
		}

		@Override
		public Icon getIconImage() {
			if(dbObTableType.getTableTypeName().equals("TABLE")){
				return IconImage.TABLE.getValue();
			}
			if(dbObTableType.getTableTypeName().equals("VIEW")){
				return IconImage.VIEW.getValue();
			}
			return IconImage.TABLETYPE.getValue();
		}

		@Override
		public boolean isStrong() {
			return false;
		}

		@Override
		public boolean isBottom() {
			return false;
		}

		@Override
		public String getSummary() {
			return getRemarks();
		}

	}

	@Override
	public String toString() {
		return "TABLE";
	}

}
