package jdbcacsess.sql;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ParameterMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import jdbcacsess.JdbcacsessException;

public class ExecuteSqCallable extends ExecuteSqlFactory {

	private PrepareExecuteCallable prepareExecute = null;
	private ResultSet resultSet;
	private boolean haveParameter=false;
	private boolean haveResultSet=false;
	
	private SqlAnalyze sqlAnalyze;
	private Connection connection;
	
	public ExecuteSqCallable(Object source, SqlAnalyze SqlAnalyze, Connection connection) throws JdbcacsessException{
		super(source);
		this.sqlAnalyze = SqlAnalyze;
		this.connection = connection;
	}

	@Override
	SqlAnalyze getSqlAnalyze() {
		return sqlAnalyze;
	}
	
	/* (non-Javadoc)
	 * @see jdbcacsess.sql.ExecuteSqlFactory#initMethod()
	 */
	@Override
	boolean initMethod() throws SQLException, Exception {
		prepareExecute = new PrepareExecuteCallable();
		
		prepareExecute.addStm(getSqlAnalyze().getSqlStatement());
		
		prepareExecute.prepare(connection);
		
		if (!solveParameter(prepareExecute)){
			return false;
		}
		
		ParameterMetaData pmd = prepareExecute.getPrepared().getParameterMetaData();
		if (pmd==null){
			throw new JdbcacsessException("getParameterMetaData() の戻り値が null でした。");
		}
		
		for (int i = 1; i < pmd.getParameterCount() + 1; i++) {
			int mode = pmd.getParameterMode(i);
			if (mode == ParameterMetaData.parameterModeInOut ||
				mode == ParameterMetaData.parameterModeOut||
				mode == DatabaseMetaData.procedureColumnReturn){
				ColumnInfoParameter colInfo = new ColumnInfoParameter(
						i,
						"Parameter" + Integer.valueOf(i).toString(),
						pmd.getParameterType(i),
						pmd.getParameterTypeName(i),
						pmd.isNullable(i),
						pmd.getParameterClassName(i),
						pmd.getPrecision(i),
						pmd.getScale(i),
						mode
						);
				getSqlMappingInfo().search(colInfo, isBinaryDatalimit());
				selectColmunInfos.add(colInfo);
				
				if (colInfo.getSqlType()== ConstSqlTypes.NUMERIC || colInfo.getSqlType()== ConstSqlTypes.DECIMAL){
					prepareExecute.getPrepared().registerOutParameter(i,colInfo.getSqlType().getValue(),colInfo.getScale());
				}else{
					prepareExecute.getPrepared().registerOutParameter(i,colInfo.getSqlType().getValue());
				}
				
				haveParameter = true;
			}
		}
		
		if (prepareExecute.execute()){
			resultSet = prepareExecute.getPrepared().getResultSet();
			getHeaderFromResultSet(resultSet, selectColmunInfos);
			haveResultSet = true;
		}
		
		return true;
	}
	
	/* (non-Javadoc)
	 * @see jdbcacsess.sql.ExecuteSqlFactory#hasNext()
	 */
	@Override
	boolean hasNext() throws SQLException {
		if ( !haveParameter && !haveResultSet){
			return false;
		}
		
		boolean rtn = false;
		
		if (haveParameter && getCnt()==0){
			rtn = true;
		}
		
		if(haveResultSet){
			rtn = rtn | resultSet.next();
		}
		
		return rtn;
	}

	/* (non-Javadoc)
	 * @see jdbcacsess.sql.ExecuteSqlFactory#getResultDetail()
	 */
	@Override
	ArrayList<Object> getResultDetail() throws IllegalArgumentException, IllegalAccessException,
			InvocationTargetException, SQLException, IOException {
		ArrayList<Object> list = new ArrayList<Object>();
		for(ColumnInfoResult cir: selectColmunInfos){
			if (cir instanceof ColumnInfoParameter) {
				Object o=null; // ２行目以降のParameterカラムはnullにする
				if (getCnt()==0){
					o = prepareExecute.getPrepared().getObject(((ColumnInfoParameter)cir).columnIndex);
				}
				list.add(o);
			}else{
				break;
			}
		}
		
		if (haveResultSet){
			getDetailFromResultSet(resultSet,list);
		}
		
		return list;
	}

	
	/* (non-Javadoc)
	 * @see jdbcacsess.sql.ExecuteSqlFactory#endMethod()
	 */
	@Override
	void endMethod() {
		try {
			if (prepareExecute!=null){
				prepareExecute.close();
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}


}
