package jdbcacsess.sql;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;

import jdbcacsess.JdbcacsessException;
import jdbcacsess.gui.JDialogPrepareParameter;
import jdbcacsess.gui.JFrameMain;
import jdbcacsess.sql.column.GetColumnResultSet;

abstract class ExecuteSqlFactory {

	private int cnt;
	private ArrayList<QueryExecuteListener> listeners;
	
	private Object source;
	
	ArrayList<ColumnInfoResult> selectColmunInfos;
	
	private SqlMappingInfo sqlMappingInfo;
	
	private boolean binaryDatalimit;

	/**
	 * @param sqlAnalyze
	 *            実行するＳＱＬ
	 * @throws JdbcacsessException 
	 */
	static  ExecuteSqlFactory getFactory(Object source, SqlAnalyze sqlAnalyze, Connection connection) throws JdbcacsessException{
		if (sqlAnalyze.getSqlStatementCatgoly()==SqlStatementCatgoly.SELECT){
			return new ExecuteStatement(source, sqlAnalyze, connection);
		}
		if (sqlAnalyze.getSqlStatementCatgoly()==SqlStatementCatgoly.CALL){
			return new ExecuteSqCallable(source, sqlAnalyze, connection);
		}
		
		return null;
	}
	
	/**
	 * @param source
	 * @throws JdbcacsessException 
	 */
	ExecuteSqlFactory(Object source) throws JdbcacsessException{
		this.source = source;

		this.binaryDatalimit = false;

		listeners = new ArrayList<QueryExecuteListener>();
		sqlMappingInfo = new SqlMappingInfo();
		
	}

	/**
	 * 結果を受取るリスナーを登録する
	 * 
	 * @param listener
	 */
	void addQueryExecuteListener(QueryExecuteListener listener) {
		listeners.add(listener);
	}

	/**
	 * @return sqlMappingInfo
	 */
	SqlMappingInfo getSqlMappingInfo() {
		return sqlMappingInfo;
	}

	/**
	 * @return sqlAnalyze
	 */
	abstract SqlAnalyze getSqlAnalyze();

	/**
	 * @return
	 */
	int getCnt() {
		return cnt;
	}
	
	/**
	 * バイナリデータの読み込み制限を行うか
	 * 
	 * @return
	 */
	boolean isBinaryDatalimit() {
		return binaryDatalimit;
	}

	/**
	 * バイナリデータの読み込み制限を行うかを設定する。デフォルトでは無制限です。
	 */
	void setBinaryDatalimit(boolean binaryDatalimit) {
		this.binaryDatalimit = binaryDatalimit;
	}


	/**
	 * @return
	 * @throws Exception
	 */
	 boolean notifyOfInitAndResultHeader() throws Exception {
		SqlAnalyze sqlAnalyze = getSqlAnalyze();
		selectColmunInfos = new ArrayList<ColumnInfoResult>();
		
		boolean rtn = initMethod();
		if (!rtn){
			return rtn;
		}
		
		boolean selectUpdatable =false;
		if (sqlAnalyze!=null){
			selectUpdatable = sqlAnalyze.isSelectUpdatable(selectColmunInfos);
		}
		
		for (QueryExecuteListener listener : listeners) {
			listener.init();
		}

		for (QueryExecuteListener listener : listeners) {
			listener.setResultHeader(selectColmunInfos, sqlAnalyze,selectUpdatable);
		}

		return rtn;
	}
	
	/**
	 * カーソルの作成とヘッダ情報の通知
	 * 
	 * @throws Exception
	 */
	abstract boolean initMethod() throws SQLException, Exception ;


	/**
	 * @return
	 * @throws IllegalArgumentException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 * @throws SQLException
	 * @throws IOException
	 */
	boolean notifyOfResultDetail() throws IllegalArgumentException, IllegalAccessException, InvocationTargetException, SQLException, IOException {
		if (! hasNext()){
			return false;
		}
		ArrayList<Object> rowData = getResultDetail();

		// get済件数をカウントアップ
		cnt++;
		
		for (QueryExecuteListener listener : listeners) {
			listener.setResultDetail(rowData);
		}
		return true;
	}
	
	/**
	 * 結果セットを１行進める。
	 * 
	 * @return 1行進められたらtrue。
	 * @throws SQLException
	 */
	abstract boolean hasNext() throws SQLException ;

	/**
	 * 検索結果の通知
	 * 
	 * @throws IllegalArgumentException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 * @throws SQLException
	 * @throws IOException
	 */
	abstract ArrayList<Object> getResultDetail() throws IllegalArgumentException, IllegalAccessException, InvocationTargetException, SQLException, IOException;


	/**
	 * 処理経過を通知
	 * 
	 * @param runner
	 * @param err
	 */
	void notifyOfStatus(boolean runner, boolean err) {
		
		QueryExecuteStatus qes = new QueryExecuteStatus(source, cnt, runner, err);

		for (QueryExecuteListener listener : listeners) {
			listener.status(qes);
		}
	}

	/**
	 * 例外の通知
	 * 
	 * @param e
	 */
	void notifyOfErrorException(Exception e) {

		e.printStackTrace();
		
		for (QueryExecuteListener listener : listeners) {
			listener.errorException(e);
		}
	}

	/**
	 * @param endOfData
	 * @param err
	 * @throws JdbcacsessException 
	 */
	void notifyOfCompleteQuery(boolean endOfData, boolean err) throws JdbcacsessException {

		endMethod();
		
		QueryExecuteStatus qes = new QueryExecuteStatus(source, cnt, endOfData, err);
		
		for (QueryExecuteListener listener : listeners) {
			listener.completeQuery(qes);
		}
	}
	
	/**
	 * カーソルクローズと検索終了の通知
	 * 
	 * @param runner
	 * @param err
	 */
	abstract void endMethod();

	/**
	 * @param prepareExecute
	 * @return
	 * @throws JdbcacsessException
	 */
	boolean solveParameter(PrepareExecute prepareExecute) throws JdbcacsessException{
		if (prepareExecute.isHavingInputParameter()) {
			JDialogPrepareParameter jdpp = new JDialogPrepareParameter(prepareExecute);
			if (!jdpp.isExecuteAns()) {
				return false;
			}
		}
		return true;
	}

	/**
	 * @param resultSet
	 * @param header
	 * @throws Exception
	 */
	void getHeaderFromResultSet(ResultSet resultSet, ArrayList<ColumnInfoResult> header) throws Exception  {
		
		// SQL発行及びレコードセットの取得
		int col = resultSet.getMetaData().getColumnCount();
		ResultSetMetaData rsm = resultSet.getMetaData();
		
		// カラム名を取得しVector（ColumnInfoResult）へ設定
		for (int i = 1; i <= col; i++) {
		
			// 引数が多すぎだ
			ColumnInfoResult colInfo = new ColumnInfoResult(i, rsm.getColumnName(i), rsm.getColumnType(i), rsm
					.getColumnTypeName(i), rsm.isNullable(i), rsm.getColumnClassName(i), rsm.getPrecision(i), rsm
					.getScale(i));
		
			getSqlMappingInfo().search(colInfo, binaryDatalimit);
		
			header.add(colInfo);
		}
	}
	
	/**
	 * @param resultSet
	 * @param rowData
	 * @throws SQLException
	 */
	void getDetailFromResultSet(ResultSet resultSet, ArrayList<Object> rowData) throws SQLException {
		for (int i = 1; i <= resultSet.getMetaData().getColumnCount(); i++) {
			if (JFrameMain.debugMode && getCnt() == 0) {
				JFrameMain.getInstanceJPanelResultSetMetaData().setData(resultSet);
			}
		
			try {
				// MaxDB ではデータ型LONGの時、getObjectだとjava.io.Reader が返り、
				// TypesはTypes.LONGVARCHARになる。
				// 将来JDBCドライバが、Types.CLOB に対応してくれるかもしれない。
				if (resultSet.getMetaData().getColumnType(i) == Types.CLOB
						|| selectColmunInfos.get(i - 1).getColumnClass() == java.io.Reader.class) {
					Clob clob = resultSet.getClob(i);
					if (isAddNull(resultSet,rowData, clob))
						continue;
					rowData.add(clob.getSubString(1, (int) clob.length()));
					continue;
				}
		
				// 取得に使用する独自クラスが指定されている時
				GetColumnResultSet gcrs = selectColmunInfos.get(i - 1).getColumnResultSet;
				if (gcrs != null) {
					rowData.add(gcrs.getData(resultSet, i));
					continue;
				}
		
				// 取得に使用するResultSetのメソッド名が指定されている時
				Method method = selectColmunInfos.get(i - 1).methodOfResultSet;
				if (method != null) {
					Object obj = method.invoke(resultSet, i);
					if (!isAddNull(resultSet,rowData, obj)) {
						rowData.add(obj);
					}
					continue;
				}
		
				// 特別対応が不要な場合
				Object obj = resultSet.getObject(i);
				if (!isAddNull(resultSet,rowData, obj)) {
					rowData.add(obj);
				}
			} catch (Exception e) {
				rowData.add(new JdbcacsessException(e));
			}
		}
	}
	/**
	 * 検索結果がNullであるか判定し、Nullであれば結果リストに追加する。Nullでなければ何もしない。
	 * 
	 * @param rowData
	 *            結果を設定するリスト
	 * @param data
	 *            検索結果
	 * @return Nullであるかの判定結果。NULLであればtrue。
	 * @throws SQLException
	 */
	private boolean isAddNull(ResultSet resultSet,ArrayList<Object> rowData, Object data) throws SQLException {
		if (resultSet.wasNull() || data == null) {
			rowData.add(null);
			return true;
		}
		return false;
	}

}
