package jdbcacsess.sql;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * @author sima prepareで更新系参照系SQL文を発行する。 パラメータは単純なテキストではなく、オブジェクトのままでドライバに渡す。
 */
public class PrepareExecuteStatement extends PrepareExecute {
	
	public PrepareExecuteStatement() {
		super();
	}

	/**
	 * prepareStatementを実行します
	 * 
	 * @param con
	 * @throws SQLException
	 */
	@Override
	public void prepare(Connection con) throws SQLException {
		msg("PrepareExecuteStatement: AutoCimmit "+ con.getAutoCommit() );
		
		msg("PrepareExecuteStatement: native "+ con.nativeSQL(statement.toString() ));
		
		prepared = con.prepareStatement(statement.toString());
	}

	/**
	 * 検索系ＳＱＬの実行だけします。prepareは行ないませんが、パラメータは実行後自動クリアされます。
	 * 
	 * @return 結果セット
	 * @throws SQLException
	 */
	public ResultSet executeQuery() throws SQLException {
		ResultSet rs = null;
		try {
			setParm();
			msg("PrepareExecuteStatement: execute query start");
			
			rs = prepared.executeQuery();
			
			msg("PrepareExecuteStatement: ---------- execute query end");
		} catch (OutOfMemoryError err) {
			throw new SQLException(err.getMessage());
		} finally {
			clearParm();
		}
		return rs;
	}

	/**
	 * 更新系ＳＱＬの実行だけします。prepareは行ないませんが、パラメータは実行後自動クリアされます。
	 * 
	 * @return
	 * @throws SQLException
	 */
	public int executeUpdate() throws SQLException {
		int cnt;
		try {
			setParm();
			msg("PrepareExecuteStatement: execute update start");
			
			cnt = prepared.executeUpdate();

			msg("PrepareExecuteStatement: ---------- execute update end");
		} catch (OutOfMemoryError err) {
			throw new SQLException(err.getMessage());
		} finally {
			clearParm();
		}
		return cnt;
	}
}
