package jdbcacsess.sql;

import java.sql.Connection;

import jdbcacsess.JdbcacsessException;

/**
 * 
 * 途中経過をQueryExecuteListener.statusで受け取ることは出来ません
 * 
 * @author sima
 * 
 */
public class QueryExecuteSync {

	private ExecuteSqlFactory executeSql;
	
	private boolean err = false;

	public QueryExecuteSync(Object source, SqlAnalyze sqlAnalyze, Connection con) throws JdbcacsessException {
		executeSql = ExecuteSqlFactory.getFactory(source,sqlAnalyze,con);
	}

	public void addQueryExecuteListener(QueryExecuteListener listener) {
		executeSql.addQueryExecuteListener(listener);
	}
	
	/**
	 * 処理処理を行います<br>
	 * QueryExecuteListener.init() の通知<br>
	 * SQL文の準備とカーソルのオープン<br>
	 * QueryExecuteListener.setResultHeader() の通知<br>
	 * <br>
	 * エラー時は、<br>
	 * QueryExecuteListener.errorExceptionの通知<br>
	 * を行います。
	 */
	public void open() {
		try {
			if (!executeSql.notifyOfInitAndResultHeader()){
				return;
			}
		} catch (Exception e) {
			executeSql.notifyOfErrorException(e);
			err = true;
		}
	}

	/**
	 * 行の読み込み及びその結果データの通知を行います。データを読み込めたか?を返します<br>
	 * カーソルのfetch<br>
	 * QueryExecuteListener.setResultDetail の通知<br>
	 * <br>
	 * エラー時は、<br>
	 * QueryExecuteListener.errorExceptionの通知<br>
	 * を行います。
	 * 
	 * @return boolean データが終了した場合はtrue。
	 */
	public boolean read() {
		try {
			// 検索行データの取得と通知
			if (! executeSql.notifyOfResultDetail()) {
				return false;
			}
			
		} catch (Error error) {
			executeSql.notifyOfErrorException(new Exception(error));
			err = true;
			return false;
		} catch (Exception e) {
			executeSql.notifyOfErrorException(e);
			err = true;
			return false;
		}

		return true;
	}

	/**
	 * カーソルのクローズと終了処理<br>
	 * QueryExecuteListener.completeQuery の通知<br>
	 * @throws JdbcacsessException 
	 */
	public void close() throws JdbcacsessException {
		executeSql.notifyOfCompleteQuery(false, err);
	}

}
