package jdbcacsess.sql;

import java.sql.Connection;

import jdbcacsess.JdbcacsessException;
import jdbcacsess.gui.common.JDialogMessage;

/**
 * 別スレッドでコマンド実行＆カラム名取得＆結果取得を行う。
 */
public class QueryExecuteThread implements Runnable {

	private ExecuteSqlFactory executeSql;
	
	private boolean endOfData;
	
	public QueryExecuteThread(Object source, SqlAnalyze sqlAnalyze, Connection con) throws JdbcacsessException{
		executeSql = ExecuteSqlFactory.getFactory(source, sqlAnalyze,con);
	}
	
	public void addQueryExecuteListener(QueryExecuteListener listener) {
		executeSql.addQueryExecuteListener(listener);
	}
	public void setBinaryDatalimit(boolean binaryDatalimit) {
		executeSql.setBinaryDatalimit(binaryDatalimit);
	}


	/*
	 * (非 Javadoc)
	 * 
	 * @see java.lang.Runnable#run()
	 */
	public void run() {

		endOfData = true;
		int notifyCnt = 0;
		boolean err = false;

		// データ取得
		try {
			if (!executeSql.notifyOfInitAndResultHeader()){
				// 途中で中断した時と同じ扱いにするのでエラーセット
				err=true;
				return;
			}

			// SELECT実行結果を行単位に取得する
			while (true) {
				if (endOfData == false) {
					// 中断時はループ強制脱出
					break;
				}

				// 検索行データの取得と通知
				if (! executeSql.notifyOfResultDetail() ) {
					break;
				}

				// 処理経過を通知。毎回だと辛いので100件毎
				notifyCnt++;
				if (notifyCnt > 99) {
					notifyCnt = 0;
					executeSql.notifyOfStatus(endOfData, false);
				}
			}

		} catch (Error error) {
			err = true;
			executeSql.notifyOfErrorException(new Exception(error));
			return;
		} catch (Exception exception) {
			err = true;
			executeSql.notifyOfErrorException(exception);
			return;
		} finally {
			// エラーがあっても無くても通知は行う
			try {
				executeSql.notifyOfCompleteQuery(endOfData, err);
			} catch (JdbcacsessException e) {
				JDialogMessage.errorDialog(e);
			}
		}
	}

	/**
	 * 検索結果の中断を示すフラグを設定
	 */
	public void setCancelFlg() {
		endOfData = false;
	}

}