package jdbcacsess.sql;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.StringTokenizer;


/**
 * <P>
 * SQL文を解析し、SQL文の種類、テーブル名、を抽出解析します。<br>
 * また、更新可能検索結果であるか判断する為に、 <br>
 * １）単一テーブル検索<br>
 * ２）主キーを含んだ検索<br>
 * であるかを判断します。<br>
 * </P>
 * <P>
 * ・SQL文の種類<br>
 * ・テーブル名<br>
 * ・単一テーブル検索<br>
 * ・主キーを含んだ検索<br>
 * がわかります。<br>
 * </p>
 * 
 * @author sima
 */
public class SqlAnalyze {

	private SchemaTableName schemaTableName;

	private boolean selectUpdatable;

	private String sqlStatement;

	private SqlStatementCatgoly sqlStatementCatgoly;

	private String errorReasonMsg;

	/**
	 * @param sqlStatement
	 *            SQL文
	 */
	public SqlAnalyze(String sqlStatement) {
		this.sqlStatement = sqlStatement.trim();

		this.selectUpdatable = false;
		this.schemaTableName = new SchemaTableName();
		this.errorReasonMsg = "";

		StringTokenizer st = new StringTokenizer(sqlStatement.replaceAll(",",
				" , "), " \t;" + System.getProperty("line.separator"));

		if (st.hasMoreTokens()) {

			String command = st.nextToken().toLowerCase();
			if (command.equals("select")) {
				sqlStatementCatgoly = SqlStatementCatgoly.SELECT;
				this.selectUpdatable = checkUpdateble(st);

			} else if (command.equals("insert"))
				sqlStatementCatgoly = SqlStatementCatgoly.INSERT;

			else if (command.equals("update"))
				sqlStatementCatgoly = SqlStatementCatgoly.UPDATE;

			else if (command.equals("delete"))
				sqlStatementCatgoly = SqlStatementCatgoly.DELETE;

			else if (command.equals("create"))
				sqlStatementCatgoly = SqlStatementCatgoly.CREATE;

			else if (command.equals("drop"))
				sqlStatementCatgoly = SqlStatementCatgoly.DROP;

			else if (command.equals("alter"))
				sqlStatementCatgoly = SqlStatementCatgoly.ALTER;

			else if (command.equals("grant"))
				sqlStatementCatgoly = SqlStatementCatgoly.GRANT;

			else if (command.equals("call"))
				sqlStatementCatgoly = SqlStatementCatgoly.CALL;

			else
				sqlStatementCatgoly = SqlStatementCatgoly.UNKNOWN;
		} else {
			sqlStatementCatgoly = SqlStatementCatgoly.UNKNOWN;
		}
		if ( this.sqlStatement.matches(" *\\? *= *[Cc][Aa][Ll][Ll] .*") ){
			sqlStatementCatgoly = SqlStatementCatgoly.CALL;
		}
	}

	/**
	 * 検索系ｓｑｌ文の結果が更新可能であるか判定する。 条件は以下の２つを満たす場合<br>
	 * <OL>
	 * <li>アクセス対象テーブルが１つだけである事</li>
	 * <li>選択リストのカラム名に、アクセス対象テーブルの主キーが全て含まれる事</li>
	 * </OL>
	 * 
	 * @param st
	 * @return 更新可能であるか
	 */
	private boolean checkUpdateble(StringTokenizer st) {

		// 最初のFROM迄進める
		while (true) {
			if (!st.hasMoreTokens()) {
				errorReasonMsg = "FROMがありません";
				System.out.println("（更新不可）FROMが無い");
				return false;
			}
			if (st.nextToken().toUpperCase().equals("FROM")) {
				break;
			}
		}

		if (!st.hasMoreTokens()) {
			errorReasonMsg = "FROMの後に何もありません";
			System.out.println("（更新不可）FROMの後に何も無い");
			return false;
		}

		// FROMの次の文字列をテーブル名と仮定する
		String tableName = st.nextToken();

		String tmpStr;
		for (int i = 0; i < 3; i++) {
			if (!st.hasMoreTokens()) {
				break;
			}
			tmpStr = st.nextToken().toUpperCase();
			if (tmpStr.equals(",") || tmpStr.equals("SELECT")
					|| tmpStr.equals("FULL") || tmpStr.equals("LEFT")
					|| tmpStr.equals("RIGHT") || tmpStr.equals("OUTER")
					|| tmpStr.equals("INNER") || tmpStr.equals("JOIN")) {
				errorReasonMsg = "複数テーブルの問い合わせでは更新できません";
				System.out.println("（更新不可）複数テーブルの問い合わせ");
				return false;
			}
		}

		// その後にSEECTがあるか調べる
		while (st.hasMoreTokens()) {
			if (st.nextToken().toUpperCase().equals("SELECT")) {
				errorReasonMsg = "SELECT文が複数あるので更新できません";
				System.out.println("（更新不可）SELECT文が複数ある");
				return false;
			}
		}

		this.schemaTableName = new SchemaTableName(tableName);
		return true;
	}

	/**
	 * @return schemaTableName を戻します。
	 */
	public SchemaTableName getSchemaTableName() {
		return schemaTableName;
	}

	/**
	 * @return sqlStatement を戻します。
	 */
	public String getSqlStatement() {
		return sqlStatement;
	}

	/**
	 * @return sqlStatementCatgoly を戻します。
	 */
	public SqlStatementCatgoly getSqlStatementCatgoly() {
		return sqlStatementCatgoly;
	}

	/**
	 * @return errorReasonMsg を戻します。
	 */
	public String getErrorReasonMsg() {
		return errorReasonMsg;
	}

	/**
	 * 更新可能かを判定
	 * 
	 * @return selectUpdatable を戻します。
	 * @param selectColmunInfos
	 * @throws SQLException
	 */
	public boolean isSelectUpdatable(ArrayList<ColumnInfoResult> selectColmunInfos)
			throws SQLException {
		if (sqlStatementCatgoly == SqlStatementCatgoly.SELECT
				&& selectUpdatable == true) {
			selectUpdatable = isExistsPrimeKey(selectColmunInfos);
		}
		return selectUpdatable;
	}

	/**
	 * 検索結果カラム名リストに、対象テーブルの主キーが全て含まれるかチェック
	 * 
	 * @return
	 * @throws SQLException
	 */
	private boolean isExistsPrimeKey(ArrayList<ColumnInfoResult> selectColmunInfos) throws SQLException {

		// 主キー定義の取得
		ArrayList<String> primaryKeys = SqlExec.getPrimaryKeys(schemaTableName
				.getSchemaName(), schemaTableName.getTableName());
		if (primaryKeys.size() == 0) {
			// 主キー定義が無いので更新不可
			errorReasonMsg = "主キー定義がないので更新できません";
			System.out.println("（更新不可）主キー定義がない。" + "#"
					+ schemaTableName.getSchemaName() + "#"
					+ schemaTableName.getTableName() + "#");
			return false;
		}

		// 主キー定義カラムの全てが、検索結果カラムに存在するかチェックする
		for (String string : primaryKeys) {
			if (!isExistsColumnInfo(string,selectColmunInfos)) {
				// 存在しないカラムがあるので更新不可
				errorReasonMsg = "全ての主キーが検索されていないので更新できません";
				System.out.println("（更新不可）全ての主キーが検索されていない。");
				return false;
			}
		}
		return true;

	}

	/**
	 * 検索結果カラム名リストに、指定カラムが存在するか検索する。
	 * 
	 * @param primaryColumnName
	 * @return
	 */
	private boolean isExistsColumnInfo(String primaryColumnName,ArrayList<ColumnInfoResult> selectColmunInfos) {
		for (ColumnInfoResult colInfo : selectColmunInfos) {
			if (primaryColumnName.equals(colInfo.getColumnName())) {
				// 存在するのでColumnInfoの主キー定義にマークする
				colInfo.setPrimaryKey(true);
				return true;
			}
		}
		// 存在しない
		return false;
	}

}
