package jdbcacsess.sql;

import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.Driver;
import java.sql.ResultSet;
import java.sql.SQLData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.TreeMap;
import java.util.Map.Entry;

import jdbcacsess.JdbcacsessException;
import jdbcacsess.LogOutput;
import jdbcacsess.sql.DBObjectProcedures.DBObjectProcedure;

/**
 * <p>
 * タイトル: JDBCアクセス処理
 * </p>
 * <p>
 * 説明: 各種SQLコマンドを実行する。staticとして全て定義されているのでインスタンス生成不要。
 * </p>
 * <p>
 * よって、複数種類の接続・検索結果レコードセットを保持することは出来ない。
 * </p>
 * <p>
 * 著作権: Copyright (c) 2002
 * </p>
 * <p>
 * 会社名:
 * </p>
 * 
 * @author 未入力
 * @version 1.0
 */
public class SqlExec {
	private static Connection _con;

	private static String _userId;

	private static String _passWord;

	private static String _jdbcUrl;

	private static boolean lower;

	private static boolean upper;

	private static Driver driver;

	private static String _driverClassName;

	private static URLClassLoader loader;
	/**
	 * 接続状態の判定
	 * 
	 * @return
	 */
	public static boolean isConnect() {
		if (_con == null) {
			return false;
		}
		try {
			if (_con.isClosed()) {
				return false;
			} else {
				return true;
			}
		} catch (SQLException e) {
			e.printStackTrace();
			return false;
		}
	}

	public static Class<?> classForName(String className) throws JdbcacsessException{
		Class<?> rtn = null;
		try{
			if (loader==null){
				rtn =  Class.forName(className);
			}else{
				rtn = Class.forName(className, true, loader);
			}
		} catch (NoClassDefFoundError e){
			throw new JdbcacsessException(e);
		} catch (Exception e) {
			throw new JdbcacsessException(e);
		}
		return rtn;
	}
	/**
	 * ＤＢ接続を行う
	 * 
	 * @param userId
	 * @param passWord
	 * @param jdbcUrl
	 * @param driverClassName
	 * @throws SQLException
	 * @throws ClassNotFoundException
	 * @throws MalformedURLException
	 * @throws IllegalAccessException
	 * @throws InstantiationException
	 * @throws JdbcacsessException 
	 */
	public static void connect(String userId, String passWord, String jdbcUrl,
			String driverClassName, final URL[] driverUrlPaths) throws 
			 JdbcacsessException {

		_userId = userId;
		_passWord = passWord;
		_jdbcUrl = jdbcUrl;
		_driverClassName = driverClassName;

		if (driverUrlPaths == null) {
			loader = null;
		} else {
			AccessController.doPrivileged(new PrivilegedAction<Object>() {
				 public Object run() {
						loader = new URLClassLoader(driverUrlPaths);
						return null;
				 }
			});

		}
		
		try {
			driver = (Driver) classForName(_driverClassName).newInstance();
		} catch (NoClassDefFoundError e){
			throw new JdbcacsessException(e);
		} catch (Exception e) {
			throw new JdbcacsessException(e);
		}

		disconnect();
		Properties info = new java.util.Properties();
		info.put("user", userId);
		info.put("password", passWord);

		try {
			_con = driver.connect(jdbcUrl, info);
			_con.setAutoCommit(true);

			lower = _con.getMetaData().storesLowerCaseIdentifiers();
			upper = _con.getMetaData().storesUpperCaseIdentifiers();
		} catch (Exception e) {
			throw new JdbcacsessException(e);
		}
		tableNamesMap.clear();
		columnInfoTableMap.clear();

		LogOutput.info("URL:" + jdbcUrl.toString());
		// Logger.global.info("接続ユーザ:" + info.toString());
		LogOutput.info("ドラバクラス名:" + driverClassName.toString());
		if (driverUrlPaths != null) {
			StringBuilder sb = new StringBuilder("ドラバライブラリパス:");
			for (int i = 0; i < driverUrlPaths.length; i++) {
				sb.append("[" + i + "]" + driverUrlPaths[i].toString());
			}
			LogOutput.info(sb.toString());
		}
		
		mapping(_con);
	}

	/**
	 * @return
	 */
	public static String getConnectString() {
		return _jdbcUrl + " [User=" + _userId + "]" + " " + _driverClassName;
	}

	/**
	 * ＤＢ切断を行う
	 */
	public static void disconnect() {
		if (isConnect()) {
			try {
				_con.close();
			} catch (SQLException e) {
				// TODO 自動生成された catch ブロック
				e.printStackTrace();
			}
		}
		_con = null;
	}

	/**
	 * @param con
	 */
	public static void disconnect(Connection con) {
		if (con != null) {
			try {
				con.close();
			} catch (SQLException e) {
				// TODO 自動生成された catch ブロック
				e.printStackTrace();
			}
		}
	}

	/**
	 * テーブルに定義されている主キーのカラム名を返却する
	 */
	public static ArrayList<String> getPrimaryKeys(String schemaName,
			String tableName) throws SQLException {

		TreeMap<Short, String> columnNamesMap = new TreeMap<Short, String>();
		DatabaseMetaData md = _con.getMetaData();
		ResultSet rs = md.getPrimaryKeys(null, schemaName, tableName);

		while (rs.next()) {
			columnNamesMap.put(rs.getShort(5), rs.getString(4));
		}
		rs.close();

		ArrayList<String> currentTablePrimkeys = new ArrayList<String>();
		for (Entry<Short, String> e : columnNamesMap.entrySet()) {
			currentTablePrimkeys.add(e.getValue());
		}
		return currentTablePrimkeys;
	}

	/**
	 * スキーマ一覧を返却します
	 * 
	 * @return スキーマ一覧
	 * @throws SQLException
	 */
	public static ArrayList<String> getSchemaNames() throws SQLException {
		ArrayList<String> schemaNames = new ArrayList<String>();
		DatabaseMetaData md = _con.getMetaData();
		ResultSet rs = md.getSchemas();
		// System.out.println("#getSchemaNames()#getColumnCount=" +
		// rs.getMetaData().getColumnCount());
		while (rs.next()) {
			String schema = rs.getString(1);
			schemaNames.add(schema);
		}
		rs.close();
		if (schemaNames.isEmpty()) {
			schemaNames.add(" ");
		}
		LogOutput.fine("_con.getMetaData().getSchemas():"
				+ schemaNames.toString());
		return schemaNames;
	}

	static Map<String, ArrayList<String>> tableNamesMap = new HashMap<String, ArrayList<String>>();

	static Map<String, ArrayList<ColumnInfoTable>> columnInfoTableMap = new HashMap<String, ArrayList<ColumnInfoTable>>();

	/**
	 * @param schema
	 * @return
	 * @throws SQLException
	 */
	private static ArrayList<String> getSimpleTableNames(String schema)
			throws SQLException {
		ArrayList<String> tableNames = tableNamesMap.get(schema);
		if (tableNames != null) {
			LogOutput.fine("schema:" + schema + " cache_tableNames:"
					+ tableNames.toString());
			return tableNames;
		}

		tableNames = new ArrayList<String>();
		DatabaseMetaData md = _con.getMetaData();
		String[] tableType = { "TABLE", "VIEW" };
		ResultSet rs = md.getTables(null, schema, null, tableType);
		while (rs.next()) {
			tableNames.add(rs.getString(3));
		}
		rs.close();
		tableNamesMap.put(schema, tableNames);
		LogOutput.fine("schema:" + schema
				+ " _con.getMetaData().getTables():" + tableNames.toString());
		return tableNames;
	}

	/**
	 * @param schema
	 * @return
	 * @throws SQLException
	 */
	public static ArrayList<SchemaTableName> getCompleteTableNames(String schema)
			throws SQLException {
		ArrayList<SchemaTableName> completeTableNames = new ArrayList<SchemaTableName>();
		for (String table : getSimpleTableNames(schema)) {
			completeTableNames.add(new SchemaTableName(schema, table));
		}
		LogOutput.fine("completeTableNames:" + completeTableNames);
		return completeTableNames;
	}

	/**
	 * @param schema
	 * @param tableName
	 * @return
	 * @throws SQLException
	 */
	public static ArrayList<ColumnInfoTable> getColumnMames(SchemaTableName schemaTable) throws SQLException {
		String schema = schemaTable.getSchemaName();
		String tableName = schemaTable.getTableName();
		ArrayList<ColumnInfoTable> columnInfoTable;
		columnInfoTable = columnInfoTableMap.get(schema + tableName);
		if (columnInfoTable != null) {
			return columnInfoTable;
		}

		columnInfoTable = new ArrayList<ColumnInfoTable>();
		
		ArrayList<String> primaryKeys = getPrimaryKeys(schema, tableName);

		ResultSet rs = _con.getMetaData().getColumns(null, schema, tableName, null);
		while (rs.next()) {
			ColumnInfoTable ci = new ColumnInfoTable(
					rs.getInt(17),
					rs.getString(4), 
					rs.getInt(5),
					rs.getString(6),
					rs.getInt(11),
					new SchemaTableName(schema, rs.getString(3)),
					rs.getInt(7), 
					rs.getInt(9) 
					);
			ci.setRemarks(rs.getString(12));
			// 主キーフラグをセット
			ci.setPrimaryKey(primaryKeys.contains(ci.getColumnName()));
			columnInfoTable.add(ci);
		}
		rs.close();

		Collections.sort(columnInfoTable,ColumnInfo.getComparatorByPosition());

		columnInfoTableMap.put(schema + tableName, columnInfoTable);
		return columnInfoTable;
	}

	/**
	 * テーブル１カラム分の定義情報
	 * 
	 * @param stcn
	 * @return
	 * @throws SQLException
	 */
	public static ColumnInfoTable getColumnInfoTable(SchemaTableColumnName stcn)
			throws SQLException {
		for (ColumnInfoTable cit : getColumnMames(stcn.getSchemaTableName())) {
			if (cit.getColumnName().equals(stcn.getColumnName())) {
				return cit;
			}
		}
		return null;
	}

	/**
	 * @param str
	 * @return
	 */
	public static String convertStoresIdentifiers(String str) {
		if (lower) {
			return str.toLowerCase();
		}
		if (upper) {
			return str.toUpperCase();
		}
		return str;

	}

	/**
	 * データベース製品の名称を取得
	 * 
	 * @return
	 * @throws SQLException
	 */
	public static String getDatabaseProductName() throws SQLException {
		return _con.getMetaData().getDatabaseProductName();
	}

	/**
	 * @return _con を戻します。
	 * @throws SQLException
	 * @throws JdbcacsessException 
	 */
	public static Connection getNewConnection() throws  JdbcacsessException {
		Connection con;
		Properties info = new java.util.Properties();
		info.put("user", _userId);
		info.put("password", _passWord);

		try {
			con = driver.connect(_jdbcUrl, info);
			con.setAutoCommit(_con.getAutoCommit());

			if (getDatabaseProductName().equals("Oracle")) {
				oracleSrtting(con);
			}
		} catch (Exception e) {
			throw new JdbcacsessException(e);
		}

		mapping(con);

		return con;
	}

	
	static String[] sqlDataList = {};
//	static String[] sqlDataList = {"mydata.TintXXy"};
	
	private static void mapping(Connection con) throws JdbcacsessException {
		if (sqlDataList.length==0){
			return;
		}
		try {
			java.util.Map<String, Class<?>> map = con.getTypeMap();
			for (String element : sqlDataList) {
				Class<?> c = classForName(element);
				String typeName = ((SQLData) c.newInstance()).getSQLTypeName();

				map.put(typeName, c);
			}
			con.setTypeMap(map);
		} catch (NoClassDefFoundError e){
			throw new JdbcacsessException(e);
		} catch (Exception e) {
			throw new JdbcacsessException(e);
		}
	}

	/**
	 * Oracle固有の設定処理を行う
	 * 
	 * @param con
	 * @throws SQLException
	 */
	private static void oracleSrtting(Connection con) throws SQLException {
		Statement stmt=null;
		try {
			stmt = con.createStatement();
			stmt.execute("ALTER SESSION SET NLS_DATE_FORMAT = 'YYYY-MM-DD HH24:MI:SS'");
			LogOutput.info("Oracleなので、NLS_DATE_FORMATを設定しました。"
					+ "select * from nls_session_parameters で確認して下さい ");

			stmt.execute("ALTER SESSION SET NLS_TIMESTAMP_FORMAT = 'YYYY-MM-DD HH24:MI:SS.FF9'");
			stmt.execute("ALTER SESSION SET NLS_TIMESTAMP_TZ_FORMAT = 'YYYY-MM-DD HH24:MI:SS.FF9 TZR'");
			LogOutput
					.info("Oracleなので、NLS_TIMESTAMP_FORMATとNLS_TIMESTAMP_TZ_FORMATも設定しました。"
							+ "select * from nls_session_parameters で確認して下さい ");
		} catch (SQLException e) {
			LogOutput.severe("TIMESTAMP_FORMATのalter sessionn 失敗", e);
		} finally {
			if (stmt != null) {
				try {
					stmt.close();
				} catch (SQLException e) {
				}
			}
		}
	}

	/**
	 * @return ドライバ情報 を戻します。
	 */
	public static String getDriverInfo() {
		String str = "";
		try {
			DatabaseMetaData dmd = SqlExec.getDatabaseMetaData();
			if (dmd!=null){
				str = dmd.getDriverVersion();
			}
		} catch (SQLException e) {
			if (driver != null) {
				str = "[" + driver.getClass().getName() + " Ver" + driver.getMajorVersion() + "." + driver.getMinorVersion() + "]";
			}
		}
		return str;
	}
	
	/**
	 * @return
	 * @throws SQLException
	 */
	public static DatabaseMetaData getDatabaseMetaData() throws SQLException{
		if (_con==null){
			return null;
		}
		return _con.getMetaData();
	}
	
	/**
	 * @param procedure
	 * @return
	 * @throws SQLException
	 */
	public static List<ColumnInfoParameter> getProcedureColumns(DBObjectProcedure procedure) throws SQLException{
		List<ColumnInfoParameter> rtn = new ArrayList<ColumnInfoParameter>();
		
		ResultSet rs = getDatabaseMetaData().getProcedureColumns(procedure.getProcedureCat(), procedure.getProcedureSchem(), procedure.getProcedureName(), null);
		int idx=0;
		
		while (rs.next()) {
			ColumnInfoParameter cip = new ColumnInfoParameter(
					++idx,
					rs.getString(4),
					rs.getInt(6),
					rs.getString(7),
					rs.getInt(12),
					"java.lang.String",
					rs.getInt(8),
					rs.getInt(10),
					rs.getInt(5)
					);
			if (rs.getMetaData().getColumnCount()>12){
				cip.setRemarks(rs.getString(13));
			}
			
			rtn.add(cip);
		}
		rs.close();
		return rtn;
	}
}
