/**
 * 
 */
package jdbcacsess.sql;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.Array;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.StringTokenizer;

import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import jdbcacsess.InfoListSetting;
import jdbcacsess.JdbcacsessException;
import jdbcacsess.gui.JFrameMain;
import jdbcacsess.sql.column.GetColumnArray;
import jdbcacsess.sql.column.GetColumnBlob;
import jdbcacsess.sql.column.GetColumnBytesOrBinaryStream;
import jdbcacsess.sql.column.GetColumnResultSet;

import org.w3c.dom.Element;

/**
 * @author sima
 * 
 */
public class SqlMappingInfo extends InfoListSetting<SqlMappingInfo> {

	public static final String BASETABNAME = "sqlmappinginfo";

	/**
	 * キーを編集する。キーの構成は、DBMS製品名とデータ型名を連結したもの。
	 * 
	 * @param dbms
	 * @param type
	 * @return 編集結果のキー値
	 */
	public static String editKey(String dbms, String type) {
		return dbms + "#$#" + type;
	}

	private String cellEditorClassName;

	private String cellRendererClassName;

	private String dbms;

	private Boolean enable = true;

	private String getResultSetClassName;

	private String resultSetMethodName;

	private String type = "";
	
	private ConstSqlTypes sqlTypes = ConstSqlTypes.UNUSE;

	
	public SqlMappingInfo() throws JdbcacsessException {
		Element e = topElement();
		if (e.getElementsByTagName(getBaseTagName()).getLength() == 0) {
			save("Oracle", "BINARY_DOUBLE", 	"getDouble", "" ,ConstSqlTypes.UNUSE);
			save("Oracle", "BINARY_FLOAT", 		"getFloat", "" ,ConstSqlTypes.UNUSE);
			save("Oracle", "DATE", 			"", "jdbcacsess.sql.column.GetColumnTimestampSS" ,ConstSqlTypes.UNUSE);
			save("Oracle", "TIMESTAMP", 		"getTimestamp", "" ,ConstSqlTypes.UNUSE);
			save("Oracle", "TIMESTAMPTZ", 		"getString", "" ,ConstSqlTypes.UNUSE);
			save("Oracle", "TIMESTAMPLTZ", 		"", "jdbcacsess.sql.column.GetColumnOracleTIMESTAMPLTZ" ,ConstSqlTypes.UNUSE);
			save("Oracle", "CLOB", 			"getString", "" ,ConstSqlTypes.UNUSE);
			save("Oracle", "INTERVALDS", 		"getString", "" ,ConstSqlTypes.UNUSE);
			save("Oracle", "INTERVALYM", 		"getString", "" ,ConstSqlTypes.UNUSE);
			save("Oracle", "ROWID", 			"getString", "" ,ConstSqlTypes.UNUSE);
			save("Oracle", "BFILE", 			"getString", "" ,ConstSqlTypes.UNUSE);

			save("MySQL", "BIT", 				"", "jdbcacsess.sql.column.GetColumnBytesOrBinaryStream" ,ConstSqlTypes.UNUSE);
			save("MySQL", "DATETIME", 			"", "jdbcacsess.sql.column.GetColumnTimestampSS" ,ConstSqlTypes.UNUSE);
			save("MySQL", "TIMESTAMP", 		"", "jdbcacsess.sql.column.GetColumnTimestampSS" ,ConstSqlTypes.UNUSE);
			save("MySQL", "YEAR", 			"", "jdbcacsess.sql.column.GetColumnDateYYYY" ,ConstSqlTypes.UNUSE);

			save("PostgreSQL", "interval", 		"", "org.postgresql.util.PGInterval", ConstSqlTypes.UNUSE);
			save("PostgreSQL", "date", 		"getString", "", ConstSqlTypes.DATE);
			save("PostgreSQL", "time", 		"getString", "", ConstSqlTypes.TIME);
			save("PostgreSQL", "timestamp", 	"getString", "", ConstSqlTypes.TIMESTAMP);
			
			save("PostgreSQL", "timetz", 		"getString", "", ConstSqlTypes.TIME);
			save("PostgreSQL", "timestamptz", 	"getString", "", ConstSqlTypes.TIMESTAMP);
			save("PostgreSQL", "box", 			"", "org.postgresql.geometric.PGbox", ConstSqlTypes.UNUSE);
			save("PostgreSQL", "circle", 		"", "org.postgresql.geometric.PGcircle", ConstSqlTypes.UNUSE);
			save("PostgreSQL", "lseg", 		"", "org.postgresql.geometric.PGlseg", ConstSqlTypes.UNUSE);
			save("PostgreSQL", "path", 		"", "org.postgresql.geometric.PGpath", ConstSqlTypes.UNUSE);
			save("PostgreSQL", "point", 		"", "org.postgresql.geometric.PGpoint", ConstSqlTypes.UNUSE);
			save("PostgreSQL", "polygon", 		"", "org.postgresql.geometric.PGpolygon", ConstSqlTypes.UNUSE);

			// 8.xのドライバでの更新はギブアップ version2を指定してもらう
			save("PostgreSQL", "bit", 			"getString", "", ConstSqlTypes.VARCHAR);
			save("PostgreSQL", "varbit", 		"getString", "", ConstSqlTypes.VARCHAR);
			save("PostgreSQL", "cidr", 		"getString", "", ConstSqlTypes.UNUSE);
			save("PostgreSQL", "inet", 		"getString", "", ConstSqlTypes.UNUSE);
			save("PostgreSQL", "macaddr", 		"getString", "", ConstSqlTypes.UNUSE);

			//LINUX!! どうしよう
			save("DB2/LINUX", "CLOB", 			"getString", "" ,ConstSqlTypes.UNUSE);
			save("DB2/LINUX", "DATE", 			"getDate", "" ,ConstSqlTypes.UNUSE);
			save("DB2/LINUX", "DBCLOB", 		"getString", "" ,ConstSqlTypes.UNUSE);
			save("DB2/LINUX", "TIME", 			"getTime", "" ,ConstSqlTypes.UNUSE);
			save("DB2/LINUX", "TIMESTAMP", 		"getTimestamp", "" ,ConstSqlTypes.UNUSE);
			save("DB2/LINUX", "XML", 			"getString", "" ,ConstSqlTypes.VARCHAR);
		}
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.InfoListSetting#addToElement(org.w3c.dom.Element)
	 */
	@Override
	public void addToElement(Element element) {
		element.setAttribute("dbms", getDbms());
		element.setAttribute("type", getType());
		element.setAttribute("getResultSetClassName",	getGetResultSetClassName());
		element.setAttribute("methodName", getResultSetMethodName());
		element.setAttribute("cellRendererClassName",	getCellRendererClassName());
		element.setAttribute("cellEditorClassName", getCellEditorClassName());
		element.setAttribute("enable", getEnable().toString());
		element.setAttribute("sqlTypes", getSqlTypes().toString());
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.InfoListSetting#getBaseTagName()
	 */
	@Override
	protected String getBaseTagName() {
		return BASETABNAME;
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.InfoListSetting#getInstance()
	 */
	@Override
	protected SqlMappingInfo getInstance() throws JdbcacsessException {
		return new SqlMappingInfo();
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.InfoListSetting#getKeyValue()
	 */
	public @Override
	String getKeyValue() {
		return editKey(getDbms(), getType());
	}

	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.InfoListSetting#setFromElement(org.w3c.dom.Element)
	 */
	@Override
	public void setFromElement(Element element) {
		setDbms(element.getAttribute("dbms"));
		setType(element.getAttribute("type"));
		setGetResultSetClassName(element.getAttribute("getResultSetClassName"));
		setResultSetMethodName(element.getAttribute("methodName"));
		setCellRendererClassName(element.getAttribute("cellRendererClassName"));
		setCellEditorClassName(element.getAttribute("cellEditorClassName"));
		setEnable(Boolean.valueOf(element.getAttribute("enable")));
		setSqlTypes(ConstSqlTypes.searchName(element.getAttribute("sqlTypes")));
	}
	
	/*
	 * (非 Javadoc)
	 * 
	 * @see jdbcacsess.InfoListSetting#setKeyValue(java.lang.String)
	 */
	@Override
	protected void setKeyValue(String key) {
		StringTokenizer st = new StringTokenizer(key, "#$#");
		if (st.hasMoreTokens()) {
			setDbms(st.nextToken());
			if (st.hasMoreTokens()) {
				setType(st.nextToken());
			}
		}
	}

	/**
	 * @return cellEditorClassName を戻します。
	 */
	public String getCellEditorClassName() {
		return cellEditorClassName;
	}

	/**
	 * @return cellRendererClassName を戻します。
	 */
	public String getCellRendererClassName() {
		return cellRendererClassName;
	}

	/**
	 * @return dbms を戻します。
	 */
	public String getDbms() {
		return dbms;
	}

	/**
	 * @return enable を戻します。
	 */
	public Boolean getEnable() {
		return enable;
	}

	/**
	 * @return getResultSetClassName を戻します。
	 */
	public String getGetResultSetClassName() {
		return getResultSetClassName;
	}

	
	/**
	 * @return
	 * @throws SecurityException
	 * @throws NoSuchMethodException
	 */
	public Method getResultSetMethod() throws SecurityException,
			NoSuchMethodException {
		if (getResultSetMethodName().equals("")) {
			return null;
		}
		Method rtn = ResultSet.class.getMethod(getResultSetMethodName(),
				int.class);
		return rtn;
	}

	/**
	 * @return reSultSetMethod を戻します。
	 */
	public String getResultSetMethodName() {
		return resultSetMethodName;
	}

	/**
	 * @return type を戻します。
	 */
	public String getType() {
		return type;
	}

	
	
	
	private void save(String dbms, String type, String method, String cls, ConstSqlTypes sqlTypes) throws JdbcacsessException {
		setDbms(dbms);
		setType(type);
		setResultSetMethodName(method);
		setGetResultSetClassName(cls);
		setSqlTypes(sqlTypes);
		
		currentDataSave();
	}

	@SuppressWarnings("unchecked")
	private Object instanceForClassname(String className) throws ClassNotFoundException, SecurityException, NoSuchMethodException, IllegalArgumentException, InstantiationException, IllegalAccessException, InvocationTargetException, JdbcacsessException{
		Class c = SqlExec.classForName(className);
		return c.getConstructor().newInstance();
	}
	
	public void search( ColumnInfoResult colInfoResult, boolean binaryDatalimit ) 
		throws SecurityException, ClassNotFoundException, NoSuchMethodException, IllegalArgumentException, InstantiationException, IllegalAccessException, InvocationTargetException, SQLException, JdbcacsessException{

		int binaryDataSize = 0;
		if (binaryDatalimit){
			binaryDataSize = JFrameMain.binaryDataSize;
		}

		SqlMappingInfo smi = getInfoMap().get(editKey(SqlExec.getDatabaseProductName(), colInfoResult.columnTypeName));
		
		if (smi != null && smi.getEnable()) {
			
			// SQL型マッピングに存在した時
			if (! smi.getResultSetClassName.equals("")) {
				// java.sql.Resultset のgetterは使用しない
				colInfoResult.methodOfResultSet = null;
				// カラムデータオブジェクトとResultSetからgetter の両方を実装したクラス名を設定
				colInfoResult.setColumnClassName(smi.getResultSetClassName);
				if (GetColumnResultSet.class.isAssignableFrom(colInfoResult.getColumnClass())){
					// カラムデータオブジェクトとResultSetからgetter の両方を実装したクラス名のオブジェクトを設定
					colInfoResult.getColumnResultSet = (GetColumnResultSet)colInfoResult.getColumnClass().newInstance();
				}
				
			} else {
				
				// java.sql.Resultset のgetterを使用する
				colInfoResult.methodOfResultSet = smi.getResultSetMethod();
				// java.sql.Resultset のgetterの返り値のクラスを設定する
				colInfoResult.setColumnClass(smi.getResultSetMethod().getReturnType());
			}
			
			// JDBCに従う時以外は、設定情報をセットする
			if (smi.getSqlTypes()!=ConstSqlTypes.JDBC){
				colInfoResult.setSqlMappingSqlType(smi.getSqlTypes());
			}
			
			//指定されていたら、セルレンダラーとセルエディターを設定
			if (!smi.getCellRendererClassName().equals("")) {
				colInfoResult.setTableCellRenderer((TableCellRenderer) instanceForClassname(smi.getCellRendererClassName()));
			}
			if (!smi.getCellEditorClassName().equals("")) {
				colInfoResult.setTableCellEditor((TableCellEditor) instanceForClassname(smi.getCellEditorClassName()));
			}
			
			colInfoResult.setExistsSqlMapping(true);

		} else {
			
			// SQL型マッピングに存在しない時
			if (isBinaryClassName( colInfoResult.getColumnClassName())
					|| colInfoResult.getSqlType().getCategoly() == ConstSqlTypes.Categoly.BIN) {
				if (colInfoResult.getSqlType() == ConstSqlTypes.BLOB) {
					colInfoResult.setColumnClass(GetColumnBlob.class);
					colInfoResult.getColumnResultSet = new GetColumnBlob(binaryDataSize);
				} else {
					colInfoResult.setColumnClass(GetColumnBytesOrBinaryStream.class);
					colInfoResult.getColumnResultSet = new GetColumnBytesOrBinaryStream(binaryDataSize);
				}
			}else{
				// 配列データ型のは、自前クラスで取得
				if (isArrayClassName( colInfoResult.getColumnClassName())
					||	Array.class.isAssignableFrom(colInfoResult.getColumnClass())) {
					//colInfoResult.setTableCellRenderer(new CellRendererSqlTableArray());
					colInfoResult.setColumnClass(GetColumnArray.class);
					colInfoResult.getColumnResultSet = new GetColumnArray();
				}
			}

			colInfoResult.setExistsSqlMapping(false);
		}
		
	}

	/**
	 * JAVAデータ型を判断して、バイナリ系データ型であるかを返却します
	 * 
	 * @param className
	 *            クラス名文字列
	 * @return バイナリデータ型の場合はtrue
	 */
	private  boolean isBinaryClassName(String className){
		if (className.equals("byte[]") || className.equals("[B")) {
			return true;
		}
		return false;
		
	}
	
	/**
	 * JAVAデータ型を判断して、配列型であるかを返却します
	 * 
	 * @param className
	 *            クラス名文字列
	 * @return 配列型の場合はtrue
	 */
	private  boolean isArrayClassName(String className){
		
		if (className.indexOf("[") == -1) {
			return false;
		}
		return true;
		
	}
	
	/**
	 * @param cellEditorClassName
	 *            設定する cellEditorClassName。
	 */
	public void setCellEditorClassName(String cellEditorClass) {
		this.cellEditorClassName = cellEditorClass;
	}

	/**
	 * @param cellRendererClassName
	 *            設定する cellRendererClassName。
	 */
	public void setCellRendererClassName(String cellRendererClass) {
		this.cellRendererClassName = cellRendererClass;
	}

	/**
	 * @param dbms
	 *            設定する dbms。
	 */
	public void setDbms(String dbms) {
		this.dbms = dbms;
	}

	/**
	 * @param enable
	 *            設定する enable。
	 */
	public void setEnable(Boolean enable) {
		this.enable = enable;
	}

	/**
	 * @param getResultSetClassName
	 *            設定する getResultSetClassName。
	 */
	public void setGetResultSetClassName(String getResultSetClass) {
		this.getResultSetClassName = getResultSetClass;
	}

	/**
	 * @param resultSetMethodName
	 *            設定する reSultSetMethod。
	 */
	public void setResultSetMethodName(String method) {
		this.resultSetMethodName = method;
	}

	/**
	 * @param type
	 *            設定する type。
	 */
	public void setType(String type) {
		this.type = type;
	}

	public ConstSqlTypes getSqlTypes() {
		return sqlTypes;
	}

	public void setSqlTypes(ConstSqlTypes sqlTypes) {
		this.sqlTypes = sqlTypes;
	}

}
