/**
 * 
 */
package jdbcacsess.sql.column;

import java.io.IOException;
import java.io.InputStream;

/**
 * @author sima
 * 
 */
public class BinaryInputStream implements DenyEditInCell {

	private InputStream inputStream;

	private int size;

	public BinaryInputStream(InputStream inputStream, int size) {
		this.inputStream = inputStream;
		this.size = size;
	}

	/**
	 * @return inputStream を戻します。
	 */
	public InputStream getInputStream() {
		return inputStream;
	}

	/**
	 * @return size を戻します。
	 */
	public int getSize() {
		return size;
	}

	/**
	 * １行分のバイト配列を読み込み返却します。
	 * read()で読み込むので、bufferdInputStreamが被さっていても、確実にROWBYTE分読み込みます（ハズ）。
	 * 最終行では、ROWBYTEより小さなバイト配列長になる事があります。
	 * 
	 * @return ファイル終端の時はnull、読み込めた場合はバイト配列
	 * @throws IOException
	 */
	static public byte[] readBytes(InputStream in, int size) throws IOException {
		int len = 0;
		byte[] rowBuf = new byte[size];

		for (int i = 0; i < rowBuf.length; i++) {
			len = in.read();
			if (len == -1) {
				if (i == 0) {
					return null;
				} else {
					byte[] rowBufTemp = new byte[i];
					System.arraycopy(rowBuf, 0, rowBufTemp, 0, i);
					return rowBufTemp;
				}
			}
			// InputStreamのreadは、0～255の範囲なので、byteにキャストは可能
			rowBuf[i] = (byte) len;
		}

		return rowBuf;
	}

}
